package gov.va.med.via.common.taglet;

import com.sun.tools.doclets.Taglet; 
import com.sun.javadoc.*;
import java.util.Map;

/**
 * A Taglet representing @mdws. This tag can be used in any kind of
 * {@link com.sun.javadoc.Doc}. It is not an inline tag. The text is displayed
 * to indicate how a MDWS param is mapped in the VIA application. For example,
 * "@mdws accessionNumber queryBean.criteria" would be shown as:
 * <DL>
 * <DT>
 * <B>Legacy MDWS Parameter mapping(s) to VIA:</B>
 * <DD>
 * <table cellpadding=2 cellspacing=0>
 * <tr>
 * <td><b>- <code>accessionNumber</code></b> maps to <b>
 * <code>queryBean.criteria</b></code></td>
 * </tr>
 * </table>
 * </DD>
 * </DL>
 * 
 * @author Jamie Ho
 * @since 1.4
 */

public class MdwsParamTaglet implements Taglet {

    private static final String NAME = "mdws";
    private static final String HEADER = "Legacy MDWS Parameter mapping(s) to VIA:";

    /**
     * Return the name of this custom tag.
     */
    public String getName() {
        return NAME;
    }

    /**
     * Will return true since <code>@mdws.param</code>
     * cannot be used in field documentation.
     * @return false since <code>@mdws.param</code>
     * cannot be used in field documentation.
     */
    public boolean inField() {
        return false;
    }

    /**
     * Will return true since <code>@mdws.param</code>
     * cannot be used in constructor documentation.
     * @return false since <code>@mdws.param</code>
     * cannot be used in constructor documentation.
     */
    public boolean inConstructor() {
        return true;
    }

    /**
     * Will return true since <code>@mdws.param</code>
     * can be used in method documentation.
     * @return true since <code>@mdws.param</code>
     * can be used in method documentation.
     */
    public boolean inMethod() {
        return true;
    }

    /**
     * Will return false since <code>@mdws.param</code>
     * can be used in method documentation.
     * @return false since <code>@mdws.param</code>
     * cannot be used in overview documentation and false
     * otherwise.
     */
    public boolean inOverview() {
        return false;
    }

    /**
     * Will return true since <code>@mdws.param</code>
     * can be used in package documentation.
     * @return true since <code>@mdws.param</code>
     * can be used in package documentation and false
     * otherwise.
     */
    public boolean inPackage() {
        return false;
    }

    /**
     * Will return true since <code>@mdws.param</code>
     * can be used in type documentation (classes or interfaces).
     * @return true since <code>@mdws.param</code>
     * can be used in type documentation and false
     * otherwise.
     */
    public boolean inType() {
        return true;
    }

    /**
     * Will return false since <code>@mdws.param</code>
     * is not an inline tag.
     * @return false since <code>@mdws.param</code>
     * is not an inline tag.
     */

    public boolean isInlineTag() {
        return false;
    }

    /**
     * Register this Taglet.
     * @param tagletMap  the map to register this tag to.
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
	public static void register(Map tagletMap) {
       MdwsParamTaglet tag = new MdwsParamTaglet();
       Taglet t = (Taglet) tagletMap.get(tag.getName());
       if (t != null) {
           tagletMap.remove(tag.getName());
       }
       tagletMap.put(tag.getName(), tag);
    }

    private String toHtmlLine(String text)
    {
    	if ( text==null || text.trim().equals("")) return "&nbsp;";
    	
    	String[] parts = text.split("\\s");    
    	
    	for (int i = 0 ; i < parts.length; i++  )
    	{
    		String part = parts[i];
    		if ( part.toLowerCase().startsWith("querybean") || part.toLowerCase().indexOf("querybean")!=-1)
    		{
    			parts[i] = "<code>"+part+"</code>";
    		}    		
   	    }
    	if ( !parts[0].startsWith("<code>"))
    	{
    		parts[0] = "<code>"+parts[0]+"</code>";
    	}
    	
    	if ( parts.length==1)
    	{
    		return parts[0];
    	}
    	else if ( parts.length==2)
    	{
    		return parts[0] + " maps to " + parts[1];
    	}
    	else if ( parts.length==3)
    	{
    		return parts[0] + " "+parts[1]+ " " + parts[2];
    	}
    	else if ( parts.length==4)
    	{
    		if ( text.toLowerCase().indexOf("maps to")==-1)
    		{
    			return parts[0]+ " maps to " + parts[1]+" - "+ parts[2] + " " + parts[3];
    		}
    		else
    		{
    			return parts[0] + " maps to " + parts[3];
    		}
    	}
    	else
    	{
    		text = parts[0]+ " maps to " + parts[1]+" -";
    		for ( int i = 2; i < parts.length; i++)
    		{
    			text+=" "+parts[i]; 
    		}    		
    		return text;
    	}

    }
    
    /**
     * Given the <code>Tag</code> representation of this custom
     * tag, return its string representation.
     * @param tag   the <code>Tag</code> representation of this custom tag.
     */
    public String toString(Tag tag) 
    {
    	String text = tag.text();
    	text = toHtmlLine(text);
    	
    	return "<p><DT><B>" + HEADER + "</B><DD>"
               + "<table cellpadding=2 cellspacing=0><tr><td\">"
               + "- "+text
               + "</td></tr></table></DD>\n";
    }

    /**
     * Given an array of <code>Tag</code>s representing this custom
     * tag, return its string representation.
     * @param tags  the array of <code>Tag</code>s representing of this custom tag.
     */
    public String toString(Tag[] tags) {
        if (tags.length == 0) {
            return null;
        }
        String result = "<p><DT><B>" + HEADER + "</B><DD>";
        result += "<table cellpadding=2 cellspacing=0><tr><td>";
        for (int i = 0; i < tags.length; i++) {
            if (i > 0) 
            {
                result+= "<br/>";
            }
            result += "- "+ toHtmlLine(tags[i].text());
        }
        return result + "</td></tr></table></DD>\n";
    }
    
}
