package gov.va.med.via.common.validator;

import gov.va.med.via.util.StringUtil;

public class RuleExpressionParser 
{
	/// make sure all markers are case-insensitive
	public static final String SEPARATOR_MARKER = "separator=".toLowerCase();
	public static final String NAME_MARKER = "name=".toLowerCase();
	public static final String TARGET_MARKER="target=".toLowerCase();
	public static final String TARGET_TYPE_MARKER = "targetType=".toLowerCase();
	public static final String VALUE1_MARKER="value1=".toLowerCase();
	public static final String VALUE2_MARKER = "value2=".toLowerCase();
	public static final String NULL_ALLOWED_MARKER="nullAllowed".toLowerCase();
	public static final String JAVADOC_MARKER = "javadoc".toLowerCase();

	public RuleExpressionParser() {
	}
	
	/** parses the 'expression' attribute of a Rule so that the other attributes can be populated without overly-verbose xml.
	 *  <p>
	 *  The first part of the expression can be the optional separator value.  By default, a semicolon is used to separate the expression 'parts'
	 *  from each other.  For example, a '||' separated string can be:<br/><b>separator=||patient.id=notempty||targetType=String</b>
	 *  <p>
	 *  Example: the expression 'patient.id=notempty;targetType=String' would parse as follows:
	 *  <ul>
	 *  <li>the rule's target attribute is <b>patient.id</b></li>
	 *  <li>the rule's name attribute is <b>notempty</b></li>
	 *  <li>the rule's targetType attribute is <b>String</b></li> 
	 *  </ul>   
	 * 
	 * @param rule the rule with an expression to parse
	 * @throws IllegalArgumentException if the rule expression is not valid
	 */
	public void parseRuleExpression(Rule rule)
	{
		if ( rule.expression==null || rule.expression.trim().equals("")) return;
		
		checkForDefinedSeparator(rule);
		
		String[] parts = StringUtil.split(rule.expression,rule.getSeparator());
		
		for ( int i =0; i < parts.length; i++)
		{
			String part=parts[i];
			
			if (part==null || part.trim().equals(""))
			{
				continue;
			}
			validatePart(part);
			
			if ( part.toLowerCase().startsWith(TARGET_TYPE_MARKER))
			{
				rule.targetType = part.substring(TARGET_TYPE_MARKER.length());
				validateTargetType(rule.targetType);
				continue;
			}
			if ( part.toLowerCase().startsWith(VALUE1_MARKER))
			{
				rule.value1 = part.substring(VALUE1_MARKER.length());
				continue;
			}
			if ( part.toLowerCase().startsWith(VALUE2_MARKER))
			{
				rule.value2 = part.substring(VALUE2_MARKER.length());
				continue;
			}
			if ( part.toLowerCase().startsWith(NULL_ALLOWED_MARKER))
			{
				try
				{
					String sbool = part.substring(NULL_ALLOWED_MARKER.length());
					rule.nullAllowed = Boolean.parseBoolean(sbool);
					continue;
				}
				catch ( Exception ignore)
				{			
					continue;
				}				
			}
			if ( part.toLowerCase().startsWith(JAVADOC_MARKER))
			{
				rule.setJavadoc(part.substring(JAVADOC_MARKER.length()));
				continue;				
			}
			if ( partContainsRule(part))
			{
				rule.target=part.substring(0,part.indexOf('='));
				rule.name=part.substring(part.indexOf('=')+1);
				continue;
			}
			
			
			
			String message = "The expression part '" + part + "' could not be parsed.";
			message+=" The expression 'parts' supported are: separator=<chars>,<target>=<rulename>,targetType=,value1=,value2=,javadoc=,nullAllowed=";
			throw new IllegalArgumentException(message);			
		}
		
	}
	
	private static void validatePart( String part)
	{
		int equalsIndex=part.indexOf("=");
		if ( equalsIndex ==-1) throw new IllegalArgumentException("The expression part '" + part + "' contains no '=' character");
		if ( equalsIndex==0) throw new IllegalArgumentException("The expression part '" + part + "' cannot start with '=' character");
		if ( equalsIndex== (part.length()-1)) throw new IllegalArgumentException("The expression part '" + part + "' cannot end with '=' character");
	}
	
	private static void validateTargetType( String targetType)
	{
		if ( targetType== null || targetType.compareToIgnoreCase("String")==0){// By default target is treated as String
			return;
		}
		if (targetType.compareToIgnoreCase("Date")==0)
		{
			return;
		}
		if (targetType.compareToIgnoreCase("int")==0)
		{
			return;
		}
		if (targetType.compareToIgnoreCase("Object")==0)
		{
			return;
		}
		if (targetType.compareToIgnoreCase("StringArray")==0)
		{
			return;
		}
		throw new IllegalArgumentException("The targetType '"+targetType+" is not a valid targetType. Supported targetType values are: String,Date,int,Object,StringArray");
		
	}
	
	private static boolean partContainsRule(String part )
	{
		String value = part.substring(part.indexOf("=")+1);
		for ( RuleNames ruleName: RuleNames.values())
		{
			if ( value.equals(ruleName.name()))
			{
				return true;
			}			
		}
		return false;
	}
	
	
	private void checkForDefinedSeparator(Rule rule) 
	{
		/// the first part of the expression may be a separator character;  by default the separator is the semicolon, but can be changed 
		/// if the developer wants to specify a different one  
		String separator = ";";
		char charSeparator=';';
		if ( !rule.expression.startsWith(SEPARATOR_MARKER)) return;
		
		/// the first part of the expression is like separator=||, so determine the string separator
		int separatorIndex = SEPARATOR_MARKER.length();
		charSeparator=rule.expression.charAt(separatorIndex);
		separator=""+charSeparator;
		/// allow a multiple occurrence of same character separator like &&  or || or <<<
		for ( int i=1; i < rule.expression.length(); i++ )
		{
			char nextChar = rule.expression.charAt(separatorIndex+i);
			if ( nextChar == charSeparator)
			{
				separator+=nextChar;
			}
			else
			{
				break;  /// the character is not the same as the separator character
			}
		}
		/// trim the rule expression to eliminate the separator expression component
		rule.expression = rule.expression.substring(separatorIndex+separator.length());
		rule.setSeparator(separator);
	}
}
