package gov.va.med.via.hl7.mpi;

import gov.va.med.via.rpc.site.DataSource;
import gov.va.med.via.util.StringUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.Socket;
import java.nio.charset.Charset;


public class MpiConnection 
{

	public static final int DEFAULT_PORT = 10000;
	public static final int DEFAULT_TIMEOUT = 60000;
	public static final String END_MESSAGE = "\u001B\u001B\u001B";

	public static final Charset characterSet = Charset.forName("US-ASCII");
	private static final boolean DEBUG = false;
	
	String hostname;
	int port;
	int timeout;
	Socket socket;

	private DataSource dataSource;
	private boolean connected;



	public MpiConnection() 
	{
	}

	public MpiConnection(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public void connect()
	{
		setConnected(false);
		if ( dataSource==null )
		{
			throw new RuntimeException("Datasource is null");
		}
		if  ( !dataSource.getProtocol().equals("HL7"))
		{
			throw new RuntimeException("Incorrect Protocol: '" + dataSource.getProtocol() + "' should be 'HL7'.");
		}
		if  ( !dataSource.getModality().equals("MPI"))
		{
			throw new RuntimeException("Incorrect Modality: '" + dataSource.getModality() + "' should be 'MPI'.");
		}

		hostname= dataSource.getProvider();
		if ( StringUtil.isEmpty(hostname))
		{
			throw new RuntimeException("No provider (hostname)" );
		}
		port = dataSource.getPort();
		if ( port==0)
		{
			port = DEFAULT_PORT;
		}
		timeout = DEFAULT_TIMEOUT;

		try
		{
			socket = new Socket(hostname,port);
			socket.setSoTimeout(timeout);
			setConnected(true);
		}
		catch ( Exception e)
		{
			setConnected(false);
			throw new RuntimeException(e);			
		}

	}
	
	public String query( String request) throws IOException
	{
        /// for diagnostics only
		if ( DEBUG)
		{
			String repeatableResponse = loadRepeatableResponse();
			if ( repeatableResponse!=null ) return repeatableResponse;
		}
		
		
		connect();
		String msg = "HELO " + getDataSource().getSiteId().getKey() + "\r\n";  /// check value against MDWS
		String reply = sendReceive( msg,"\r\n");
		if ( !reply.startsWith("220"))
		{
			disconnect();
			throw new RuntimeException("Error sending HELO: " + reply);
		}
		
        String datamsg = "DATA PARAM=MPI\r\n";
        String hl7msg = "";
        String[] segments = StringUtil.split(request, "\\r");
        segments = StringUtil.trimArray(segments);
        
        /** start fix */
        int vtqIndex = request.indexOf("VTQ");
        int rdfIndex = request.indexOf("RDF");
        String mshPart = request.substring(0,vtqIndex);
        String vtqPart = request.substring(vtqIndex,rdfIndex);
        String rdfPart = request.substring(rdfIndex);
        segments = new String[3];
        segments[0]=mshPart.trim();
        segments[1]=vtqPart.trim();
        segments[2]=rdfPart.trim();
        /** end fix */
        
        for (int i = 0; i < segments.length; i++)
        {
            segments[i] += '\r';    //Gotta put the terminator back after splitting on it
            hl7msg += StringUtil.strPack(segments[i], 3);
        }
        hl7msg += StringUtil.strPack(END_MESSAGE, 3);

        send(datamsg);
        reply = sendReceive(hl7msg, "\r\n");
        if (!reply.startsWith("220"))
        {
            disconnect();
            throw new RuntimeException("ERROR sending DATA PARAM=MPI: " + reply);
        }
        msg = "TURN\r\n";
        reply = sendReceive(msg, "\r\n");
        if (!reply.startsWith("220"))
        {
            disconnect();
            throw new RuntimeException("ERROR sending HL7: " + reply);
        }
        reply = receive(END_MESSAGE);
        msg = "QUIT\r\n";
        send(msg);
        disconnect();
        
        
       
        return reply;

		
		
		
	}
	
	private String loadRepeatableResponse() 
	{
		try {
			String filename = "c:\\test\\mpi\\_response.txt";
			if ( !new File(filename).exists()) return null;
			FileInputStream fis =new FileInputStream(filename);
			byte[] bytes = new byte[fis.available()];
			fis.read(bytes);
			fis.close();
			String s = new String(bytes);
			return s;
		} 
		catch (Exception e) 
		{
			return null;
		}
	}

	String sendReceive(String request, String terminateString) throws IOException
    {
        send(request);
        return receive(terminateString);
    }

	protected void send(String msg) throws IOException
    {
		
		byte[] bytes = msg.getBytes(characterSet); //Encoding.ASCII.GetBytes(msg);
        socket.getOutputStream().write(bytes, 0, bytes.length);
    }
	
	protected String receive( String terminateString) throws IOException
	{
        byte[] bytesReceived = new byte[256];
        StringBuilder reply = new StringBuilder("");
        int endIndex = -1;
        do
        {
        	socket.getInputStream().read(bytesReceived, 0, bytesReceived.length);
        	String thisBatch = new String(bytesReceived,characterSet);
        	endIndex = thisBatch.indexOf(terminateString);
        	if ( endIndex!=-1)
        	{
        		thisBatch = thisBatch.substring(0,endIndex);
        	}
        	reply.append(thisBatch);
        }
        while (endIndex == -1);

        return reply.toString();
	}
	
	
	
	
	public void disconnect()
	{
		if (!isConnected())
		{
			setConnected(false);
			return;
		}

		try 
		{
			socket.close();
		} 
		catch (IOException ignored) 
		{
		}
		setConnected(false);
	}





	public DataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public boolean isConnected() {
		return connected;
	}

	public void setConnected(boolean connected) {
		this.connected = connected;
	}





}
