package gov.va.med.via.hl7.mpi;

import gov.va.med.via.util.StringUtil;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class PatientMatchesRequest 
{
	private static SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

	EncodingCharacters encodingChars;
    MshSegment msh;
    VtqSegment vtq;
    RdfSegment rdf;
    
	public PatientMatchesRequest() {
		setMsh(new MshSegment());
		setVtq(new VtqSegment());
		setRdf(new RdfSegment());	
	}

    public String toMessage()
    {
        String result = getMsh().toSegment() + getVtq().toSegment() + getRdf().toSegment();
        
        /// fix message so it is identical to mdws
        result= result.replaceFirst("\\\\", "\\\\\\\\");
        
        int vtqIndex = result.indexOf("VTQ");
        int rdfIndex = result.indexOf("RDF");
        String mshPart = result.substring(0,vtqIndex);
        String vtqPart = result.substring(vtqIndex,rdfIndex);
        String rdfPart = result.substring(rdfIndex);
        
        result = mshPart.trim()+"\\" + "r" + vtqPart.trim()+"\\"+"r" + rdfPart.trim();
        return result;
    }
    public String extractHL7Message(String response)
    {
        if (!response.startsWith("DATA PARAM=MPI"))
        {
        	throw new RuntimeException("Unexpected response: MPI data missing 'DATA PARAM=MPI' header" );
        }
        String[] parts = StringUtil.split(response, StringUtil.CRLF);
        String msg = parts[1];

        String mshSeg = getSegment(msg, "MSH");
        String msaSeg = getSegment(msg, "MSA");
        String qakSeg = getSegment(msg, "QAK");
        String rdfSeg = getSegment(msg, "RDF");
        rdfSeg = rdfSeg.substring(0, rdfSeg.length() - 1);    // Lose the \r at the end

        String result = mshSeg + '\r' + msaSeg + '\r' + qakSeg + '\r' + rdfSeg;

        int startIndex = msg.indexOf("RDT");
        while (startIndex != -1)
        {
            String rdtSeg = getRdt(msg, startIndex);
            result += '\r' + rdtSeg;
            startIndex = msg.indexOf("RDT", startIndex + 3);
        }
        return result;
    }
    public String getRdt(String response, int startIndex)
    {
        int index = response.indexOf("RDT", startIndex);
        if (index == -1)
        {
            return "";
        }
        int rdtSectionLength = Integer.parseInt( response.substring(index-3,index));
        try
        {
        	String rdtSection = null;
        	String remainingChunk = response.substring(index);
        	
        	if ( rdtSectionLength>remainingChunk.length())
        	{
        		/// the section length in the return string is wrong - so send back the remaining chunk        		
        		return remainingChunk; 
        	}
        	rdtSection = response.substring(index, index + rdtSectionLength);
        	/// let's check the section for additional occurrences of the ###RDT| header
        	
        	String rdtRegex = "[0-9]{3}RDT\\|";   /// this regex looks for 3 numbers, followed by RDT|
        	
        	String[] rdtSectionsWithinSection = rdtSection.split(rdtRegex);
        	if ( rdtSectionsWithinSection.length>1)
        	{
        		/// try to return just the first section
        		return rdtSectionsWithinSection[0];
        	}
        	
        	return response.substring(index, index+rdtSectionLength);
        }
        catch (StringIndexOutOfBoundsException e)
        {
        	/// sometimes the last RDT segment may have a preceding value like 000125 - even though only a few character are left, 
        	/// so when this happens, then we'll try to send the RDT segment back with what is left
        	return "";/// response.substring(index); -- just ignore the segment
        }
    }
    
    
    public String getSegment(String response, String hdr)
    {
        int index = response.indexOf(hdr);
        if (index == -1)
        {
            return "";
        }
        
        //int cnt = Convert.ToInt16(response.Substring(index - 3, 3));
        int cnt = Short.parseShort( response.substring(index-3,index));  // the three prior characters should be a count
        return response.substring(index, index+cnt);  /// the three prior characters tell the length of the segment 
    }
    
    public String buildMatchSsnRequest( String ssn)
    {
        PatientMatchesRequest hl7 = getBlankHl7Request();
        hl7.getVtq().setQueryName( MpiConstants.VQQ_QUERY_NAME_FUZZY);
        hl7.getVtq().setSelectionCriteria( new ArrayList<SelectionCriterion>(2));
        hl7.getVtq().getSelectionCriteria().add(
            new SelectionCriterion(MpiConstants.FLD_SSN.getFieldName(), "EQ", ssn, "OR"));
        hl7.getVtq().getSelectionCriteria().add(
            new SelectionCriterion(MpiConstants.FLD_LASTNAME.getFieldName(), "EQ", "", ""));
        return hl7.toMessage();
    }
    
    
    public PatientMatchesRequest getBlankHl7Request()
    {
        PatientMatchesRequest request = new PatientMatchesRequest();

        request.getMsh().setSendingApplication( MpiConstants.VQQ_SENDING_APP);
        request.getMsh().setSendingFacility( MpiConstants.VQQ_SENDING_FACILITY);
        request.getMsh().setReceivingApplication( MpiConstants.VQQ_RECEIVING_APP);
        request.getMsh().setReceivingFacility( MpiConstants.VQQ_RECEIVING_FACILITY);
        request.getMsh().setMessageCode( MpiConstants.VQQ_MSG_CODE);
        request.getMsh().setEventTrigger( MpiConstants.VQQ_TRIGGER);
        request.getMsh().setMessageControlId( sdf.format(new Date()));// DateTime.Now.ToString("yyyyMMddHHmmss"));
        request.getMsh().setProcessingId( MpiConstants.VQQ_PROCESSING_ID);
        request.getMsh().setVersionId( MpiConstants.VQQ_VERSION_ID);
        request.getMsh().setAcceptAckType( MpiConstants.VQQ_ACCEPT_ACK_TYPE);
        request.getMsh().setApplicationAckType(MpiConstants.VQQ_APP_ACK_TYPE);
        request.getMsh().setCountryCode(MpiConstants.VQQ_COUNTRY_CODE);

        request.getVtq().setQueryTag( MpiConstants.VQQ_QUERY_TAG);
        request.getVtq().setFormatCode(MpiConstants.VQQ_FORMAT_CODE);
        request.getVtq().setVirtualTableName(MpiConstants.VQQ_VIRTUAL_TABLE);

        request.getRdf().setNcols( 12);
        request.getRdf().setColumns( new RdfColumn[12]);
        request.getRdf().getColumns()[0] = new RdfColumn(MpiConstants.FLD_ICN);
        request.getRdf().getColumns()[1] = new RdfColumn(MpiConstants.FLD_SSN);
        request.getRdf().getColumns()[2] = new RdfColumn(MpiConstants.FLD_LASTNAME);
        request.getRdf().getColumns()[3] = new RdfColumn(MpiConstants.FLD_FIRSTNAME);
        request.getRdf().getColumns()[4] = new RdfColumn(MpiConstants.FLD_MIDDLENAME);
        request.getRdf().getColumns()[5] = new RdfColumn(MpiConstants.FLD_NAME_SUFFIX);
        request.getRdf().getColumns()[6] = new RdfColumn(MpiConstants.FLD_SSN);
        request.getRdf().getColumns()[7] = new RdfColumn(MpiConstants.FLD_DOB);
        request.getRdf().getColumns()[8] = new RdfColumn(MpiConstants.FLD_SEX);
        request.getRdf().getColumns()[9] = new RdfColumn(MpiConstants.FLD_DECEASED_DATE);
        request.getRdf().getColumns()[10] = new RdfColumn(MpiConstants.FLD_CMOR);
        request.getRdf().getColumns()[11] = new RdfColumn(MpiConstants.FLD_SITES);

        return request;
    }

	
	public EncodingCharacters getEncodingChars() {
		return encodingChars;
	}

	public void setEncodingChars(EncodingCharacters encodingChars) {
		this.encodingChars = encodingChars;
	}

	public MshSegment getMsh() {
		return msh;
	}

	public void setMsh(MshSegment msh) {
		this.msh = msh;
	}

	public VtqSegment getVtq() {
		return vtq;
	}

	public void setVtq(VtqSegment vtq) {
		this.vtq = vtq;
	}

	public RdfSegment getRdf() {
		return rdf;
	}

	public void setRdf(RdfSegment rdf) {
		this.rdf = rdf;
	}
	
	
	
}
