package gov.va.med.via.hl7.mpi;

import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.util.StringUtil;

import java.util.ArrayList;
import java.util.List;

public class PatientMatchesResponse {

	private EncodingCharacters encodingChars;
	private MshSegment msh;
	private MsaSegment msa;
	private QakSegment qak;
	private RdfSegment rdf;
	private RdtSegment[] rdt;
	
	public PatientMatchesResponse(String rawMessage)
    {
        parse(rawMessage);
    }
	
    public Patient getPatient(int rdtIdx)
    {
        if (rdt == null)
        {
            return null;
        }
        if (!"OK".equals(qak.getQueryResponseStatus()))
        {
            return null;
        }
        if (rdtIdx > rdt.length)
        {
            throw new IndexOutOfBoundsException("Only " + rdt.length + " RDTs");
        }
        Patient result = new Patient();

        RdtColumn column = rdt[rdtIdx].getColumn(MpiConstants.LASTNAME_FLDNAME);
        String patname = column.getValues()[0];
        column = rdt[rdtIdx].getColumn(MpiConstants.FIRSTNAME_FLDNAME);
        if (column.getValues() != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            patname += ',' + column.getValues()[0];
        }
        column = rdt[rdtIdx].getColumn(MpiConstants.MIDDLENAME_FLDNAME);
        if (column.getValues() != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            patname += ' ' + column.getValues()[0];
        }
        column = rdt[rdtIdx].getColumn(MpiConstants.NAMESUFFIX_FLDNAME);
        if (column.getValues() != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            patname += ' ' + column.getValues()[0];
        }
        result.setName(patname);

        column = rdt[rdtIdx].getColumn(MpiConstants.SSN_FLDNAME);
        result.setSsn(column.getValues()[0]);

        column = rdt[rdtIdx].getColumn(MpiConstants.ICN_FLDNAME);
        String[] parts = StringUtil.split(column.getValues()[0], 'V');
        result.setMpiPid( parts[0]);
        result.setMpiChecksum( parts[1]);

        column = rdt[rdtIdx].getColumn(MpiConstants.DOB_FLDNAME);
        if (column.getValues() != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            result.setDob(column.getValues()[0]);
        }

        column = rdt[rdtIdx].getColumn(MpiConstants.SEX_FLDNAME);
        if (column != null && column.getValues() != null)
        {
            result.setGender(column.getValues()[0]);
        }

        column = rdt[rdtIdx].getColumn(MpiConstants.DECEASEDDATE_FLDNAME);
        if (column.getValues() != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            result.setDeceasedDate(column.getValues()[0]);
        }

        column = rdt[rdtIdx].getColumn(MpiConstants.SITES_FLDNAME);
        if (column != null && column.getValues() != null && column.getValues().length > 0)
        {
            ArrayList<SiteId> lst = new ArrayList<SiteId>(column.getValues().length);
            for (int i = 0; i < column.getValues().length; i++)
            {
                parts = StringUtil.split(column.getValues()[i], StringUtil.CARET);
                if (parts[0].length() == 3 && StringUtil.isNumeric(parts[0]) && !"003".equals(parts[0]))
                {
                    SiteId s = new SiteId(parts[0],"","","");
                    if (parts.length > 2)
                    {
                        s = new SiteId(parts[0], "", parts[1], parts[2]);
                    }
                    lst.add(s);
                }
            }
            result.setSiteIDs(lst); 
        }

        column = rdt[rdtIdx].getColumn(MpiConstants.CMOR_FLDNAME);
        if (column != null && !StringUtil.isEmpty(column.getValues()[0]))
        {
            result.setCmorSiteId(column.getValues()[0]);
        }

        return result;
    }

    public Patient[] getPatients()
    {
        if (rdt == null || rdt.length == 0)
        {
            return null;
        }
        Patient[] result = new Patient[rdt.length];
        for (int i = 0; i < rdt.length; i++)
        {
            result[i] = getPatient(i);
        }
        return result;
    }

	
	public Patient[] getPatientsForSSN(String ssn)
    {
        if (rdt == null || rdt.length == 0)
        {
            return null;
        }
        ArrayList<Patient> lst = new ArrayList<Patient>(rdt.length);
        for (int i = 0; i < rdt.length; i++)
        {
            Patient p = getPatient(i);
            if (p.getSsn().equals(ssn))
            {
                lst.add(p);
            }
        }
        return lst.toArray(new Patient[lst.size()]);
    }
	
	
	
	private void parse(String rawMessage)
    {
        String[] segments = StringUtil.split(rawMessage, '\r');
        if (segments.length < 4)
        {
            throw new RuntimeException("Invalid message: needs 5 segments");
        }
        
        /// these next few lines fix the situation where empty line segments are being returned 
        /// the empty lines are removed and the segment[] only has lines with data
        List<String> filledSegments = new ArrayList<String>();
        for ( String segment: segments)
        {
        	if ( segment ==null || segment.trim().isEmpty()) continue;
        	filledSegments.add(segment);
        }
        segments = filledSegments.toArray(new String[filledSegments.size()]);
        
        
        msh = new MshSegment(segments[0]);
        msa = new MsaSegment(segments[1]);
        qak = new QakSegment(segments[2]);
        rdf = new RdfSegment(segments[3]);
        if (segments.length > 4)
        {
        	List<RdtSegment> rdtSegments = new ArrayList<RdtSegment>();
            for (int segIdx = 4; segIdx < segments.length; segIdx++)
            {
            	try
            	{
            		RdtSegment rdtSegment = new RdtSegment(rdf, segments[segIdx]);
            		rdtSegments.add(rdtSegment);  /// should only get this far if rdtSegment parsing succeeds
            		//	rdt[rdtIdx] = new RdtSegment(rdf, segments[segIdx]);
            	}
            	catch ( Exception ignore)
            	{
            		/// ignore incomplete RdtSegments
            	}
            }
        
            rdt =  rdtSegments.toArray(new RdtSegment[rdtSegments.size()]);
        }
        
    }
	
	
    public String extractHL7Message(String response)
    {
        if (!response.startsWith("DATA PARAM=MPI"))
        {
            throw new RuntimeException("Unexpected response: " + response);
        }
        String[] parts = StringUtil.split(response, StringUtil.CRLF);
        String msg = parts[1];

        String mshSeg = getSegment(msg, "MSH");
        String msaSeg = getSegment(msg, "MSA");
        String qakSeg = getSegment(msg, "QAK");
        String rdfSeg = getSegment(msg, "RDF");
        rdfSeg = rdfSeg.substring(0, rdfSeg.length() - 1);    // Lose the \r at the end

        String result = mshSeg + '\r' + msaSeg + '\r' + qakSeg + '\r' + rdfSeg;

        int startIndex = msg.indexOf("RDT");
        while (startIndex != -1)
        {
            String rdtSeg = getRdt(msg, startIndex);
            result += '\r' + rdtSeg;
            startIndex = msg.indexOf("RDT", startIndex + 3);
        }
        return result;
    }

    
    public String getSegment(String response, String header)
    {
        int index = response.indexOf(header);
        if (index == -1)
        {
            return "";
        }
        
        //int cnt = Convert.ToInt16(response.Substring(index - 3, 3));
        int cnt = Short.parseShort( response.substring(index-3,index));  // the three prior characters should be a count
        return response.substring(index, index+cnt);  /// the three prior characters tell the length of the segment 
    }
  
	
    public String getRdt(String response, int startIndex)
    {
        int index = response.indexOf("RDT", startIndex);
        if (index == -1)
        {
            return "";
        }
        //int cnt = Convert.ToInt16(response.Substring(index - 3, 3));
        int cnt = Short.parseShort( response.substring(index-3,index));
        try
        {
        	return response.substring(index, index+cnt);
        }
        catch (StringIndexOutOfBoundsException e)
        {
        	/// sometimes the last RDT segment may have a preceding value like 000125 - even though only a few character are left, 
        	/// so when this happens, then we'll try to send the RDT segment back with what is left
        	return "";/// response.substring(index); -- just ignore the segment
        }
    }

    
	
	public EncodingCharacters getEncodingChars() {
		return encodingChars;
	}
	public void setEncodingChars(EncodingCharacters encodingChars) {
		this.encodingChars = encodingChars;
	}
	public MshSegment getMsh() {
		return msh;
	}
	public void setMsh(MshSegment msh) {
		this.msh = msh;
	}
	public MsaSegment getMsa() {
		return msa;
	}
	public void setMsa(MsaSegment msa) {
		this.msa = msa;
	}
	public QakSegment getQak() {
		return qak;
	}
	public void setQak(QakSegment qak) {
		this.qak = qak;
	}
	public RdfSegment getRdf() {
		return rdf;
	}
	public void setRdf(RdfSegment rdf) {
		this.rdf = rdf;
	}
	public RdtSegment[] getRdt() {
		return rdt;
	}
	public void setRdt(RdtSegment[] rdt) {
		this.rdt = rdt;
	}
	
	
	
}
