package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.exception.ErrorCodeEnum;
import gov.va.med.via.exception.RpcException;
import gov.va.med.via.response.DataBeanParser;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.ConnectionSpecManager;
import gov.va.med.via.vistalink.VistaLinkTemplate;
import gov.va.med.via.vistalink.command.VistaLinkAppProxyCommandImpl;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.via.vistalink.command.VistaLinkDuzCommandImpl;
import gov.va.med.via.vistalink.exception.MissingDuzException;

import java.util.List;
import java.util.Map;

/**
 * Abstract Base class for all Dao types that manage access to information areas/domains within the VIA application
 * 
 * @author ZZZZZZTALBOM
 *
 */
public abstract class BaseDao implements Dao {
	
	protected ConnectionSpecManager connectionSpecManager;
	protected Map<String,DataBeanParser> dataBeanParsers;
	protected Map<String,Dao> daoMap;

	/**
	 *	Method sets the ConnectionSpecManager - used for configuration of the instance class
	 *
	 * @param connectionSpecManager
	 */
	public void setConnectionSpecManager(ConnectionSpecManager connectionSpecManager) {
		this.connectionSpecManager = connectionSpecManager;
	}

	
	/**
	 * Factory method to create a core VistaLinkTemplate class with the connectionFactorLocator instance.
	 * 
	 * @param connectionFactoryLocator
	 * @return a VistaLinkTemplate constructed/configured with a connectionFactoryLocator
	 */
	protected VistaLinkTemplate createVistaLinkTemplate(ConnectionFactoryLocator connectionFactoryLocator) {
		return new VistaLinkTemplate(connectionFactoryLocator);
	}

	/**
	 * Locates the user DUZ, which must be present. If it is not the supplied Provider has not logged in and been authenticated.
	 * 
	 * @param provider
	 * @return a String value of the provider's DUZ
	 * @throws MissingDuzException
	 */
	protected String getUserDuz(Provider provider) throws MissingDuzException{
		String duz = null;
		{
			duz = provider.getUserId();
			if(duz == null){
				throw new MissingDuzException(ErrorCodeEnum.MISSING_DUZ_EXCEPTION);
			}
		
		}
		
		return duz;	
	}

	/**
	 * Locates the local DFN for the current site - if it is not present then an RPC/Vista call must be made to obtain the local IEN (DFN) for the patient for the current siteId.
	 * 
	 * @param patient
	 * @param currentSiteId
	 * @param vistaLinkCommand 
	 * @return String value of the patients local IEN (DFN) for the currentSiteId
	 */
	protected String getLocalDfn(Patient patient, String currentSiteId, VistaLinkCommand vistaLinkCommand) {
	
		String dfn = null;
		
		//do not need a dfn - must be getting a record by IEN (itemId) - patient was not required
		if(patient == null) return dfn;
		
		if(!currentSiteId.equals(patient.getLocalSiteId())){
			if(patient.getMpiPid()!=null){
				
				dfn = populateLocalPid(patient.getMpiPid(), vistaLinkCommand);//this is the method that needs to be created that will call the RPC etc...
			}
			else{
				//NOTE: VIA should have validated for an mpiPid/icn upfront in webservices, but there may be some missing validation...this would indicate missing validation in webservices
				throw new RpcException(ErrorCodeEnum.DAO_EXCEPTION_MISSING_ICN, currentSiteId);
			}
			
			if(dfn == null){
				//NOTE: the remote site may not know or have a localPid/DFN for the provided ICN - we must abort processing...
				throw new RpcException(ErrorCodeEnum.DAO_EXCEPTION_UNFOUND_DFN, currentSiteId, patient.getMpiPid());
			}
		}
		else{
			
			dfn = patient.getLocalPid();
		}
		
		if(dfn == null){
			//NOTE: this should never happen - we have validated well before this point in the application
			throw new RpcException(ErrorCodeEnum.DAO_EXCEPTION_MISSING_DFN);
		}
		
		return dfn;
	}

	protected String populateLocalPid(String icn, VistaLinkCommand vistaLinkCommand) {
		
       return vistaLinkCommand.executeRpc(RpcNameConstants.VAFCTFU_CONVERT_ICN_TO_DFN, icn);
    
	}


	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#getDaoName()
	 */
	@Override
	public abstract String getDaoName();

	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#getDataBeanParsers()
	 */
	@Override
	public Map<String, DataBeanParser> getDataBeanParsers() {
		return dataBeanParsers;
	}

	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.Dao#setDataBeanParsers(java.util.Map)
	 */
	@Override
	public void setDataBeanParsers(Map<String, DataBeanParser> dataBeanParsers) {
		this.dataBeanParsers = dataBeanParsers;
	}

	/**
	 * Returns a DataBeanParser from a map which has been stored by a key which represents the Service call access by client applications - the parser is configured to return the data content expected for each service method.
	 * 
	 * @param webMethodName
	 * @return DataBeanParser stored against the service method name
	 */
	protected DataBeanParser getDataBeanParser(String webMethodName) {
		// TODO Auto-generated method stub
		return this.getDataBeanParsers().get(webMethodName);
	}

	/**
	 * Method parses an RPC response based upon the parser stored/configured for a service method name.
	 * 
	 * @param response
	 * @param queryBean
	 * @param dfn
	 * @param siteId
	 * @param webMethodName
	 * @param vistaLinkCommand
	 * @return List<DataBean> parsed data bean content of the RPC response
	 */
	protected List<DataBean> parseResponseToDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand) {
		DataBeanParser dataBeanParser = getDataBeanParser(webMethodName);
		return dataBeanParser.populateDataBeans(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand);
	}
	
	

	/**
	 * Determines the type (DUZ of APP PROXY) of connectionSpec and creates a Command object for that type that will delegate calls of that type to the VistaLinkTemplate 
	 * 
	 * @param connectionFactoryLocator
	 * @param targetSiteCode
	 * @param provider
	 * @param rpcContext
	 * @return VistaLinkCommand object for the type of ConnectionSpec
	 */
	protected VistaLinkCommand createVistaLinkCommand(ConnectionFactoryLocator connectionFactoryLocator, String targetSiteCode, Provider provider, String rpcContext) {
		//check to see if this should be a DUZ connection or App Proxy connection - OR something more involved - depending upon Business need - this might need
		//to be delegated to another component for authorization - BSE or some other authorization - this logic may be too simple
		int connectionSpecType = connectionSpecManager.getConnectionSpecType(provider, targetSiteCode);
		
		VistaLinkTemplate vistaLinkTemplate = new VistaLinkTemplate(connectionFactoryLocator);
		
		VistaLinkCommand vistaLinkCommand = null;
		
		if(connectionSpecType == VistaLinkTemplate.USER_TYPE_DUZ){
			String duz = getUserDuz(provider);
			vistaLinkCommand = new VistaLinkDuzCommandImpl(targetSiteCode, rpcContext, duz,vistaLinkTemplate);
		}
		else if(connectionSpecType == VistaLinkTemplate.USER_TYPE_APPLICATION_PROXY){
			vistaLinkCommand = new VistaLinkAppProxyCommandImpl(targetSiteCode, rpcContext, APP_PROXY_NAME, vistaLinkTemplate);
		}
		return vistaLinkCommand;
	}
	
	public void setDaoMap(Map<String, Dao> daoMap) {
		this.daoMap = daoMap;
	}
	
	protected Dao getDao(String daoNameKey)
	{
		return this.daoMap.get(daoNameKey);
	}
	
}
