package gov.va.med.via.dao;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaRequestMapKeyConstants;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RPCUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.util.DateTimeIteratorUtil;
import gov.va.med.via.util.DateTimeUtil;
import gov.va.med.via.vistalink.ConnectionFactoryLocator;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.joda.time.DateTime;

public class InpatientMoveDao extends BaseDao 
{				
	/**
	 * Retrieves the patient moves from Vista for an in-patient
	 *   
	 * Patient Movement Fields in Vista (405)
     *  0:  IEN
     *  1:  DATE/TIME                       .01     Timestamp
     *  2:  TRANSACTION                     .02     Transaction
     *  3:  PATIENT                         .03     Patient.LocalPid
     *  4:  TYPE OF MOVEMENT                .04     MovementType
     *  5:  WARD LOCATION                   .06     AssignedLocation.Id
     *  6:  ROOM-BED                        .07     AssignedLocation.Room, AssignedLocation.Bed
     *  7:  FACILITY TREATING SPECIALTY     .09     Specialty, Service
     *  8:  ADMISSION/CHECK-IN MOVEMENT     .14     CheckInId
     *  9:  DISCHARGE/CHECK-OUT MOVEMENT    .17     CheckOutId
     * 10:  MAS MOVEMENT TYPE               .18     MasMovementType
     * 11:  ATTENDING PHYSICIAN             .19     Attending.Uid
     * 12:  RELATED PHYSICAL MOVEMENT       .24     RelatedPhysicalMovementId
     * 13:  LENGTH OF STAY                  201     LengthOfStay
     * 14:  PASS DAYS                       202     PassDays
     * 15:  DAYS ABSENT                     203     DaysAbsent
     * 16:  DIAGNOSIS [SHORT]               .1      Diagnosis
     * 17:  PTF ENTRY                       .16     PatientTxId
     * 18:  ADMITTED FOR SC CONDITION?      .11     AdmittedForScCondition
     * 19:  SCHEDULED ADMISSION?            .25     ScheduledAdmission
     * 20:  ^VA(200,                        (0,1)   Admitting.Name
     *
	 */
	@Override
	public List<DataBean> callRpc( String currentSiteId, Map<String,Object> requestParameterMap, ConnectionFactoryLocator connectionFactoryLocator) 
	{			
		QueryBean queryBean = (QueryBean)requestParameterMap.get(ViaRequestMapKeyConstants.QUERY_BEAN_KEY); 
		String webMethodName = (String)requestParameterMap.get(ViaRequestMapKeyConstants.WEB_METHOD_NAME_KEY);			
		
		VistaLinkCommand vistaLinkCommand = createVistaLinkCommand(connectionFactoryLocator, currentSiteId, queryBean.getProvider(), VistaConstants.CPRS_CONTEXT);

		String result = "";
		boolean executeSingleRPCCall = true;
		String dfn = getLocalDfn(queryBean.getPatient(), currentSiteId, vistaLinkCommand);
		
		List<Object> params =new ArrayList<Object>();		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("FILE",405);
		hashMap.put("FIELDS",".01;.02;.03;.04;.06;.07;.09;.14;.17;.18;.19;.24;201;202;203;.1;.16;.11;.25");
		hashMap.put("FLAGS","IP");
		hashMap.put("ID","S DZ=$P(^(0),U,19) I DZ'=\"\",($D(^VA(200,DZ,0))#10)'=0 D EN^DDIOL($P($G(^VA(200,DZ,0)),U,1))");
		
		if (ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVES.compareTo(webMethodName) == 0)
		{			
			hashMap.put("FROM",RPCUtils.adjustForNumericSearch(dfn));
			hashMap.put("PART",dfn);
			hashMap.put("XREF","C");
		} 
		else if (ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVESBYCHECKINID.compareTo(webMethodName) == 0)
		{
			String checkInId = queryBean.getItemId();	
			hashMap.put("FROM",RPCUtils.adjustForNumericSearch(checkInId));
			hashMap.put("PART",checkInId);
			hashMap.put("XREF","CA");
		}
		else if (ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVESBYDATERANGE.compareTo(webMethodName) == 0)
		{
			String startDate = buildFromDate(queryBean.getStartDate());	
			String endDate = queryBean.getEndDate();
			hashMap.put("FROM", Timestamp.fromUtcString(startDate));
			hashMap.put("XREF","B");
			hashMap.put("SCREEN", RPCUtils.buildFromendDateScreenParam(queryBean.getStartDate(), endDate, 0, 1));
		}
		else if (ViaWebMethodNameConstants.NUMISVC_GETINPATIENTMOVESBYDATETIMERANGE.compareTo(webMethodName) == 0)
		{
			hashMap.put("XREF","B");		
			executeSingleRPCCall = false; // This call needs to loop to get the Inpatient Moves for all the date ranges by interval
			String iterLength = queryBean.getCriteria();
			
			DateTimeIteratorUtil iteratingDate = new DateTimeIteratorUtil(
				 DateUtils.IsoDateStringendDateTime(queryBean.getStartDate()), DateUtils.IsoDateStringendDateTime(queryBean.getEndDate()), iterLength);
				
	        while (!iteratingDate.IsDone())
	        {
	             iteratingDate.SetIterEndDate();		// Set the end date for this iteration

	             if (hashMap.containsKey("FROM") && hashMap.containsKey("PART"))	// Need to remove these keys from hashMap if exists for the next iteration
	             {
	            	 hashMap.remove("FROM");
	            	 hashMap.remove("PART");
	             }
	             
	             hashMap.put("FROM", iteratingDate.GetSCListerPart());
	             hashMap.put("PART", iteratingDate.GetSCListerPart());	             
	             params.add(hashMap); // add hashMap to parameters List to invoke RPC call		
	 			 String iterResult = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
	             if (iterResult != null)
	             {
	            	 iterResult = RPCStringUtils.replaceEndofLineCharacters(iterResult.trim());        
	                 String[] lines = RPCStringUtils.split(iterResult, RPCStringUtils.CRLF);
	                 if (lines != null && lines.length > 1)	// Response contains [DATA] tag on the first line
	                 {
	                	 for (int i = 0; i < lines.length; i++)
	                     {
	                		 if (lines[i].contains("[Data]"))
		                		 continue;
	                		 
	                		 if (lines[i] != null || !lines[1].isEmpty())
	                			 result = result.concat(lines[i]);
	                     }        	 	                	 
	                 }
	             }
	             iteratingDate.AdvanceIterStartDate();		// Advance Date by adding the interval (iterLength) received
	             params.clear();							// Clear list of parameters for next iteration
	        }

		}
		else if (ViaWebMethodNameConstants.NUMISVC_GETSTAYMOVEMENTSBYDATERANGE.compareTo(webMethodName) == 0)
		{		
			
			hashMap.remove("ID");
			hashMap.remove("FIELDS");
			
			String startDate = buildFromDate(queryBean.getStartDate());	
			String endDate = queryBean.getEndDate();
			
			hashMap.put("FIELDS",".01;.14");
			hashMap.put("FROM", Timestamp.fromUtcString(startDate));
			hashMap.put("XREF","B");
			hashMap.put("SCREEN", RPCUtils.buildFromendDateScreenParam(queryBean.getStartDate(), endDate, 0, 1));
			
		}
			
		if (executeSingleRPCCall)
		{
			params.add(hashMap); // add hashMap to parameters List to invoke RPC call		
			result = vistaLinkCommand.executeRpc(RpcNameConstants.SC_LISTER, params);
		}
 				
		return parseResponseToDataBeans(result, queryBean, dfn, currentSiteId, webMethodName, vistaLinkCommand);
	}

	
    
    /// <remarks>
    /// The aforementioned quirks include:
    /// 1. if there's no HHmmss component, it needs one
    /// 2. you can't end in 00 seconds, so increment +1 (? should it be -1 -- NEED TO ASK JOE)
    /// 3. starts after the value, so should be -1 (?)
    /// 
    /// So what do you do if you want to start at 1 second? subtracting 1 would get you to a time ending in 0,
    /// which we can't do...
    /// </remarks>
    /// <param name="fromDate"></param>
    /// <returns></returns>
    private String buildFromDate(String fromDate)
    {
        // put date in a format we can manipulate
        Calendar myFromDate = DateUtils.IsoDateStringendDateTime(fromDate);              
        
        DateTime myDate = new DateTime(myFromDate);
        if (myDate.getSecondOfDay() % 10 == 0)
        {
        	myDate = myDate.minus(-1);
        }
        
        String dateStr = myDate.toString(DateTimeUtil.FORMAT_DATE_TIME);
        
      //  DateTime myDate = new DateTime(myFromDate.getTime());
        
        // set the start time forward so that DDR LISTER catches the first desired time
        // TBD VAN: should be able to refactor
        myFromDate.add(Calendar.SECOND, -1);

        // if that sets it to end in a '0' then subtract another second...which could cause a problem.
        if (myFromDate.get(Calendar.SECOND) % 10 == 0)
        {
        	myFromDate.add(Calendar.SECOND, -1);
        }
        
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(DateTimeUtil.FORMAT_DATE_TIME);
        return simpleDateFormat.format(myFromDate.getTime()); 
    }
    
	/* (non-Javadoc)
	 * @see gov.va.med.via.dao.BaseDao#getDaoName()
	 */
	@Override
	public String getDaoName() 
	{
		return DaoNameConstants.INPATIENT_MOVE_DAO;
	}
}