package gov.va.med.via.dao.lookup;

import java.util.ArrayList;
import java.util.LinkedHashMap;

@SuppressWarnings("serial")
public class ReferenceDataCache extends LinkedHashMap<ReferenceDataCacheKey, ReferenceDataCacheValue> 
{
	
	public static final int ONE_DAY_IN_SECONDS = 86400;
	public static final int TWELVE_HOURS_IN_SECONDS = 43200;
	public static final int ONE_HOUR_IN_SECONDS = 3600;
	public static final int TWO_HOURS_IN_SECONDS = 7200;
	public static final int FOUR_HOURS_IN_SECONDS = 14400;
	public static final int DEFAULT_MAX_ENTRIES_PER_SITE = 2000;
	public static final long DEFAULT_MAX_ENTRY_AGE_IN_SECONDS = ONE_DAY_IN_SECONDS;
	
	
	private long cacheHitCount = 0;
	private long cacheMissCount = 0;
	private long maxEntries = DEFAULT_MAX_ENTRIES_PER_SITE;
	private long maxEntryAgeInSeconds = DEFAULT_MAX_ENTRY_AGE_IN_SECONDS;

	private static ReferenceDataCache instance;
	public static ReferenceDataCache getInstance()
	{
		if (instance==null)
		{
			instance = new ReferenceDataCache();
		}
		return instance;
	}
	private ReferenceDataCache() {
	}

	public synchronized void put(String fileNumber, String code, String result) {
		ReferenceDataCacheKey key = new ReferenceDataCacheKey(fileNumber, code);
		put(key, new ReferenceDataCacheValue(result));
		// / the idea is to remove the least fresh entries - so we put the most
		// fresh guy in and then remove
		// / this algorithm could be improved...
		if (size() > maxEntries) {
			removeElementsExceedingMaxSize();
		}
	}

	public synchronized String get(String fileNumber, String code) 
	{
		ReferenceDataCacheKey key = new ReferenceDataCacheKey(fileNumber,code);
		return getWithKey(key);
	}

	private synchronized String getWithKey(ReferenceDataCacheKey key) {
		ReferenceDataCacheValue cachedValue = super.get(key);

		if (cachedValue == null) {
			cacheMissCount++;
			return null;
		}
		if (cachedValue.exceedsMaxAge(maxEntryAgeInSeconds)) {
			cachedValue = null;
			remove(key); // / remove the entry from the cache since the data it
							// contains is too old (exceeds max age)
			cacheMissCount++;
			return null;
		}
		cacheHitCount++;

		// / these next two lines implement the LRU functionality of the cache -
		// a returned cached value also becomes the most recently used element
		// in the list
		remove(key);
		put(key, cachedValue); // / put the entry into the cache to up its
								// status in the LRU category

		return cachedValue.getCachedValue();
	}

	public long getCacheHitCount() {
		return cacheHitCount;
	}

	public long getCacheMissCount() {
		return cacheMissCount;
	}

	@Override
	protected synchronized boolean removeEldestEntry(java.util.Map.Entry<ReferenceDataCacheKey, ReferenceDataCacheValue> eldest) {
		return size() > maxEntries;
	}

	public synchronized long getMaxEntries() {
		return maxEntries;
	}

	public synchronized void setMaxEntries(long maxEntries) {
		this.maxEntries = maxEntries;
		if (maxEntries < size()) {
			removeElementsExceedingMaxSize();
		}
	}

	public synchronized long getMaxEntryAgeInSeconds() {
		return maxEntryAgeInSeconds;
	}

	public synchronized void setMaxEntryAgeInSeconds(long maxEntryAgeInSeconds) {
		this.maxEntryAgeInSeconds = maxEntryAgeInSeconds;
		removeEntriesExceedingMaxEntryAge();
	}

	private synchronized void removeElementsExceedingMaxSize() {
		int currentSize = size();
		int reductionNeeded = currentSize - (int) maxEntries;
		if (reductionNeeded <= 0)
			return;

		ArrayList<ReferenceDataCacheKey> keysToRemove = new ArrayList<ReferenceDataCacheKey>();
		int counter = 0;
		for (ReferenceDataCacheKey key : keySet()) {
			if (counter >= reductionNeeded)
				break;
			keysToRemove.add(key);
			counter++;
		}
		for (ReferenceDataCacheKey key : keysToRemove) {
			remove(key);
		}
	}

	public synchronized int removeEntriesExceedingMaxEntryAge() {
		int count = 0;

		ArrayList<ReferenceDataCacheKey> keysToRemove = new ArrayList<ReferenceDataCacheKey>();
		for (ReferenceDataCacheKey key : keySet()) {
			if (key == null)
				continue;
			ReferenceDataCacheValue value = get(key);
			if (value == null) {
				keysToRemove.add(key);
				continue;
			}
			if (value.exceedsMaxAge(maxEntryAgeInSeconds))
				keysToRemove.add(key);
		}

		for (ReferenceDataCacheKey key : keysToRemove) {
			remove(key);
			count++;
		}
		return count;
	}

}
