package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.Order;
import gov.va.med.via.databeans.OrderStatus;
import gov.va.med.via.databeans.OrderType;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * The Class AllOrdersParser.
 * 
 * @author DOMAINROMERJ
 */
public class AllOrdersParser extends BaseDataBeanParser
{
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toOrders(response, queryBean, dfn, siteId, webMethodName, vistaLinkCommand));
	}
	
	/**
	 * Convert the response into an array of Order containing the data received from Vista.
	 * Populates the order with the correct order type and translate the number received for 
	 * order status and signature status to a valid description.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param webMethodName the web method name
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return Array of Order objects. 
	 */
	protected Order[] toOrders(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{
		List<Order> responseArray = new ArrayList<Order>();
		Order order = new Order();
		
        if (response == null|| response.equals("")) 
        {
        	order.setFault(new Fault("Cannot retrieve orders using the information provided."));
        }
        else
        {
        	response = RPCStringUtils.replaceEndofLineCharacters(response);
        	String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        	HashMap<String, OrderType> orderTypes = getOrderTypes(vistaLinkCommand);

        	order = null;
        	for (int i= 0; i < lines.length; i++)
        	{
        		if (lines[i] == null || lines[i].equals("")) continue;
        		
        		if (lines[i].startsWith("~"))
        		{
        			if (order != null)
        			{
        				responseArray.add(order);
        			}
        			
        			order = new Order();        		
	        		String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);	        		
	        		if (fields == null || fields.length < 15) continue;
	        		
	        		order.setId(fields[0] != null? fields[0].substring(1): null);
	        		order.setTimestamp(!fields[2].equals("") ? Timestamp.toDateTime(fields[2]): null);
	        		order.setStartDate(!fields[3].equals("") ? Timestamp.toDateTime(fields[3]): null);
	        		order.setStopDate(!fields[4].equals("") ? Timestamp.toDateTime(fields[4]): null);
	        		order.setStatus( OrderStatus.getNameByValue(fields[5]) );
	        		order.setSigStatus( decodeSignatureStatus(fields[6]));
	        		order.setVerifyingNurse(fields[7]);
	        		order.setVerifyingClerk(fields[8]);
	        		
	        		Provider provider = new Provider();
	        		provider.setUserId(fields[9]);
	        		provider.setName(fields[10]);
	        		
	        		order.setProvider(provider);
	        		order.setFlag("1".equals(fields[12])? true : false);
	        		order.setChartReviewer(fields[14]);	        		
	        		order.setType(orderTypes.get(fields[1]));	        			
        		}
        		else if (lines[i].startsWith("t"))
        		{
        			if (order.getText() == null)
        			{
        				order.setText(lines[i].substring(1) + RPCStringUtils.CRLF);
        			}
        			else
        			{
        				order.setText( order.getText().concat(lines[i].substring(1) + RPCStringUtils.CRLF) );
        			}
        		}        			
        	}
        }        
        
        if (order != null) responseArray.add(order);
        
        return responseArray.toArray(new Order[0]);
	}
	
	/**
	 * Gets a list of all valid order types from the Vista Server
	 *
	 * @return A hash map containing the valid order types.
	 */
	private HashMap<String, OrderType> getOrderTypes(VistaLinkCommand vistaLinkCommand)
	{
		HashMap<String, OrderType> orderTypes = new HashMap<String, OrderType>();		
		
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWORDG_MAPSEQ);
		
		if (response == null || response.length() == 0)
		{
			return null;
		}
		
		response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
		for (int i = 0; i < lines.length; i++)
		{
			if (lines[i] == null || lines[i].equals("")) continue;
			
			// e.g. 3=65^Inpt. Meds^Inpt. Meds -> Id = 3, Name1 = Inpt. Meds, Name2 = Inpt. Meds
			String[] firstSplit = RPCStringUtils.split(lines[i], RPCStringUtils.EQUALS);			
			if (firstSplit  == null || firstSplit.length != 2) continue;
			
			String[] secondSplit = RPCStringUtils.split(firstSplit[1], RPCStringUtils.CARET);			
			if (secondSplit  == null || secondSplit.length != 3) continue;
			
			orderTypes.put(firstSplit[0], new OrderType(firstSplit[0], secondSplit[1], secondSplit[2]));
		}
        
		return orderTypes;
	}
	
	
	/**
	 * Decode signature status.
	 *
	 * @param statusCode the status code
	 * 
	 * @return the string
	 */
	private String decodeSignatureStatus(String statusCode)
    {
		Integer code;
		try
		{
			code = Integer.parseInt(statusCode);
		}
		catch ( NumberFormatException e)
		{
			return null;
		}
		
		switch(code)
		{
			case 0:
				return "ON CHART w/written orders";
			case 1:
				return "ELECTRONIC";
			case 2:
				return "NOT SIGNED";
			case 3:
				return "NOT REQUIRED";
			case 4:
				return "ON CHART w/printed orders";
			case 5:
				return "NOT REQUIRED due to cancel/lapse";
			case 6:
				return "SERVICE CORRECTION to signed order";
			case 7:
				return "DIGITALLY SIGNED";
			case 8:
				return "ON PARENT order";
			default:
		        return "Invalid Signature Status: " + statusCode;				
        }
    }
}
