package gov.va.med.via.response;

import gov.va.med.via.databeans.Allergy;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.ObservationType;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.databeans.Symptom;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Allergy DataBean parser type class - parses RPC responses and populates the content into Allergy DataBeans
 * 
 * @author ZZZZZZTALBOM
 *
 */
public class AllergyDataBeanParser extends BaseDataBeanParser {
	
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){
		
		return Arrays.asList((DataBean[])toAllergies(response, dfn, siteId, vistaLinkCommand));
	}
	

	/**
	 * Parses a response string and populates aa array of DataBean objects
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkCommand -VistaLinkCommand used to executer RPC calls thru VistaLink for additional data
	 * @return an array of Allergy DataBeans populated from the content stored in the RPC response string
	 */
	protected Allergy[] toAllergies(String response, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
        if (RPCStringUtils.isNullOrEmpty(response))
        {
            return new Allergy[0];
        }
        
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);
        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        List<Allergy> allgergyList = new ArrayList<Allergy>();
        Allergy rec = null;
        String comment = "";
        for (int i = 0; i < lines.length; i++)
        {
            if (lines[i].equals(""))
            {
                continue;
            }
            String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            short fldnum = Short.parseShort(flds[0]);
            switch (fldnum)
            {
                case 1:
                    if (rec != null)
                    {
                        rec.setComment(comment);
                        allgergyList.add(rec);
                    }
                    rec = new Allergy();
                    comment = "";
                    String[] parts = RPCStringUtils.split(flds[1], RPCStringUtils.SEMICOLON);
     
                    if (parts.length == 2)
                    {
                    	rec.setFacility( new SiteId(parts[1], parts[0]));
                    }
                    else if (!flds[1].equals(""))
                    {
                    	rec.setFacility( new SiteId(siteId, flds[1]));
                    }
                    else
                    {
                    	rec.setFacility( new SiteId(siteId));
                    }
                    break;
                case 2:
                    if (flds.length == 2)
                    {
                        rec.setAllergyName(flds[1]);
                    }
                    break;
                case 3:
                	if (flds.length == 2)
                    {
                        rec.setAllergyType(flds[1]);
                    }
                    break;
                case 4:
                	 if (flds.length == 2)
                    {
                        rec.setTimestamp(VistaTimestamp.toUtcFromRdv(flds[1]));
                    }
                    break;
                case 5:
                	if (flds.length == 2)
                    {
                        rec.setType(new ObservationType(null,"Allergies and Adverse Reactions", flds[1]));
                    }
                    break;
                case 6:
                    if (flds.length == 2)
                    {
                        comment += flds[1] + '\n';
                    }
                    break;
                case 7:
                    if (flds.length == 2)
                    {
                        rec.setAllergyId(flds[1]);
                    }
                    break;
            }

        }
        if (rec != null)
        {
        	allgergyList.add(rec);
        }
        
        //call for reactions here
        List<Allergy> reactionsList = getAllergyReactions(dfn, vistaLinkCommand);
        
        //the call to combine the list here
        List<Allergy> allgergyReactionsCombinedList = combineAllergiesWithReactions(allgergyList, reactionsList);
        
        //then return the list to array 
        return allgergyReactionsCombinedList.toArray(new Allergy[0]);
    }
	
	//ORQQAL LIST
    //Params ------------------------------------------------------------------
    //literal	100848
    //Results -----------------------------------------------------------------
    //973^NONSTEROIDAL ANTI-INFLAMMATORY^^ABDOMINAL PAIN
    //985^PENICILLIN^
	protected List<Allergy> getAllergyReactions(String dfn, VistaLinkCommand vistaLinkCommand){
	
		String result = vistaLinkCommand.executeRpc(RpcNameConstants.ORQQAL_LIST, dfn);
		
		return parseAllergiesFromReactions(result);
		
	}
	
	
	//ORQQAL LIST
    //Params ------------------------------------------------------------------
    //literal	100848
    //Results -----------------------------------------------------------------
    //973^NONSTEROIDAL ANTI-INFLAMMATORY^^ABDOMINAL PAIN
    //985^PENICILLIN^
	protected List<Allergy> parseAllergiesFromReactions(String response)
    {
		
		List<Allergy> allergyReactions = new ArrayList<Allergy>();
		if (RPCStringUtils.isNullOrEmpty(response))
        {
			return allergyReactions;
        }

        response = RPCStringUtils.replaceEndofLineCharacters(response);
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        Allergy rec = null;
        for (int i = 0; i < lines.length; i++)
        {
        	if (lines[i].equals(""))
            {
                continue;
            }
        	rec = new Allergy();
        	
            String[] flds = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
            if(flds.length >= 1 ) rec.setAllergyId(flds[0]);
           
            if(flds.length >= 2 ) rec.setAllergyName(flds[1]);	
            	
	        if (flds.length >= 3)
	        {
	            // what is in field 2??? it is always empty
	        }
	            
	        if (flds.length >= 4)
	        {
	            rec.addReaction(new Symptom(flds[3]));
	        }
	        
	        if(rec != null) allergyReactions.add(rec);
        }
        
        return allergyReactions;
    }
    
    protected List<Allergy> combineAllergiesWithReactions(List<Allergy> allergies, List<Allergy>reactions)
    {
    	if(reactions == null || reactions.size()==0){
    		return allergies;
    	}
    	if(allergies == null || allergies.size() == 0){
    		return reactions;
    	}
    	
    	List<Allergy> combinedAllergiesAndReactions = new ArrayList<Allergy>();
    	
    	for(Allergy allergy:allergies){
    		
    		for(Allergy reaction: reactions){
    			if(reaction.getAllergyId().equals(allergy.getAllergyId())){
    				allergy.setReactions(reaction.getReactions());
    				reactions.remove(reaction);
    				combinedAllergiesAndReactions.add(allergy);
    				break;
    			}
    			
    		}
    	}
    	
    	return combinedAllergiesAndReactions;
    }
    
}
