package gov.va.med.via.response;

import gov.va.med.via.databeans.Adt;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.TreatingSpecialty;
import gov.va.med.via.databeans.User;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * The Class InpatientMoveDataBeanParser.
 * 
 * @author DOMAINROMERJ
 *  
 */
public class InpatientMoveDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toAdts(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of Consult data beans using the data in the response received.
	 *   
	 * Patient Movement Fields in Vista (405)
     *  0:  IEN
     *  1:  DATE/TIME                       .01     Timestamp
     *  2:  TRANSACTION                     .02     Transaction
     *  3:  PATIENT                         .03     Patient.LocalPid
     *  4:  TYPE OF MOVEMENT                .04     MovementType
     *  5:  WARD LOCATION                   .06     AssignedLocation.Id
     *  6:  ROOM-BED                        .07     AssignedLocation.Room, AssignedLocation.Bed
     *  7:  FACILITY TREATING SPECIALTY     .09     Specialty, Service
     *  8:  ADMISSION/CHECK-IN MOVEMENT     .14     CheckInId
     *  9:  DISCHARGE/CHECK-OUT MOVEMENT    .17     CheckOutId
     * 10:  MAS MOVEMENT TYPE               .18     MasMovementType
     * 11:  ATTENDING PHYSICIAN             .19     Attending.Uid
     * 12:  RELATED PHYSICAL MOVEMENT       .24     RelatedPhysicalMovementId
     * 13:  LENGTH OF STAY                  201     LengthOfStay
     * 14:  PASS DAYS                       202     PassDays
     * 15:  DAYS ABSENT                     203     DaysAbsent
     * 16:  DIAGNOSIS [SHORT]               .1      Diagnosis
     * 17:  PTF ENTRY                       .16     PatientTxId
     * 18:  ADMITTED FOR SC CONDITION?      .11     AdmittedForScCondition
     * 19:  SCHEDULED ADMISSION?            .25     ScheduledAdmission
     * 20:  ^VA(200,                        (0,1)   Admitting.Name
     *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of Consult databeans populated from the content stored in the RPC response string
	 */
	protected Adt[] toAdts(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new Adt[0]; // TODO - is this correct?
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         
         List<Adt> result = new ArrayList<Adt>();
         ReferenceDataDelegate referenceDataDelegate = new ReferenceDataDelegate(vistaLinkCommand);
         
         for (int i = 0; i < lines.length; i++)
         {
             // ADT data starts after the [Data] tag in the response received. Skip processing the [Data] tag in the response.        	 
        	 if (lines[i].contains("[Data]"))
        		 continue;
        	 
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);
             if (fields != null && fields.length >0)
             {
            	 Adt adt = new Adt();
            	 adt.setSiteId(siteId);			
            	 adt.setId(fields[0]);
            	 adt.setTimestamp(!RPCStringUtils.isNullOrEmpty(fields[1])? Timestamp.toDateTime(fields[1]): null);
            	 adt.setTransaction(decodePatientMovementTransaction(fields[2]));
            	 Patient patient = new Patient();
            	 patient.setLocalPid(fields[3]);
            	 adt.setPatient(patient);
            	             	 
            	 // TODO - reformat using cache tables            	 
            	 String facilityMovementDesc = referenceDataDelegate.getCodeDescription(VistaConstants.FACILITY_MOVEMENT_TYPES, fields[4]);
            	 if (!RPCStringUtils.isNullOrEmpty(facilityMovementDesc))
            	 {
            		 adt.setMovementType(facilityMovementDesc);
            	 }

            	 // TODO - reformat using cache tables  
            	 String wardLocation = referenceDataDelegate.getCodeDescription(VistaConstants.WARD_LOCATIONS, fields[5]);
            	 if (!RPCStringUtils.isNullOrEmpty(wardLocation))
            	 {
            		 HospitalLocation location = new HospitalLocation();
            		 location.setId(fields[5]);
            		 location.setName(wardLocation); 
            		 adt.setAssignedLocation(location);            		 
            	 }
            	 
             	 // TODO - reformat using cache tables  
            	 String roomBedName = referenceDataDelegate.getCodeDescription(VistaConstants.ROOM_BEDS, fields[6]);
            	 if (!RPCStringUtils.isNullOrEmpty(roomBedName))
            	 {
            		 HospitalLocation location = null;
            		 if (adt.getAssignedLocation() == null)
            			 location = new HospitalLocation();
            		 else
            			 location = adt.getAssignedLocation();
            		 
            		 String[] parts = RPCStringUtils.split(roomBedName, "-");
            		 if (parts != null && parts.length > 1)
            		 {
            			 location.setRoom(parts[0]);
            			 location.setBed(parts[1]);
            		 }
            	 }
            	 
            	// TODO - reformat using cache tables  
            	 String treatingSpecialties = referenceDataDelegate.getCodeDescription(VistaConstants.TREATING_SPECIALTY, fields[7]);
            	 if (!RPCStringUtils.isNullOrEmpty(treatingSpecialties))
            	 {
            		 //TreatingSpecialty treatingSpecialty = new TreatingSpecialty();
            		 adt.setSpecialty(new KeyValuePair<String, String>(fields[7], treatingSpecialties));
            		 // adt.setService(new KeyValuePair<String, String>(fields[7], treatingSpecialties))  TODO - where do this come from?
            		 //result[i].Service = new KeyValuePair<string, string>(ts.Service.Key, ts.Service.Value);            		 
            	 }
            	             	 
            	 adt.setCheckInId(fields[8]);
            	 adt.setCheckOutId(fields[9]);
            	 String massMovementTypeDesc = referenceDataDelegate.getCodeDescription(VistaConstants.MAS_MOVEMENT_TYPES,fields[10]);	// TODO - refactor 
            	 adt.setMasMovementType(new KeyValuePair<String, String>(fields[10], massMovementTypeDesc));
            	 
            	 if (!RPCStringUtils.isNullOrEmpty(fields[11]))
            	 {
            		 User user = new User();
            		 user.setUid(fields[11]);
            		 if (fields.length > 19)
            		 {
            			 // Attending name contains name and specialty separated by ~
            			 String[] attendingParts = RPCStringUtils.split(fields[20], "~");
            			 user.setName(attendingParts != null && attendingParts.length > 0? attendingParts[0] : null);
            		 }
            		 adt.setAttending(user);
            	 } 
            	 
            	 adt.setRelatedPhysicalMovementId(fields[12]);
            	 adt.setLengthOfStay(fields[13]);
            	 adt.setPassDays(fields[14]);
            	 adt.setDaysAbsent(fields[15]);
            	 adt.setDiagnosis(fields[16]);
            	 adt.setPatientTxId(fields[17]);
            	 adt.setAdmittedForScCondition(fields[18].equals("1")? true : false);
            	 adt.setScheduledAdmission(fields[19].equals("1")? true : false);
            	 
            	 result.add(adt);
             }
         }
         
         return result.toArray(new Adt[0]);
    }
	
	/**
	 * Decode patient movement transaction code into its name
	 *
	 * @param code the code
	 * 
	 * @return the string containing the patient movement transaction name based on the code received.
	 */
	private String decodePatientMovementTransaction(String code)
	{
		if (code.equals("1"))
			return "ADMISSION";
		if (code.equals("2"))
			return "TRANSFER";
		if (code.equals("3"))
			return "DISCHARGE";				  
		if (code.equals("4"))
			return "CHECK-IN LODGER";
		if (code.equals("5"))
			return "CHECK-OUT LODGER";
		if (code.equals("6"))
			return "SPECIALTY TRANSFER";
		
		return "";
	}
}
