package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Facility;
import gov.va.med.via.databeans.Medication;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.util.StringUtil;
import gov.va.med.via.util.VistaTimestamp;
import gov.va.med.via.vistalink.command.VistaLinkCommand;
import gov.va.med.via.databeans.MedicationTypes;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This class parses the response from the get Meds RPC ORWPS ACTIVE and ORWRP REPORT TEXT(for NV Meds) call into DataBeans
 * @author DOMAINALLURS
 *
 */
public class MedsDataBeanParser extends BaseDataBeanParser {
	
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand){		
		List<DataBean> list = Arrays.asList((DataBean[])toMedication(response, dfn, siteId, webMethodName));
		return list;
	}
	
	/**
	 * Parses the get Meds RPC  ORWPS ACTIVE and populates an array of DataBean objects
	 * @param response - The response from RPC ORWPS ACTIVE
	 * @param dfn - the dfn
	 * @param siteId - the siteId
	 * @param webMethodName - the actor web method name 
	 * @return an array of Medication DataBeans populated from the content stored in the RPC response string
	 */
	protected Medication[] toMedication(String response, String dfn, String siteId, String webMethodName)
    {        
		if (response.equals("")) {
            return new Medication[0];
        }
		
		// Create this list of medications that need to retrieved for a particular web method
		// Only the medications that are added to this list will be retrieved
		ArrayList<String> medicationTypes = new ArrayList<String>();
		if (webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETOUTPATIENTMEDS)==0){
			medicationTypes.add(MedicationTypes.OP_MED);
		}else if ( webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETIMOMEDS)==0){
			medicationTypes.add(MedicationTypes.IMO_MED);
		}else if ( webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETIVMEDS)==0){
			medicationTypes.add(MedicationTypes.IV_MED);
		}else if ( webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETUNITDOSEMEDS)==0){
			medicationTypes.add(MedicationTypes.UD_MED);
		}else if (webMethodName.compareTo(ViaWebMethodNameConstants.EMRSVC_GETALLMEDS)==0){			
			medicationTypes.add(MedicationTypes.NV_MED);
			medicationTypes.add(MedicationTypes.OP_MED);
			medicationTypes.add(MedicationTypes.IV_MED);
			medicationTypes.add(MedicationTypes.UD_MED);
			medicationTypes.add(MedicationTypes.IMO_MED);
		}
		
        //check end of line characters - standardize to linux format
        response = RPCStringUtils.replaceEndofLineCharacters(response);               
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.TILDE);
        ArrayList<Medication> lst = new ArrayList<Medication>();
        
        for (int i = 0; i < lines.length; i++){
            if (lines[i].isEmpty()){
                continue;
            }       		
		    Medication medication = getMedication(lines[i], medicationTypes, siteId);
		    if ( medication!= null ){
		    	lst.add(medication);
		    }		    
        }
        Medication[] array =  (Medication[])lst.toArray(new Medication[lst.size()]);
        return array;
    }	
	
	/**
	 * Parses the get Meds RPC  ORWPS ACTIVE and populates an array of DataBean objects
	 * @param response - The response from RPC ORWPS ACTIVE
	 * @param dfn - the dfn
	 * @param siteId - the siteId
	 * @param webMethodName - the actor web method name 
	 * @return an array of Medication DataBeans populated from the content stored in the RPC response string
	 */
	private Medication getMedication(String curLine, ArrayList<String> medicationTypes, String siteId){
		// Sample response is as below
        //Typ^PharmID^Drug^InfRate^StopDt^RefRem^TotDose^UnitDose^OrderID^Status^LastFill
        //OP^1333450P;O^CIPROFLOXACIN HCL 500MG TAB^^^^^^5587565^PENDING^^^10^^0
        //CIPROFLOXACIN HCL 500MG TAB  Qty: 10
        //\ Sig: TAKE ONE TABLET BY MOUTH EVERY 12 HOURS
				
		String[] allFields = RPCStringUtils.split(curLine, RPCStringUtils.CRLF);
		String[] fields = RPCStringUtils.split(allFields[0], RPCStringUtils.CARET);		
        Medication medication = null;       
 
        if ( fields.length < 10){
        	return medication;
        }
        
        String type = null;
        if ( !StringUtil.isNullOrEmpty(fields[0]) ){
        	type = fields[0];
        }
        
        if (type == null ){
        	return medication;
        }
        // TODO
        // For lack of test data for all the medication types,
        // each medication is parsed separately. Later on code should be 
        // refactored to move common code into a separate method
        if ( medicationTypes.contains(type)){
	        if ( type.compareTo(MedicationTypes.OP_MED)==0){
	        	medication = getOutPatientMedication(curLine, siteId);
	        }else if (type.compareTo(MedicationTypes.IV_MED)==0){
	        	medication = getIVMedication(curLine, siteId);
	        }else if (type.compareTo(MedicationTypes.IMO_MED)==0){
	        	medication = toInpatientForOutpatientMed(curLine, siteId);
	        }else if (type.compareTo(MedicationTypes.UD_MED)==0){
	        	medication = getUnitDoseMedication(curLine, siteId);
	        }
        }
        return medication;
	}
	
	/**
	 * Parses an individual line in outpatient response(from RPC ORWPS ACTIVE) 
	 * @param curLine - The current line that is being parsed
	 * @param siteId - the siteId
	 * @param webMethodName - the actor web method name 
	 * @return aMedication DataBeans populated from the current response line
	 */
	private Medication getOutPatientMedication(String curLine, String siteId){
		// Sample response is as below
        //Typ^PharmID^Drug^InfRate^StopDt^RefRem^TotDose^UnitDose^OrderID^Status^LastFill
        //OP^1333450P;O^CIPROFLOXACIN HCL 500MG TAB^^^^^^5587565^PENDING^^^10^^0
        //CIPROFLOXACIN HCL 500MG TAB  Qty: 10
        //\ Sig: TAKE ONE TABLET BY MOUTH EVERY 12 HOURS
		
		String[] allFields = RPCStringUtils.split(curLine, RPCStringUtils.CRLF);
		String[] fields = RPCStringUtils.split(allFields[0], RPCStringUtils.CARET);		
        Medication medication = null;       
 
        if ( fields.length < 10){
        	return medication;
        }
        
        String type = null;
        if ( !StringUtil.isNullOrEmpty(fields[0]) ){
        	type = fields[0];
        }
        
        String id = null;
        if ( !StringUtil.isNullOrEmpty(fields[1]) ){
        	id = fields[1]; //Pharma Id
        }
        
        String name = null;
        if ( !StringUtil.isNullOrEmpty(fields[2]) ){
        	name = fields[2]; // Medication/Drug name
        }
        
        String infRate = null;
        if ( !StringUtil.isNullOrEmpty(fields[3]) ){
        	infRate = fields[3];
        }
        
        String stopDt = null;
        if ( !StringUtil.isNullOrEmpty(fields[4]) ){
        	stopDt = fields[4];
        }
        
        String refRem = null;
        if ( !StringUtil.isNullOrEmpty(fields[5]) ){
        	refRem = fields[5];
        }
        
        String totDose = null;
        if ( !StringUtil.isNullOrEmpty(fields[6]) ){
        	totDose = fields[6];
        }

        String unitDose = null;
        if ( !StringUtil.isNullOrEmpty(fields[7]) ){
        	unitDose = fields[7];
        }
        
        String orderId = null;
        if ( !StringUtil.isNullOrEmpty(fields[8]) ){
        	orderId = fields[8];
        }
        
        String status = null;
        if ( !StringUtil.isNullOrEmpty(fields[9]) ){
        	status = fields[9];
        }
        
        Facility facility = new Facility();
        facility.setTag(siteId);
        try {
			facility.setText(SiteTable.getInstance().getSiteDisplayName(siteId));
		} catch (SiteException e) {
		}
        
        String lastFillDate = null;
        if ( fields.length > 10 && !StringUtil.isNullOrEmpty(fields[10])){
        	lastFillDate = fields[10];
        }
        
        String quantity = null;
        if ( fields.length > 12 && !StringUtil.isNullOrEmpty(fields[12])){
        	quantity = fields[12];
        }
        
        String daysSupply = null;
        if ( fields.length > 13 && !StringUtil.isNullOrEmpty(fields[13])){
        	daysSupply = fields[13];
        }

        medication = new Medication();
        medication.setType(type);
        medication.setId(id);
        medication.setName(name);
        medication.setInfRate(infRate);
        medication.setStopDate(stopDt);
        medication.setComment(refRem);
        medication.setUnitDose(unitDose);
        medication.setTotalDose(totDose);
        medication.setOrderId(orderId);
        medication.setStatus(status);
        
        if ( lastFillDate!= null ){
        	medication.setLastFillDate(VistaTimestamp.toUtcString(lastFillDate));
        }
        
        medication.setQuantity(quantity);
        medication.setDaysSupply(daysSupply);
        medication.setIsInpatient(false);
        medication.setIsIV(false);
        medication.setIsImo(false);
        medication.setIsNonVA(false);
        medication.setIsUnitDose(false);       
        medication.setIsOutpatient(true);

        medication.setFacility(facility);
 
        if ( allFields.length > 1 && !StringUtil.isNullOrEmpty(allFields[1])){
        	medication.setDetail(allFields[1]);
        }
        if ( allFields.length > 2 && !StringUtil.isNullOrEmpty(allFields[2])){
        	medication.setSig(allFields[2]);
        }        
        return medication;
	}

	/**
	 * Parses the response from Other meds response(from RPC ORWRP REPORT TEXT OR_RXN:HERBAL/OTC/NON-VA MEDS~NVA;ORDV06A;0;0) 
	 * @param response - The response from the RPC
	 * @return a list of DataBeans populated from the current response line
	 */
	public List<DataBean> getOtherMedsRdv(String response){		
		/*
		 * This method assumes that each response line is separated by CRLF and fields within each line
		 *  are separated by CR. Each field number and actual field are separated by semicolon
		 */
		// Sample response(both lines will be together)
		// "1^CHEYENNE VAMC;442\r\n2^CELECOXIB CAP,ORAL\r\n3^Discontinued\r\n4^\r\n5^02/18/2005 14:18\r\n" +
		// "6^BIESCHKE,VANCE M JR\r\n7^05/20/2005 11:54\r\n8^200MG MOUTH TWICE A DAY\r\n10^Medication prescribed by Non-VA provider.  \r\n"
		List<DataBean> lst = new ArrayList<DataBean>();        
		String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        Medication medication = null;
        for (int i = 0; i < lines.length; i++){
            if (lines[i].isEmpty()){
                continue;
            }       		
            String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CR);		   

        	String comment = "";
            int numFields = fields.length;
            for(int index=0; index<numFields; index++){

            	String fieldNumberValue = fields[index];
            	if ( fieldNumberValue.isEmpty() ){
            		continue;
            	}
            	String[] fieldNumberValueArray = fieldNumberValue.split("\\^");
            	if ( fieldNumberValueArray.length != 2 ){
            		continue;
            	}
            	String fieldNumber = fieldNumberValueArray[0];
            	String fieldValue = fieldNumberValueArray[1];
            	int fieldNumberInt = Integer.parseInt(fieldNumber);
            	switch(fieldNumberInt){
	            	case 1:
	            		if ( medication != null ){
	                        medication.setType(MedicationTypes.NV_MED);
	            			lst.add(medication);
	            		}
            			medication = new Medication();
            			comment = "";
            			setNVMedsDefaults(medication);
	            		String[] field1Array = fieldValue.split(";");
	            		String siteId = field1Array[1];
	            		String siteName = field1Array[0];
	            		Facility facility = new Facility();
	            		facility.setTag(siteId);
	            		facility.setText(siteName);
	            		medication.setFacility(facility);
	            		break;
	            	case 2:
	            		medication.setName(fieldValue);
	            		break;
	            	case 3:
	            		medication.setStatus(fieldValue);
	            		break;
	            	case 4:
	            		if ( fieldValue != null && !fieldValue.isEmpty())
	            			medication.setStartDate(VistaTimestamp.toUtcFromRdv(fieldValue));
	            		break;
	            	case 5:
	            		if ( fieldValue != null && !fieldValue.isEmpty())
	            			medication.setDateDocumented(VistaTimestamp.toUtcFromRdv(fieldValue));
	            		break;
	            	case 6:
	            		if ( fieldValue != null && !fieldValue.isEmpty())
	            			medication.setDocumentor(new Author(null, fieldValue, null));
	            		break;
	            	case 7:
	            		if ( fieldValue != null && !fieldValue.isEmpty())
	            			medication.setStopDate(VistaTimestamp.toUtcFromRdv(fieldValue));
	            		break;
	            	case 8:
	            		medication.setSig(fieldValue);
	            		break;
	            	case 10:
	            		comment += fieldValue;
	            		medication.setComment(comment);
	            		break;
            	}            	
            }
        }
        // Add the last medication
		if ( medication != null ){
			lst.add(medication);
		}
       return lst;
	}
	
	/**
	 * This method sets the defaults for Other Meds(NV) 
	 * @param medication - The NV medication object whose default values are being set
	 * @return void
	 */
	private void setNVMedsDefaults(Medication medication){
        medication.setIsInpatient(false);
        medication.setIsOutpatient(true);
        medication.setIsIV(false);
        medication.setIsImo(false);
        medication.setIsNonVA(true);
        medication.setIsUnitDose(false);
        medication.setType(MedicationTypes.NV_MED);
	}
	
	// This method has not been tested as there is not test data available
	private Medication getIVMedication(String curLine, String siteId){		
		String[] allFields = RPCStringUtils.split(curLine, RPCStringUtils.CRLF);
		String[] fields = RPCStringUtils.split(allFields[0], RPCStringUtils.CARET);		
        Medication medication = null;       
 
        if ( fields.length < 16){
        	return medication;
        }
        
        String type = null;
        if ( !StringUtil.isNullOrEmpty(fields[0]) ){
        	type = fields[0];
        }
        
        String id = null;
        if ( !StringUtil.isNullOrEmpty(fields[1]) ){
        	id = fields[1]; //Pharma Id
        }
        
        String name = null;
        if ( !StringUtil.isNullOrEmpty(fields[2]) ){
        	name = fields[2]; // Medication/Drug name
        }
        String stopDt = null;
        if ( !StringUtil.isNullOrEmpty(fields[4]) ){
        	stopDt = fields[4];
        }
        
        String dose = null;
        if ( !StringUtil.isNullOrEmpty(fields[6]) ){
        	dose = fields[6];
        }

        String orderId = null;
        if ( !StringUtil.isNullOrEmpty(fields[8]) ){
        	orderId = fields[8];
        }
        
        String status = null;
        if ( !StringUtil.isNullOrEmpty(fields[9]) ){
        	status = fields[9];
        }
        
        Facility facility = new Facility();
        facility.setTag(siteId);
        try {
			facility.setText(SiteTable.getInstance().getSiteDisplayName(siteId));
		} catch (SiteException e) {
		}
      
        medication = new Medication();
        medication.setType(type);
        medication.setId(id);
        medication.setName(name);
        medication.setStopDate(stopDt);
        medication.setDose(dose);
        medication.setOrderId(orderId);
        medication.setStatus(status);
        medication.setRoute("INTRAVENOUS");
        
        medication.setIsInpatient(true);
        medication.setIsIV(true);
        medication.setIsImo(false);
        medication.setIsNonVA(false);
        medication.setIsUnitDose(false);       
        medication.setIsOutpatient(true);

        medication.setFacility(facility);
 
       /* This code needs to be implemented post test data availability
        * med.Detail = textBlob;

            string[] textFields = StringUtils.split(textBlob, StringUtils.CRLF);
            
            if (textFields.Length > 0)
            {
                // temporarily make the following assumptions until new RPCs are available for The Daily Plan
                // Line 1:  Additive  +  Dose
                // Line Containing "\in" is the solution
                // Line Containing "\IV" is the Schedule
                // Everything else we don't understand... but its in the detail field if needed
                if (String.IsNullOrEmpty(med.Dose))
                {
                    int split = StringUtils.firstIndexOfNum(textFields[0]);
                    med.Dose = textFields[0].Substring(split);
                    med.Additives = textFields[0].Substring(0, split).Trim();
                }

                foreach (String line in textFields)
                {
                    if (line.Contains(@"\in"))
                    {
                        med.Solution = line.Replace(@"\in", "").Trim();
                    }
                    else if (line.Contains(@"\IV "))
                    {
                        med.Schedule = line.Replace(@"\IV", "").Trim();
                    }
                }
            }

        */
        return medication;
	}
	
	
	// This method has not been tested as there is not test data available
	private Medication toInpatientForOutpatientMed(String curLine, String siteId){		
		String[] allFields = RPCStringUtils.split(curLine, RPCStringUtils.CRLF);
		String[] fields = RPCStringUtils.split(allFields[0], RPCStringUtils.CARET);		
        Medication medication = null;       
 
        if ( fields.length < 16){
        	return medication;
        }
        
        String type = null;
        if ( !StringUtil.isNullOrEmpty(fields[0]) ){
        	type = fields[0];
        }
        
        String id = null;
        if ( !StringUtil.isNullOrEmpty(fields[1]) ){
        	id = fields[1]; //Pharma Id
        }
        
        String name = null;
        if ( !StringUtil.isNullOrEmpty(fields[2]) ){
        	name = fields[2]; // Medication/Drug name
        }
        String stopDt = null;
        if ( !StringUtil.isNullOrEmpty(fields[4]) ){
        	stopDt = fields[4];
        }
        
        String dose = null;
        if ( !StringUtil.isNullOrEmpty(fields[6]) ){
        	dose = fields[6];
        }

        String orderId = null;
        if ( !StringUtil.isNullOrEmpty(fields[8]) ){
        	orderId = fields[8];
        }
        
        
        Facility facility = new Facility();
        facility.setTag(siteId);
        try {
			facility.setText(SiteTable.getInstance().getSiteDisplayName(siteId));
		} catch (SiteException e) {
		}
      
        medication = new Medication();
        medication.setType(type);
        medication.setId(id);
        medication.setName(name);
        medication.setStopDate(stopDt);
        medication.setDose(dose);
        medication.setOrderId(orderId);
        
        medication.setIsInpatient(false);
        medication.setIsIV(false);
        medication.setIsImo(true);
        medication.setIsNonVA(false);
        medication.setIsUnitDose(false);       
        medication.setIsOutpatient(true);

        medication.setFacility(facility);
 
        /* TODO
         *  string[] hospital = StringUtils.split(fields[0], ":");
            med.Hospital = new KeyValuePair<string, string>(hospital[2], hospital[1]);
            med.Detail = textBlob;
         */
 
        return medication;
	}
	
	// This method has not been tested as there is not test data available
	private Medication getUnitDoseMedication(String curLine, String siteId){		
		String[] allFields = RPCStringUtils.split(curLine, RPCStringUtils.CRLF);
		String[] fields = RPCStringUtils.split(allFields[0], RPCStringUtils.CARET);		
        Medication medication = null;       
 
        if ( fields.length < 16){
        	return medication;
        }
        
        String type = null;
        if ( !StringUtil.isNullOrEmpty(fields[0]) ){
        	type = fields[0];
        }
        
        String id = null;
        if ( !StringUtil.isNullOrEmpty(fields[1]) ){
        	id = fields[1]; //Pharma Id
        }
        
        String name = null;
        if ( !StringUtil.isNullOrEmpty(fields[2]) ){
        	name = fields[2]; // Medication/Drug name
        }
        String stopDt = null;
        if ( !StringUtil.isNullOrEmpty(fields[4]) ){
        	stopDt = fields[4];
        }
        
        String dose = null;
        if ( !StringUtil.isNullOrEmpty(fields[6]) ){
        	dose = fields[6];
        }

        String orderId = null;
        if ( !StringUtil.isNullOrEmpty(fields[8]) ){
        	orderId = fields[8];
        }
        
        String status = null;
        if ( !StringUtil.isNullOrEmpty(fields[9]) ){
        	status = fields[9];
        }
        
        Facility facility = new Facility();
        facility.setTag(siteId);
        try {
			facility.setText(SiteTable.getInstance().getSiteDisplayName(siteId));
		} catch (SiteException e) {
		}
      
        medication = new Medication();
        medication.setType(type);
        medication.setId(id);
        medication.setName(name);
        medication.setStopDate(stopDt);
        medication.setDose(dose);
        medication.setOrderId(orderId);
        medication.setStatus(status);
        
        medication.setIsInpatient(true);
        medication.setIsIV(true);
        medication.setIsImo(false);
        medication.setIsNonVA(false);
        medication.setIsUnitDose(false);       
        medication.setIsOutpatient(true);

        medication.setFacility(facility);
 
 
        return medication;
	}
	
}
