package gov.va.med.via.response;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Fault;
import gov.va.med.via.databeans.HospitalLocation;
import gov.va.med.via.databeans.Patient;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.request.ViaWebMethodNameConstants;
import gov.va.med.via.rpc.DateUtils;
import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;

import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * The Class PatientSelectDataBeanParser.
 * 
 * @author DOMAINROMERJ
 */
public class PatientSelectDataBeanParser extends BaseDataBeanParser
{
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toPatients(response, queryBean, dfn, siteId, vistaLinkCommand, webMethodName));
	}

	/**
	 * Parses a response string and populates all data for a patient using the response received and other RPC calls via
	 * the PatientDelegate and PatientDemographicsDelegate classes.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of Patient databeans populated from the content stored in the RPC response string
	 */
	protected Patient[] toPatients(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand, String webMethodName)
    {
        if (response.equals("")) return new Patient[0];
        
        response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
        ArrayList<Patient> lst = new ArrayList<Patient>(lines.length);
        
        for (int i = 0; i < lines.length; i++)
        {
            if (lines[i].equals("")) continue;

            Patient patient = toPatient(lines[i], queryBean, dfn, siteId, vistaLinkCommand);
            if (!patient.hasErrors())
            {
            	PatientDelegate patientDelegate = new PatientDelegate(queryBean, siteId, vistaLinkCommand);
                patientDelegate.addPatientIdInfo(patient);
            	patientDelegate.addActiveInsurance(patient);
            	patientDelegate.addDeceasedDate(patient);
            	patientDelegate.addSCListerNodeMPI(patient, siteId);
            	patientDelegate.addConfidentiality(patient);
            	patientDelegate.addPatientFlags(patient);
            	patientDelegate.addTeam(patient);
            	patientDelegate.addNeedsMeansTest(patient);
            	patientDelegate.addSiteIds(patient, siteId);
            	
            	PatientDemographicsDelegate patientDemoDelegate = new PatientDemographicsDelegate(queryBean, siteId, vistaLinkCommand);
            	patientDemoDelegate.addDemographics(patient);
            	
            	// Include the Home Data on the Patient object 
            	if (ViaWebMethodNameConstants.EMRSVC_GETDEMOGRAPHICS.equals(webMethodName) ||
            		ViaWebMethodNameConstants.FIXTHEPHONESSVC_GETDEMOGRAPHICS.equals(webMethodName))
            	{
            		patientDemoDelegate.addHomeData(patient);
            	}
            }
                
            lst.add(patient);
        }

        return lst.toArray(new Patient[0]);
    }
	
	/**
	 * Parses a response string and populates the main data for a patient using the response received.
	 * 
	 * @param response 
	 * @param dfn
	 * @param siteId
	 * @param vistaLinkTemplate
	 * 
	 * @return a Patient DataBean populated from the content stored in the RPC response string
	 */
	 protected Patient toPatient(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
     {
	    Patient patient = new Patient();
	    
        if (response.equals("")) return patient;
        
        response = RPCStringUtils.replaceEndofLineCharacters(response);        
        String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
        if (!fields[0].equals("-1")) // Add data if the response has no errors
        {
	        patient.setName(fields[0]);
	        patient.setGender(fields[1]);
	        patient.setDob(DateUtils.trimTime(Timestamp.toUtcString(fields[2])));
	        patient.setSsn(fields[3]);
	        patient.setLocalPid(dfn); 
	        patient.setInpatient(false);
	        patient.setHasInsurance(false); // TODO - This field it is not setup in MDWS!
	        if (!RPCStringUtils.isEmpty(fields[4]) && !RPCStringUtils.isEmpty(fields[5]))
	        {
	            HospitalLocation hl = new HospitalLocation();
	            hl.setId(fields[4]);
	            hl.setName(fields[5]);
	            hl.setRoom(fields[6]);
	            if (!RPCStringUtils.isEmpty(fields[15]))
	            {
	                hl.setSpecialty(new KeyValuePair<String, String>(fields[15], ""));
	            }
	            patient.setLocation(hl);
	           
	            patient.setInpatient(true);
	        }
	        patient.setCwad(fields[7]);
	        patient.setRestricted(fields[8].equals("1")? true : false);

	        if (!RPCStringUtils.isEmpty(fields[9]))
	        {
	            patient.setAdmitTimestamp(Timestamp.toUtcString(fields[9]));
	        }

	        patient.setServiceConnected(fields[11].equals("1")? true : false);
	        
	        if (!RPCStringUtils.isEmpty(fields[12]) && RPCStringUtils.isNumeric(fields[12]))
	        {
	            patient.setScPercent(new Integer(fields[12]));
	        }
	        else
	        	patient.setScPercent(new Integer(0));
	        
	        patient.setMpiPid(fields[13]);
	        if (!RPCStringUtils.isEmpty(fields[14]) && RPCStringUtils.isNumeric(fields[14]))
	        {
	        	patient.setAge(new Integer(fields[14]));
	        }        
	        
	        ArrayList<String> sites = new ArrayList<String>();
	        sites.add(siteId);
	        patient.setVistaLocations(sites.toArray(new String[0]));
        }
        else // response has errors.
        {
        	patient.setFault(new Fault(fields[5]));
        }

        return patient;
     }	 
}
