package gov.va.med.via.response;

import gov.va.med.via.databeans.Author;
import gov.va.med.via.databeans.Problem;
import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.databeans.ObservationType;
import gov.va.med.via.databeans.SiteId;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.site.SiteException;
import gov.va.med.via.rpc.site.SiteTable;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ProblemListDataBeanParser extends BaseDataBeanParser
{

	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{				
		return Arrays.asList((DataBean[])toProblemList(response, queryBean, dfn, siteId, vistaLinkCommand));
	}

	/**
	 * Parses a response string and creates a list of MicrobiologyReport data beans using the data in the response received.
	 *
	 * @param response the response
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param siteId the site id
	 * @param vistaLinkTemplate the vista link template
	 * @param connectionSpecType the connection spec type
	 * 
	 * @return an array of MicrobiologyReport databeans populated from the content stored in the RPC response string
	 */
	protected Problem[] toProblemList(String response, QueryBean queryBean, String dfn, String siteId, VistaLinkCommand vistaLinkCommand)
    {
		 if (RPCStringUtils.isNullOrEmpty(response))
         {
             return new Problem[0]; 
         }
		 
		 response = RPCStringUtils.replaceEndofLineCharacters(response);        
         String[] lines = RPCStringUtils.split(response, RPCStringUtils.CRLF);
         lines = RPCStringUtils.trimArray(lines);
         List<Problem> responseArray = new ArrayList<Problem>();
         Problem problem = null;
         for (int i = 0; i < lines.length; i++)
         {
        	 if ("".equals(lines[i])) continue;
        	 
             String[] fields = RPCStringUtils.split(lines[i], RPCStringUtils.CARET);             
             int fieldNumber = 0;
             try
             {
            	 fieldNumber = new Integer(fields[0]).intValue();         
             } catch (NumberFormatException e) 
             {
            	 continue;
             }
             switch (fieldNumber)
             {
             	case 1:
             		if (problem != null)  
             		{
             			if (problem.getProviderNarrative() != null)
             			{
             				problem.setProviderNarrative(RPCStringUtils.CSubstring(problem.getProviderNarrative(), 
             						0, problem.getProviderNarrative().length() - 1));
             			}
             			if (problem.getNoteNarratives() != null)
             			{
             				problem.setNoteNarratives(RPCStringUtils.CSubstring(problem.getNoteNarratives(), 
             						0, problem.getNoteNarratives().length() - 1));
             			}
             			if (problem.getExposures() != null)
             			{
             				problem.setExposures(RPCStringUtils.CSubstring(problem.getExposures(), 
             						0, problem.getExposures().length() - 1));
             			}
             			responseArray.add(problem);
             		}
             		problem = new Problem();
             		problem.setType(new ObservationType("","Problems and Diagnoses", "Problem"));
             		if (fields.length == 2)
             		{
             			String parts[] = RPCStringUtils.split(fields[1], RPCStringUtils.SEMICOLON);
             			if (parts.length == 2)
             			{
             				problem.setFacility(new SiteId(parts[1], parts[0], "",""));
             			}
             			else if (!fields[1].isEmpty())
             			{
             				problem.setFacility(new SiteId(siteId, fields[1], "",""));
             			}
             			else
             			{
             				try
             				{
             					problem.setFacility(new SiteId(siteId, SiteTable.getInstance().getSiteName(siteId), "",""));
             				}
             		       	catch(SiteException e)
             		       	{
             		       		// Ignore it, site will not have name. Should never happen if valid site.
             		       	}
             			}
             		}
             		break;
             	case 2:
             		problem.setStatus( fields.length == 2? fields[1]: null);
             		break;
             	case 3:
             		if (fields.length == 2)
             		{
             			problem.setProviderNarrative(problem.getProviderNarrative() != null? 
             					problem.getProviderNarrative().concat(fields[1] + '\n'): fields[1] + '\n');
             		}
             		break;             		
             	case 4:
             		problem.setOnsetDate(fields.length == 2? Timestamp.toUtcFromRdv(fields[1]) : null);
             		break;
             	case 5:
             		/// bug encountered where the fields[1] value is "V65.9 [UNSPECIFIED REASON FOR CONSULTATION]"
             		/// to minimize this problem, a try catch is wrapped around the method invocation - allowing as much of the data as possible to be parsed 
             		try
             		{
             			problem.setLastModifiedDate(fields.length == 2? Timestamp.toUtcFromRdv(fields[1]) : null);
             		}
             		catch ( Exception ignore)
             		{             			
             		}
             		break;
             	case 6:
             		if (fields.length == 2)
             		{
             			problem.setObserver(new Author("",fields[1],""));
             		}
             		break;
             	case 7:            	 
             		if (fields.length == 2)
             		{
             			problem.setNoteNarratives(problem.getNoteNarratives() != null? 
             					problem.getNoteNarratives().concat(fields[1] + '\n'): fields[1] + '\n');
             		}
             		break;
            	case 8:            
            		if (fields.length == 2)
             		{
            			problem.setExposures(problem.getExposures() != null? 
            					problem.getExposures().concat(fields[1] + '\n'): fields[1] + '\n');
             		}
             		break;
             }
         } 
           
         if (problem != null)  
  		 {
  			if (problem.getProviderNarrative() != null)
  			{
  				problem.setProviderNarrative(RPCStringUtils.CSubstring(problem.getProviderNarrative(), 
  						0, problem.getProviderNarrative().length() - 1));
  			}
  			if (problem.getNoteNarratives() != null)
  			{
  				problem.setNoteNarratives(RPCStringUtils.CSubstring(problem.getNoteNarratives(), 
  						0, problem.getNoteNarratives().length() - 1));
  			}
  			if (problem.getExposures() != null)
  			{
  				problem.setExposures(RPCStringUtils.CSubstring(problem.getExposures(), 
  						0, problem.getExposures().length() - 1));
  			}
  			responseArray.add(problem);
  		 }
         
         return responseArray.toArray(new Problem[0]);
    }
}
