package gov.va.med.via.response;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import gov.va.med.via.databeans.DataBean;
import gov.va.med.via.databeans.Order;
import gov.va.med.via.databeans.Provider;
import gov.va.med.via.databeans.QueryBean;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.RpcNameConstants;
import gov.va.med.via.rpc.Timestamp;
import gov.va.med.via.rpc.VistaConstants;
import gov.va.med.via.vistalink.command.VistaLinkCommand;

/**
 * The Class WriteSimpleOrderByPolicyDataBeanParser.
 * 
 * @author DOMAINROMERJ
 */
public class WriteSimpleOrderByPolicyDataBeanParser extends BaseDataBeanParser 
{	
	
	/* (non-Javadoc)
	 * @see gov.va.med.via.response.BaseDataBeanParser#populateDataBeans(java.lang.String, gov.va.med.via.databeans.QueryBean, java.lang.String, java.lang.String, java.lang.String, gov.va.med.via.vistalink.command.VistaLinkCommand)
	 */
	@Override
	public List<DataBean> populateDataBeans(String response, QueryBean queryBean, String dfn, String siteId, String webMethodName, VistaLinkCommand vistaLinkCommand)
	{		
		List<DataBean> list = Arrays.asList((DataBean[])toOrder(queryBean, response, dfn, vistaLinkCommand));
		return list;
	}
	
	/**
	 * Parses the get admissions RPC response string and populates an array of DataBean objects.
	 *
	 * @param queryBean the query bean
	 * @param response - The response from RPC ORWPT ADMITLST
	 * @param dfn - the dfn
	 * @param vistaLinkCommand the vista link command
	 * 
	 * @return an array of InpatientStay DataBeans populated from the content stored in the RPC response string
	 */
	protected Order[] toOrder(QueryBean queryBean, String response, String dfn, VistaLinkCommand vistaLinkCommand)
    {
		if (RPCStringUtils.isNullOrEmpty(response))
		{
            return new Order[0];
        }

		String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);		
        List<Order> result = new ArrayList<Order>();		
		Order order = new Order();
		order.setId(fields[0].substring(1));
		order.setTimestamp(fields[2] != null? Timestamp.toDateTime(fields[2]): null);
		order.setStartDate(fields[3] != null? Timestamp.toDateTime(fields[3]): null);
		order.setStopDate(fields[4] != null? Timestamp.toDateTime(fields[4]): null);
		order.setStatus(fields[5]);
		order.setSigStatus(fields[6]);
		order.setVerifyingNurse(fields[7]);
		order.setVerifyingClerk(fields[8]);
		Provider provider = new Provider();
		provider.setUserId(fields[9]);
		provider.setName(fields[10]);
		order.setProvider(provider);
		result.add(order);
	
		releaseOrderByPolicy(queryBean, dfn, order, vistaLinkCommand);
		
		return result.toArray(new Order[0]);
    }
	
	/**
	 * Release order by policy.
	 *
	 * @param queryBean the query bean
	 * @param dfn the dfn
	 * @param order the order
	 * @param vistaLinkCommand the vista link command
	 */
	private void releaseOrderByPolicy(QueryBean queryBean, String dfn, Order order, VistaLinkCommand vistaLinkCommand)
	{ 
		if (!lockOrdersForPatient(dfn, vistaLinkCommand))
	    {
			order.setErrMsg("Unable to lock patient's orders");
	        return;
	    }
	    String rtn = lockOrder(order.getId(), vistaLinkCommand);
	    if (!rtn.equals("OK"))
	    {
	        order.setErrMsg("Unable to lock order: " + rtn);
	        unlockOrdersForPatient(dfn, vistaLinkCommand);
	        return;
	    }
	    rtn = getComplexOrderMessage(order.getId(), vistaLinkCommand);
	    if (!rtn.equals(""))
	    {
	        order.setErrMsg("Complex order: " + rtn);
	        unlockOrdersForPatient(dfn, vistaLinkCommand);
	        unlockOrder(order.getId(), vistaLinkCommand);
	        return;
	    }
	    rtn = checkReleaseOrder(order, dfn, vistaLinkCommand);
	    
	    if (!rtn.equals(""))
	    {
	        order.setErrMsg("Release order error: " + rtn);
	        unlockOrdersForPatient(dfn,vistaLinkCommand);
	        unlockOrder(order.getId(), vistaLinkCommand);
	        return;
	    }

	    sendOrderByPolicy(dfn, queryBean, order, vistaLinkCommand);
	    
	    if (!unlockOrder(order.getId(), vistaLinkCommand))
	    {
	        order.setErrMsg("Unable to unlock order: " + rtn);
	    }
	    
	    if (!unlockOrdersForPatient(dfn, vistaLinkCommand))
	    {
	        order.setErrMsg("Unable to unlock patient's orders: " + rtn);
	    }		
	}
	

	/**
	 * Lock orders for patient.
	 *
	 * @param dfn the dfn
	 * @param vistaLinkCommand the vista link command
	 * @return true, if successful
	 */
	private boolean lockOrdersForPatient(String dfn, VistaLinkCommand vistaLinkCommand)
	{
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_LOCK, dfn);				
		return "1".equals(response);
	}
	
	/**
	 * Lock order.
	 *
	 * @param orderIEN the order ien
	 * @param vistaLinkCommand the vista link command
	 * @return the string
	 */
	private String lockOrder(String orderIEN, VistaLinkCommand vistaLinkCommand)
	{
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_LOCK_ORDER, orderIEN);
		String[] fields = response.split("\\^");
		/// if the order result ==1, then the order has successfully been locked otherwise, the order has not been locked
		if ( "1".equals(fields[0]))
		{
			//return fields[1];			/// order successfully locked
		}
		return "OK"; 
		/// the 2nd part of the caret separated return string will contain some type of error message
		//throw new RpcException(ErrorCodeEnum.NOTE_UNABLE_TO_LOCK_ORDER, flds[1]);
	}
	
	/**
	 * Unlock orders for patient.
	 *
	 * @param dfn the dfn
	 * @param vistaLinkCommand the vista link command
	 * @return true, if successful
	 */
	private boolean unlockOrdersForPatient(String dfn, VistaLinkCommand vistaLinkCommand)
	{
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_UNLOCK, dfn);				
		return "1".equals(response);
	}
	
	/**
	 * Gets the complex order message.
	 *
	 * @param orderId the order id
	 * @param vistaLinkCommand the vista link command
	 * @return the complex order message
	 */
	private String getComplexOrderMessage(String orderId, VistaLinkCommand vistaLinkCommand)
	{
		return vistaLinkCommand.executeRpc(RpcNameConstants.ORWDXA_OFCPLX, orderId);
	}
	
	/**
	 * Unlock order.
	 *
	 * @param ien the ien
	 * @param vistaLinkCommand the vista link command
	 * @return true, if successful
	 */
	private boolean unlockOrder(String ien, VistaLinkCommand vistaLinkCommand )
    {
		 List<Object> params =new ArrayList<Object>();
		 params.add(ien);
		 String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_UNLOCK_ORDER, params);
		 return "1".equals(response);
    }
	
	/**
	 * Check release order.
	 *
	 * @param order the order
	 * @param dfn the dfn
	 * @param vistaLinkCommand the vista link command
	 * @return the string
	 */
	private String checkReleaseOrder(Order order, String dfn, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(dfn);
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		hashMap.put("1", order.getId() + "^^1");
		params.add(hashMap);
		
		return vistaLinkCommand.executeRpc(RpcNameConstants.ORWDXC_SESSION, params);
	}
	
	/**
	 * Send order by policy.
	 *
	 * @param dfn the dfn
	 * @param queryBean the query bean
	 * @param order the order
	 * @param vistaLinkCommand the vista link command
	 */
	private void sendOrderByPolicy(String dfn, QueryBean queryBean, Order order, VistaLinkCommand vistaLinkCommand)
	{
		List<Object> params =new ArrayList<Object>();
		params.add(dfn);
		params.add("0");
		params.add(queryBean.getCriteria());	//Location IEN
		params.add(queryBean.getTarget());		// ESIG
		
		HashMap<Object,Object> hashMap = new HashMap<Object,Object>();
		String value = order.getId() + '^' + VistaConstants.SS_ESIGNED + '^' +
					   VistaConstants.RS_RELEASE + '^' + VistaConstants.NO_POLICY;
		hashMap.put("1", value);
		params.add(hashMap);
		
		String response = vistaLinkCommand.executeRpc(RpcNameConstants.ORWDX_SEND, params);
		String[] fields = RPCStringUtils.split(response, RPCStringUtils.CARET);
			
		if (!order.getId().equals(fields[0]))
		{
			order.setErrMsg("ID mismatch returning from send: expected " + order.getId() + ", found " + fields[0]);
			return;
		}
	
		if (fields.length > 1 && fields[1].contains("RS"))
		{
			order.setStatus("Released");
		}
		else
		{
			if (fields.length > 3)
			{
				order.setStatus("Error" + fields[3]);
			}
		}
	}
}
