package gov.va.med.via.vistalink;


import gov.va.med.via.vistalink.exception.VistaLinkDataAccessException;

import gov.va.med.exception.FoundationsException;

import gov.va.med.vistalink.adapter.cci.VistaLinkAppProxyConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkConnection;
import gov.va.med.vistalink.adapter.cci.VistaLinkConnectionSpec;
import gov.va.med.vistalink.adapter.cci.VistaLinkDuzConnectionSpec;
import gov.va.med.vistalink.adapter.record.LoginsDisabledFaultException;
import gov.va.med.vistalink.adapter.record.VistaLinkFaultException;
import gov.va.med.vistalink.adapter.spi.VistaLinkServerInfo;
import gov.va.med.vistalink.rpc.NoRpcContextFaultException;
import gov.va.med.vistalink.rpc.RpcNotInContextFaultException;
import gov.va.med.vistalink.rpc.RpcNotOkForProxyUseException;
import gov.va.med.vistalink.rpc.RpcRequest;
import gov.va.med.vistalink.rpc.RpcRequestFactory;
import gov.va.med.vistalink.rpc.RpcResponse;
import gov.va.med.vistalink.security.m.SecurityFaultException;

import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.resource.ResourceException;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.ConnectionSpec;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.springframework.dao.DataAccessException;
import org.springframework.dao.DataAccessResourceFailureException;
import org.springframework.dao.InvalidDataAccessApiUsageException;
import org.springframework.jca.cci.core.CciTemplate;
import org.springframework.jca.cci.core.ConnectionCallback;
import org.springframework.util.Assert;

/**
 * This class is intended to simplify the use of VistaLink RPCs to reduce or avoid common errors.
 * It executes core RPC workflow, leaving application code to provide RPC names and parameters and extract results.
 * This class executes RPCs with VistaLink, initiating iteration over RPC results and catching VistaLink exceptions 
 * and translating them to the generic data access exception hierarchy defined in the org.springframework.dao package.
 */

public class VistaLinkTemplate implements  VistaRpcUserDuzTemplate, VistaRpcAppProxyTemplate {

	public static final int USER_TYPE_DUZ = 1;
	public static final int USER_TYPE_APPLICATION_PROXY = 2;

	static final int DEFAULT_TIMEOUT = 90000;
	private Log logger = LogFactory.getLog( VistaLinkTemplate.class );
	private int timeOut = DEFAULT_TIMEOUT;
	private ConnectionFactoryLocator connectionFactoryLocator;
	private static final String UNABLE_TO_EXECUTE_RPC = "Unable to execute RPC";
	
	protected ConnectionFactory connectionFactory = null;
	CciTemplate cciTemplate = null;
	

	public VistaLinkTemplate(ConnectionFactoryLocator connectionFactoryLocator) {

		this.connectionFactoryLocator = connectionFactoryLocator;
	}

	public String callRpcAsUser(final String division, final String duz, final String rpcContext, final String rpcName)
			throws DataAccessException {
		return callRpcAsUser(division, duz, rpcContext, rpcName, (List<Object>) null);
	}

	public String callRpcAsUser(final String division, final String duz, final String rpcContext, final String rpcName,
			Object... params) throws DataAccessException {
		return callRpcAsUser(division, duz, rpcContext, rpcName, createParameterList(params));
	}

	public String callRpcAsUser(final String division, final String duz, final String rpcContext, final String rpcName,
			final List<Object> params) throws DataAccessException {
		return callRpc(new VistaLinkDuzConnectionSpec(division, duz), division, duz, rpcContext, rpcName, params);
	}

	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext,
			final String rpcName) throws DataAccessException {
		return callRpcAsApplication(division, applicationProxyName, rpcContext, rpcName, (List<Object>) null);
	}

	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext,
			final String rpcName, Object... params) throws DataAccessException {
		return callRpcAsApplication(division, applicationProxyName, rpcContext, rpcName, createParameterList(params));
	}

	public String callRpcAsApplication(final String division, final String applicationProxyName, final String rpcContext,
			final String rpcName, final List<Object> params) throws DataAccessException {
		return callRpc(new VistaLinkAppProxyConnectionSpec(division, applicationProxyName), division, applicationProxyName,
				rpcContext, rpcName, params);
	}

	public String callRpc(ConnectionSpec connectionSpec, final String division, final String user, final String rpcContext, final String rpcName, final List<Object> params) throws DataAccessException {

      if (logger.isDebugEnabled())
          logger.debug(MessageFormat.format("''{0}'' called in context ''{1}'' by ''{2}'' at facility ''{3}'' with params: {4}", new Object[]{rpcName, rpcContext, user, division, params}));

      	if(connectionFactory == null){
      		connectionFactory = connectionFactoryLocator.getConnectionFactory(division);
      	}

        try {

        	if(cciTemplate == null){
        		cciTemplate = new CciTemplate(connectionFactory, connectionSpec);
        	}
        	
            RpcResponse response = (RpcResponse) cciTemplate.execute(new ConnectionCallback<Object>() 
            {
            	public Object doInConnection(Connection connection, ConnectionFactory connectionFactory) throws ResourceException, SQLException, DataAccessException {
            		try{
            			Assert.isInstanceOf(VistaLinkConnection.class, connection);
                        VistaLinkConnection conn = (VistaLinkConnection) connection;
                        conn.setTimeOut(getTimeOut());
                        RpcRequest request = RpcRequestFactory.getRpcRequest(rpcContext, rpcName);
                        request.setUseProprietaryMessageFormat(true);
                        request.setXmlResponse(false);

                        if (params != null) {
                           request.setParams(params);
                        }

                        return conn.executeRPC(request);
                   } 
            		catch (IllegalArgumentException e) {
                        throw new VistaLinkDataAccessException(UNABLE_TO_EXECUTE_RPC, new VistaLinkFaultException(e.getMessage()));
                    } 
              		catch (NoRpcContextFaultException e) {
                        throw new VistaLinkDataAccessException(e);
              		} 
              		catch (RpcNotOkForProxyUseException e) {
                       throw new VistaLinkDataAccessException(e);
              		} 
              		catch (RpcNotInContextFaultException e) {
                        throw new VistaLinkDataAccessException(e);
              		} 
              		catch (LoginsDisabledFaultException e) {
                        throw new VistaLinkDataAccessException(e);
              		} 
              		catch (SecurityFaultException e) {
                        throw new VistaLinkDataAccessException(e);
              		} 
              		catch (VistaLinkFaultException e) {
                         throw new VistaLinkDataAccessException(e);
              		} 
              		catch (FoundationsException e) {
                        throw new VistaLinkDataAccessException(UNABLE_TO_EXECUTE_RPC, new VistaLinkFaultException(e.getMessage()));
                    }
            	}
            });

            String result = response.getResults();
            if (logger.isDebugEnabled()) {
            	logger.debug(MessageFormat.format("''{0}'' returned: {1}", new Object[]{rpcName, result}));
            }

         return result;

        } 
        catch (IllegalArgumentException e) {
        	throw new InvalidDataAccessApiUsageException(UNABLE_TO_EXECUTE_RPC, e);
        }
	}

	public int getTimeOut() {

		return timeOut;
	}

	public void setTimeOut(int timeOut) {

		this.timeOut = timeOut;

	}

	public ConnectionFactoryLocator getConnectionFactoryLocator() {

		return connectionFactoryLocator;
	}

	private List<Object> createParameterList(Object... params) {
		List<Object> paramList = new ArrayList<Object>();
		for (Object param : params) {
			paramList.add(param);
		}

		return paramList;
	}

	public VistaLinkServerInfo getConnectionInfo(final String division, final String user, final int userType)
			throws DataAccessException {

		VistaLinkConnection conn = null;
		ConnectionFactory connectionFactory = connectionFactoryLocator.getConnectionFactory(division);

		try {
			conn = (VistaLinkConnection) connectionFactory
					.getConnection(createConnectionSpec(division, user, userType));

		} catch (ResourceException e) {

			throw new DataAccessResourceFailureException("error getting connection info", e);

		}
		VistaLinkServerInfo vistaLinkServerInfo = conn.getConnectionInfo();
		return vistaLinkServerInfo;

	}

	private VistaLinkConnectionSpec createConnectionSpec(String division, String user, int userType) {
		switch (userType) {
		case USER_TYPE_APPLICATION_PROXY:
			return new VistaLinkAppProxyConnectionSpec(division, user);
		case USER_TYPE_DUZ:
		default:
			return new VistaLinkDuzConnectionSpec(division, user);

		}
	}

}
