package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.ClosestFacilityTO;
import gov.va.med.via.dto.common.NoteResultTO;
import gov.va.med.via.dto.common.PatientArray;
import gov.va.med.via.dto.common.PatientRecordFlagArray;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.StateArray;
import gov.va.med.via.dto.common.TaggedConsultArrays;
import gov.va.med.via.dto.common.TaggedHospitalLocationArray;
import gov.va.med.via.dto.common.TaggedInpatientStayArray;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedText;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedVisitArray;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserArray;
import gov.va.med.via.dto.common.UserTO;
import gov.va.med.via.dto.common.ZipcodeTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

@WebService
public interface CallServiceInterface extends BaseServiceInterface
{

	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
    /**
     * Lookup a CPRS-enabled user
     * 
     * @param queryBean the query bean
     * 
     * @return UserArray - list of users matching the target string
     * 
     * @mdws target queryBean.target
     * 
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a user name in LASTNAME, FIRSTNAME format
     */
    @WebMethod(operationName = "cprsUserLookup")
    public @WebResult(name="UserArray") UserArray cprsUserLookup(
       	@WebParam(name = "target") String target,
       	@WebParam(name = "queryBean") QueryBean queryBean        
    );

    /**
     * Match by name, city, state at logged in site.
     * 
     * @param name - patient name used to find patients
     * @param city - city used to find patients
     * @param stateAbbr state used to find patients
     * @param queryBean the query bean.
     *
     * @mdws name queryBean.patient.name
     * @mdws stateAbbr queryBean.criteria
     * @mdws city queryBean.target
     * 
     * @return TaggedPatientArray - list of matching patients found using teh name, city, state received if successful.
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.name should be the sought patient's name in LASTNAME,FIRSTNAME format
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a valid city name
     */
    @WebMethod(operationName = "matchByNameCityState")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray matchByNameCityState(
/*    	@WebParam(name = "name") String name,
        @WebParam(name = "city") String city,
        @WebParam(name = "stateAbbr") String stateAbbr,
*/        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Select a patient at logged in site. DFN is the IEN of the patient
     * 
     * @param queryBean the query bean
     * 
     * @returns PatientTO - patient record information
     * 
     * @mdws DFN queryBean.patient.localPid
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "select")
    public @WebResult(name="PatientTO") PatientTO select(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Write a note.
     * 
     * @return NoteResultTO - result of the write note
     * 
     *
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule When set, the prfIEN must be a valid IEN (a numeric or decimal value)
     * @autorule encounterString must be a valid encounter string
     * @autorule queryBean.patient.localPid must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule text cannot be a null value or an empty string
     * @autorule titleIEN must be a valid IEN (a numeric or decimal value)
     */
    @WebMethod(operationName = "writeNote")
    public @WebResult(name="NoteResultTO") NoteResultTO writeNote(
        @WebParam(name = "titleIEN") String titleIEN,
    	@WebParam(name = "encounterString") String encounterString,
        @WebParam(name = "text") String text,
    	@WebParam(name = "authorDUZ") String authorDUZ,
        @WebParam(name = "cosignerDUZ") String cosignerDUZ,        
    	@WebParam(name = "consultIEN") String consultIEN,
        @WebParam(name = "prfIEN") String prfIEN,
        @WebParam(name = "queryBean") QueryBean queryBean        
    );
    
    /**
     * Get patient record flag actions.
     * 
     * @param queryBean the queryBean
     *  
     * @return PatientRecordFlagArray - the record flags
     * 
     * @mdws noteDefintionIEN queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getPrfNoteActions")
    @WebResult(name="PatientRecordFlagArray")
    public PatientRecordFlagArray getPrfNoteActions(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Sign a note.
     * 
     *
     * @autorule esig cannot be a null value or an empty string
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "signNote")
    public @WebResult(name="TextTO") TextTO signNote(
    	@WebParam(name = "noteIEN") String noteIEN,
    	@WebParam(name = "userDUZ") String userDUZ, 
    	@WebParam(name = "esig") String esig,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get selected patient's admissions
     * 
     * @return TaggedInpatientStayArray - list of the patient's admissions if successful.
	 *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAdmissions")
    public @WebResult(name="TaggedInpatientStayArray") TaggedInpatientStayArray getAdmissions(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get selected patient's visits
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedVisitArray - list of visit events
     *
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getVisits")
    public @WebResult(name="TaggedVisitArray") TaggedVisitArray getVisits(
       	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a PRF note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - response indicating postive or negative
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */ 
    @WebMethod(operationName = "isPrfNote")
    public  @WebResult(name="TextTO") TextTO isPrfNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
  
    /**
     * Get patient's current consults.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedConsultArrays - list of consults
     * 
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConsultsForPatient")
    public @WebResult(name="TaggedConsultArrays") TaggedConsultArrays getConsultsForPatient(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a surgery note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedText - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @mdws noteTitle queryBean.target
     * @mdws visitString queryBean.criteria
     * 
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isOneVisitNote")
    public @WebResult(name="TaggedText") TaggedText isOneVisitNote(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a VistA's hospital locations (clinics, etc.).
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedHospitalLocationArray - list of locations
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     * 
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned hospital locations.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of matching hospital locations.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getLocations")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getLocations(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get note titles.
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedTextArray - list of note titles
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     * 
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned note titles.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of up to 44 matching note titles.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getNoteTitles")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getNoteTitles(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a surgery note?
     * 
     * @param queryBean the queryBean
     * 
     * @return TaggedText - positive or negative response. contains a "Yes" if the note is a surgery note, "No" otherwise, if successful. 
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * 
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isSurgeryNote")
    public @WebResult(name="TaggedText") TaggedText isSurgeryNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is given note a consult note?
     * 
     * VIA Increment 5
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @param queryBean the query bean
     * 
     * @return TextTO - contains a "Yes" if the note is a consult note, "No" otherwise, if successful.
     *  
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isConsultNote")
    public @WebResult(name="TextTO") TextTO isConsultNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is cosigner required?
     * 
     * @param queryBean the queryBean
     * 
     * @return TextTO - positive or negative response
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @mdws authorDUZ queryBean.criteria
     *
     * @autorule The optional queryBean.criteria value must be the valid, numeric DUZ of the provider in question.  When queryBean.criteria value is not defined, the logged in provider's user id will be used
     * @autorule The queryBean.itemId needs to be a numeric note definition ien (note definition id)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isCosignerRequired")
    public @WebResult(name="TextTO") TextTO isCosignerRequired(      
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Close a note.
     * 
     * @param noteIEN - note's IEN
     * @param consultIEN - consult's IEN
     * @param queryBean the query bean
     * 
     * @return TextTO - contains the result of closing the note if successful. 
     *
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "closeNote")
    public @WebResult(name="TextTO") TextTO closeNote(
    	@WebParam(name = "noteIEN") String noteIEN,
        @WebParam(name = "consultIEN") String consultIEN,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets the closest facility for the city and state received.
     * 
     * @param queryBean.criteria - state abbreviation to use to find the nearest facility
     * @param queryBean.target - city name used to find the nearest facility
     * @param queryBean the query bean.
     * 
     * @mdws stateAbbr queryBean.criteria
     * @mdws city queryBean.target
     * 
     * @return ClosestFacilityTO - closest facility for the city and state received if successful.
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a valid city name
     */
    @WebMethod(operationName = "getNearestFacility")
    public @WebResult(name="ClosestFacilityTO") ClosestFacilityTO getNearestFacility(
    	//@WebParam(name = "city") String city,
        //@WebParam(name = "stateAbbr") String stateAbbr,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets a list of all the cities in the state provided.
     *
     * @mdws stateAbbr queryBean.criteria
     *
     * @param queryBean the query bean.
     * 
     * @return ZipcodeTO[] - list of all the zipcode/cities in the state provided if successful.
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getCitiesInState")
    public @WebResult(name="ZipcodeTO") ZipcodeTO[] getCitiesInState(
        @WebParam(name = "queryBean") QueryBean queryBean        
    );

    /**
     * Match patients at logged in site.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArrays - a list of all the patients found that match the target received if successful.
     * 
     * @mdws target queryBean.target
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target target should be SSN or LAST,FIRST or LXXXX(first digit of last name followed by last four digits of SSN) pattern
     */
    @WebMethod(operationName = "match")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray match(
    	@WebParam(name = "queryBean") QueryBean queryBean        
    );
    
    /**
     * Lookup a patient from MPI.
     * 
     * @param queryBean.criteria should be a matching SSN
     * @param queryBean the query bean
     * @mdws ssn criteria
     * 
     * @return PatientArray -a patient array is returned containng a list of Patients found in MPI if successful.
     *
     *
     * @autorule queryBean.criteria must be a valid Social Security Number
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
 	@WebMethod(operationName = "mpiLookup")
    public @WebResult(name="PatientArray") PatientArray mpiLookup(
     	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Lookup a patient from NPT.
     * 
     * VIA Increment 8
     * 
     * @param ssn
     * @param queryBean the query bean
     * 
     * @return PatientArray -a patient array is returned containing a list of Patients found in NPT if successful.
     */
 	@WebMethod(operationName = "nptLookup")
    public @WebResult(name="PatientArray") PatientArray nptLookup(
    	@WebParam(name = "ssn") String ssn,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get VHA sites by states
     * 
     * VIA Increment 9
     * 
     * @param queryBean the query bean
     * 
     * @return StateArray -a list of the VHA sites by states if successful.
     */
 	@WebMethod(operationName = "getVhaByStates")
    public @WebResult(name="StateArray") StateArray getVhaByStates(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
}