package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.TaggedDemographicsRecordArrays;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedProstheticClaimArray;
import gov.va.med.via.dto.common.TaggedRatedDisabilityArray;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

/**
 * The ClaimSvcInterface is the Claim Service.  
 * This interface provides web services used to obtain claims-related data for patients. 
 * Users of this interface are intended to be healthcare providers. 
 */
@WebService
public interface ClaimSvcInterface extends BaseServiceInterface
{
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
    /**
     * Select a patient at logged in site. DFN is the IEN of the patient
     * 
     * @param queryBean the query bean
     * 
     * @returns PatientTO - patient record information
     * 
     * @mdws DFN queryBean.patient.localPid
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "select")
    public @WebResult(name="PatientTO") PatientTO select(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
	
    /**
     * Get consult note.
     * 
     * @param consultId
     * @param queryBean the query bean
     * 
     * @mdws consultId consultId
     * 
     * @return TextTO - contains the consult note found based on the consultId if successful.
     *
     * @autorule consultId must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConsultNote")
    public @WebResult(name="TextTO") TextTO getConsultNote(
    	@WebParam(name = "consultId") String consultId,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get patient's rated disabilities from local site.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedRatedDisabilityArrays - all rated disability records for a given patient, if successful.
     * 
     * @mdws dfn queryBean.patient.localPid
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getRatedDisabilitiesForPatient")
    public @WebResult(name="TaggedRatedDisabilityArray") TaggedRatedDisabilityArray getRatedDisabilitiesForPatient(
            @WebParam(name = "queryBean") QueryBean queryBean
    );
    /**
     * Get patient prosthetic claims from local site.  supplementalParameters is a list of
     * dates in the format Month/Day/Year, e.g. 091696 12704
     * 
     * @param queryBean the query bean
     * 
     * @mdws dfn queryBean.patient.localPid
     * @mdws dateList queryBean.supplementalParameters
     * 
     * @return TaggedProstheticClaimArray - list of the patient prosthetic claims from local site if successful. 
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.supplementalParameters cannot be a null value
     */
    @WebMethod(operationName = "getProstheticClaims")
    public @WebResult(name="TaggedProstheticClaimArray") TaggedProstheticClaimArray getProstheticClaims(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Match patients at logged in site.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArrays - a list of all the patients found that match the target received if successful.
     * 
     * @mdws target queryBean.target
     *
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target target should be SSN or LAST,FIRST or LXXXX(first digit of last name followed by last four digits of SSN) pattern
     */
    @WebMethod(operationName = "match")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray match(
    	@WebParam(name = "queryBean") QueryBean queryBean        
    );

    /**
     * Get claimants.
     * 
     * VIA Increment 8
     * 
     * @param lastName - last name
     * @param firstName - first name
     * @param middleName - middle name
     * @param dob - date of bith
     * @param zipcode - zip code
     * @param state - state
     * @param city - city
     * @param maxrex
     * @param queryBean the query bean
     * 
     * @return TaggedDemographicsRecordArrays - list of claimants if successful.
     */
    @WebMethod(operationName = "getClaimants")
    public TaggedDemographicsRecordArrays getClaimants(
    	@WebParam(name = "lastName") String lastName,
    	@WebParam(name = "firstName") String firstName,
    	@WebParam(name = "middleName") String middleName,
    	@WebParam(name = "dob") String dob,
    	@WebParam(name = "zipcode") String zipcode,
    	@WebParam(name = "state") String state,
    	@WebParam(name = "city") String city,
    	@WebParam(name = "maxrex") int maxrex,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Issue patient confidentiality bulletin to all connected sites.
     * 
     * @param queryBean the queryBean
     *  
     * @return TaggedTextArray 
     * 
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "issueConfidentialityBulletin")
    @WebResult(name="TaggedTextArray")
    public TaggedTextArray issueConfidentialityBulletin(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
}