package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.PatientArray;
import gov.va.med.via.dto.common.PatientLocationTO;
import gov.va.med.via.dto.common.RegionArray;
import gov.va.med.via.dto.common.SiteArray;
import gov.va.med.via.dto.common.SiteTO;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedPatientArrays;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

@WebService
public interface FindPatientServiceInterface extends BaseServiceInterface
{

	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
    /**
      * Match by name, city, state at logged in site.
     * 
     * @param name - patient name used to find patients
     * @param city - city used to find patients
     * @param stateAbbr state used to find patients

     * @mdws name queryBean.name
     * @mdws stateAbbr queryBean.criteria
     * @mdws city queryBean.target
     * 
     * @return TaggedPatientArray - list of matching patients found using teh name, city, state received if successful.
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.name should be the sought patient's name in LASTNAME,FIRSTNAME format
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a valid city name
     */
    @WebMethod(operationName = "matchByNameCityStateMS")
    public @WebResult(name="TaggedPatientArrays") TaggedPatientArrays matchByNameCityStateMS(
    	@WebParam(name = "name") String name,
    	@WebParam(name = "city") String city,
    	@WebParam(name = "stateAbbr") String stateAbbr,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get the facilities given a fips number.
     *  
     * @param queryBean the query bean
     * @mdws fips queryBean.criteria
     * 
     * @return SiteArray - list of sites if successful.
     *
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.criteria fips should be a numeric value
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getFacilitiesForCounty")
    public @WebResult(name="SiteArray") SiteArray getFacilitiesForCounty(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a site
     * 
     * VIA Increment 8
     * 
     * @param siteCode
     * @param queryBean the query bean  
     * @return SiteTO -a the site information if successful.
     */
    @WebMethod(operationName = "getSite")
    public SiteTO getSite(
    	@WebParam(name = "sitecode") String sitecode,
     	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
	 /**
     * Match by city and state at logged in site.
     * 
     * @param city
     * @param stateAbbr
     * @param queryBean the query bean
     * 
     * @mdws stateAbbr queryBean.criteria
     * @mdws city queryBean.target
     * 
     * @return TaggedTextArray list of city and states.
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a valid city name
     */
 	 @WebMethod(operationName = "matchCityAndState")
     public @WebResult(name="TaggedTextArray") TaggedTextArray matchCityAndState(
     	@WebParam(name = "city") String city,
     	@WebParam(name = "stateAbbr") String stateAbbr,
     	@WebParam(name = "queryBean") QueryBean queryBean
    );
 	
    /**
     * Lookup a patient from MPI.
     * 
     * @param queryBean the query bean
     *
     * @mdws ssn queryBean.criteria
     * 
     * @return PatientArray -a patient array is returned containing a list of Patients found in MPI if successful.
     *
     *
     * @autorule queryBean.criteria must be a valid Social Security Number
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
 	@WebMethod(operationName = "mpiLookup")
    public @WebResult(name="PatientArray") PatientArray mpiLookup(
     	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Lookup a patient from NPT.
     * 
     * VIA Increment 8
     * 
     * @param ssn
     * @param queryBean the query bean
     * 
     * @return PatientArray -a patient array is returned containing a list of Patients found in NPT if successful.
     */
 	@WebMethod(operationName = "nptLookup")
    public PatientArray nptLookup(
    	@WebParam(name = "ssn") String ssn,
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get all VHA sites
     * 
     * VIA Increment 9
     *      
     * @param queryBean the query bean
     * 
     * @return RegionArray - list of all regions found is returned if successful. 
     */
 	@WebMethod(operationName = "getVHA")
    public RegionArray getVHA(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get the VISNs in a given state.
     * 
     * @param queryBean the query bean
     * @mdws state queryBean.criteria
     * 
     * @return RegionArray - a list of Visns for the state if successful. 
     *
     *
     * @autorule queryBean.criteria must be a valid, 2-character US State abbreviation or valid US territory abbreviation
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
 	@WebMethod(operationName = "getVisnsForState")
    public @WebResult(name="RegionArray") RegionArray getVisnsForState(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Locate a patient in VHA.
     * 
     * VIA Increment 9
     *      
     * @param queryBean the query bean
     * 
     * @return String[] - a list of Visns for the state if successful. 
     **/
 	@WebMethod(operationName = "locatePatient")
    public PatientLocationTO locatePatient(
    	@WebParam(name = "siteCoden") String sitecode,
    	@WebParam(name = "dfn") String dfn,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

}