package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.InpatientStayTO;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.RegionArray;
import gov.va.med.via.dto.common.TaggedAdtArray;
import gov.va.med.via.dto.common.TaggedAdtArrays;
import gov.va.med.via.dto.common.TaggedDrgArrays;
import gov.va.med.via.dto.common.TaggedHospitalLocationArray;
import gov.va.med.via.dto.common.TaggedInpatientStayArrays;
import gov.va.med.via.dto.common.TaggedPatientArray;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedUserArrays;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

/**
 * 
 * The NumiServiceInterface is the National Utilization Management Integration Service.  
 * This service provides webservice methods that allow the user to track inpatient bed movements of patients.  
 * 
 * All access provided by this interface is read-only.
 *
 */
@WebService
public interface NumiServiceInterface extends BaseServiceInterface
{	
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * 
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;

    /**
     * Finds a user by partial name.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedUserArrays - list of TaggedUserArray of users found that match the target received if successful.
     * 
     * @mdws target queryBean.target
     * @mdws maxRex queryBean.maxRecords
     *
     *
     * @autorule queryBean.maxRecords needs to be numeric
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a user name in LASTNAME, FIRSTNAME format
     */
    @WebMethod(operationName = "userLookup")
    public @WebResult(name="TaggedUserArrays") TaggedUserArrays userLookup(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Match patients at logged in site using the target received.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedPatientArrays - list of patients that were found based on the target received, if successful.
     * 
     * @mdws target queryBean.target
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target target should be SSN or LAST,FIRST or LXXXX(first digit of last name followed by last four digits of SSN) pattern
     */
    @WebMethod(operationName = "match")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray match(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Select a patient at logged in site.
     * 
     * @param queryBean the query bean
     * 
     * @return PatientTO - patient found based on the dfn received, if successful.
     * 
     * @mdws dfn queryBean.patient.localPid
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId and queryBean.provider.loginSiteCode must have equal values
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "select")
    public @WebResult(name="PatientTO") PatientTO select(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );    
    
    /**
     * Get patient confidentiality from all connected sites.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of confidentiality from all the connected sites, if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConfidentiality")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getConfidentiality(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get Patient Movement discharge records for given patient.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedAdtArray - all patient discharge records for a given patient, if successful.
     * 
     * @mdws siteCode queryBean.patient.localSiteId
     * @mdws dfn queryBean.patient.localPid
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getInpatientDischarges")
    public @WebResult(name="TaggedAdtArray") TaggedAdtArray getInpatientDischarges(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Patient Movement records for selected patient.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedAdtArrays - the patient Movement records for selected patient
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getInpatientMoves")
    public @WebResult(name="TaggedAdtArrays") TaggedAdtArrays getInpatientMoves(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get Patient Movement records associated with a checkinId
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedAdtArrays - the patient Movement records associated with the checkinId received, if successful.
     * 
     * @mdws checkinId queryBean.itemId
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getInpatientMovesByCheckinId")
    public @WebResult(name="TaggedAdtArrays") TaggedAdtArrays getInpatientMovesByCheckinId(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

   /* //**
     * Use when switching patient lookup sites.
     * 
     * VIA Increment 4
     * 
     * @param pwd
     * @param siteList
     * @param userSiteCode
     * @param userName
     * @param duz
     * @param context
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - ??????
     *//*
    @WebMethod(operationName = "visit")
    public @WebResult(name="TaggedTextArray") TaggedTextArray visit(
    	@WebParam(name = "pwd") String pwd,
        @WebParam(name = "siteList") String sitelist,
        @WebParam(name = "userSiteCode") String userSitecode,
        @WebParam(name = "userName") String userName,
        @WebParam(name = "duz") String duz,
        @WebParam(name = "context") String context,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
*/
    /**
     * Get a VistA timestamp
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - a Vista timestamp if successful.
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getVistaTimestamps")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getVistaTimestamps(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get DRG records for connected site(s)
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedDrgArrays - a list of DRG records for connected site if successful.
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getDRGRecords")
    public @WebResult(name="TaggedDrgArrays") TaggedDrgArrays getDRGRecords(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get wards for connected site(s)
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedHospitalLocationArrays - a list of wards for connected site(s) if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getWards")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getWards(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets patient Movement records falling within given start and end dates
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedAdtArrays - a list of the patient Movement records falling within given start and end dates if successful.
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getInpatientMovesByDateRange")
    public @WebResult(name="TaggedAdtArrays") TaggedAdtArrays getInpatientMovesByDateRange(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets patient Movement records falling within given start and end dateTime. yyyyMMdd.HHmmss
     * 
     * VIA Increment 4
     * @param queryBean the query bean
     * 
     * @return TaggedAdtArrays - a list of the patient movement records falling within given start and end dateTime, if successful.
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     * @mdws interationLength queryBean.criteria
     *
     * @autorule queryBean.criteria Format must be dd.hhmmss or subset such as 1.0 for a day or .1 for an hour
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getInpatientMovesByDateTimeRange")
    public @WebResult(name="TaggedAdtArrays") TaggedAdtArrays getInpatientMovesByDateTimeRange(
        @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Get Patient Movement records associated with a checkinId
     *
     * @param queryBean the query bean
     * 
     * @return InpatientStayTO - a patient Movement records associated with a checkinId if successful.
     * 
     * @mdws checkinId queryBean.itemId
     *
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getStayMovements")
    public @WebResult(name="InpatientStayTO") InpatientStayTO getStayMovements(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets Patient Movement records falling within given start and end dateTime
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedInpatientStayArrays - the patient movement records falling within given start and end dateTime if successful.
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getStayMovementsByDateRange")
    public @WebResult(name="TaggedInpatientStayArrays") TaggedInpatientStayArrays getStayMovementsByDateRange(
        @WebParam(name = "queryBean") QueryBean queryBean    
    );

    /**
     * Get patient confidentiality from all connected sites.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - a list of the patient confidentiality from all connected sites if successful.
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "issueConfidentialityBulletin")
    public @WebResult(name="TaggedTextArray") TaggedTextArray issueConfidentialityBulletin(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets all selected patient movement records
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedInpatientStayArrays - a list of all selected patient movement records if successful.
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getStayMovementsByPatient")
    public @WebResult(name="TaggedInpatientStayArrays") TaggedInpatientStayArrays getStayMovementsByPatient(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get all VHA sites
     * 
     * VIA Increment 9
     *      
     * @param queryBean the query bean
     * 
     * @return RegionArray - list of all regions found is returned if successful. 
     */
 	@WebMethod(operationName = "getVHA")
    public @WebResult(name="RegionArray") RegionArray getVHA(
        @WebParam(name = "queryBean") QueryBean queryBean
    );
 	
 	@WebMethod(operationName="getVersion")
 	public String getVersion();
}