package gov.va.med.via.service;

import gov.va.med.via.common.validator.ValidationException;
import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.PatientMedicalRecordTO;
import gov.va.med.via.dto.common.PatientTO;
import gov.va.med.via.dto.common.SiteTO;
import gov.va.med.via.dto.common.Synopsis;
import gov.va.med.via.dto.common.TaggedAllergyArrays;
import gov.va.med.via.dto.common.TaggedAppointmentArrays;
import gov.va.med.via.dto.common.TaggedChemHemRptArrays;
import gov.va.med.via.dto.common.TaggedCytologyRptArrays;
import gov.va.med.via.dto.common.TaggedInpatientStayArray;
import gov.va.med.via.dto.common.TaggedMedicationArrays;
import gov.va.med.via.dto.common.TaggedMicrobiologyRptArrays;
import gov.va.med.via.dto.common.TaggedProblemArrays;
import gov.va.med.via.dto.common.TaggedRadiologyReportArrays;
import gov.va.med.via.dto.common.TaggedSurgeryReportArrays;
import gov.va.med.via.dto.common.TaggedSurgicalPathologyRptArrays;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedVisitArray;
import gov.va.med.via.dto.common.TaggedVitalSignSetArrays;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserTO;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

//@WebService
public interface PhrServiceInterface extends BaseServiceInterface
{
	
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * VIA Increment 1
     * 
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * @return UserTO profile if successful.
	 * @throws ValidationException 
     *
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
	@WebMethod(operationName = "login")
    public UserTO login(
        	@WebParam(name = "siteCode") String siteCode, 
        	@WebParam(name = "accessCode") String accessCode, 
        	@WebParam(name = "verifyCode") String verifyCode, 
        	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;
    
	/**
	 * Connects to a site
	 * 
	 * VIA Increment 1
	 * 
	 * @param pwd - Application's security phrase
	 * @param sitecode - Station # of the VistA site
	 * @param mpiPid - National patient identifier (ICN)
	 * 
	 * @return SiteTO: name, hostname, port, etc. of connected site</returns>
	 */
	@WebMethod(operationName = "connect")
	public SiteTO connect(
			@WebParam(name = "pwd") String pwd, 
        	@WebParam(name = "siteCode") String siteCode,         	
        	@WebParam(name = "mpiPid") String mpiPid 
	);
	
	/**
	 * Get all the patient's appointments from VistA sources.
	 * 
	 * VIA Increment 2
	 * 
	 * @param queryBean the query bean
     * 
     * @return TaggedAppointmentArrays - list of all patient's appointments if successful.
	 */
	@WebMethod(operationName = "getAppointments")
	public TaggedAppointmentArrays getAppointments(
		@WebParam(name = "queryBean") QueryBean queryBean
	); 

	/**
	 * Gets all the detailed MHV health summaries from VistA sources.
	 * 
	 * VIA Increment 2
	 * 
	 * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of detailed MHV healthe summaries if successful.
	 */
	@WebMethod(operationName = "getDetailedHealthSummary")
	public TaggedTextArray getDetailedHealthSummary(
		@WebParam(name = "queryBean") QueryBean queryBean
	);
	
    /**
     * Gets all the MHV health summaries from VistA sources.
     * 
     * VIA Increment 2 
     * 	
     * @param queryBean the query bean
	 * 
	 * @return TaggedTextArray the text array if successful.
     */
	@WebMethod(operationName = "getHealthSummary")
    public TaggedTextArray getHealthSummary(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
	
    /**
     * Gets Patient Medical Record
     * 
     * VIA Increment 3 
     * 
     * @param cert
     * @param patientId - the patient Id
     * @param queryBean the query bean
	 * 
	 * @return PatientMedicalRecordTO- the patient's medical record if successful.
     */
	@WebMethod(operationName = "getMedicalRecord")
    public PatientMedicalRecordTO getMedicalRecord(
    	@WebParam(name = "cert") byte[] cert, 
    	@WebParam(name = "patientId") String patientId,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /**
     * Gets all the patient's demographics from VistA sources.
     * 
     * VIA Increment 3 
     * 
     * @param queryBean the query bean
     * 
     * @return PatientTO - patinet information including demographics if successful.
     */
	@WebMethod(operationName = "getDemographics")
    public PatientTO getDemographics(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
	
    /**
     * Get selected patient's visits
     * 
     * VIA Increment 3
     * 
	 * @param fromDate
     * @param toDate
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedVisitArray - a list of the patient visits found if successful.
     *
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
	@WebMethod(operationName = "getVisits")
    public TaggedVisitArray getVisits(
    	@WebParam(name = "fromDate") String fromDate,
        @WebParam(name = "toDate") String toDate,
        @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's advance directives from VistA sources.
     * 
     * VIA Increment 3
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedTextArray - a list of the patient advance directives from Vista sources if successful.
     */
	@WebMethod(operationName = "getAdvanceDirectives")
    public TaggedTextArray getAdvanceDirectives(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
            @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's unit dose meds from VistA sources.
     * 
     * VIA Increment 3
     * 
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedMedicationArrays - a list of all the patient's unit dose meds from VistA sources if successful.
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     **/
	@WebMethod(operationName = "getUnitDoseMeds")
	public TaggedMedicationArrays getUnitDoseMeds(
            @WebParam(name = "queryBean") QueryBean queryBean
    );
	   
   /**
    * Gets the patient's radiology reports from VistA sources for a given time frame.
    *  
    * VIA Increment 3
    * 
    * @param fromDate
    * @param toDate
    * @param nRpts
    * @param queryBean the query bean
    * 
    * @return TaggedRadiologyReportArrays - the patient's radiology reports from VistA sources for a given time frame if successful
    */
	@WebMethod(operationName = "getRadiologyReports")
    public TaggedRadiologyReportArrays getRadiologyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
            @WebParam(name = "queryBean") QueryBean queryBean
    );	
    
    /**
    * Gets all the patient's allergies from VistA sources.
    * 
    * VIA Increment 3
    * 
	* @param queryBean the query bean	 
	* 
	* @return TaggedAllergyArrays - a list of all the patient's allergies from VistA sources if successful.
    */
    @WebMethod(operationName = "getAllergies")
    public TaggedAllergyArrays getAllergies(
		   @WebParam(name = "queryBean") QueryBean queryBean
	); 

    /**
     * Gets all the patient's admissions from VistA sources.
     * 
     * VIA Increment 4
     * 
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedInpatientStayArray - a list of all the patient's admissions from VistA sources if successful. 
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     **/
    @WebMethod(operationName = "getAdmissions")
    public TaggedInpatientStayArray getAdmissions(
    		 @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's cytology reports from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean 
	 * 
	 * @return TaggedTextArray - a list of all the patient's cytology reports from VistA sources if successful. 
     */
    @WebMethod(operationName = "getCytopathologyReports")
    public TaggedTextArray getCytopathologyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets all the patient's microbiology reports from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean	 
	 * 
	 * @return TaggedMicrobiologyRptArrays - a list of all the patient's microbiology reports from VistA sources if successful. 
     */
    @WebMethod(operationName = "getMicrobiologyReports")
    public TaggedMicrobiologyRptArrays getMicrobiologyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets all the patient's vital signs from VistA sources.
     * 
     * VIA Increment 4
     * 
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedVitalSignSetArrays - a list of all the patient's vital signs from VistA sources if successful. 
     */
    @WebMethod(operationName = "getVitalSigns")
    public TaggedVitalSignSetArrays getVitalSigns(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's immunizations from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean
	 * 
	 * @return TaggedTextArray - a list of all the patient's immunizations from VistA sources if successful. 
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.startDate should be before queryBean.endDate
     */
    @WebMethod(operationName = "getImmunizations")
    public TaggedTextArray getImmunizations(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
   
    /**
     * Gets all the patient's surgery reports from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedSurgeryReportArrays - all the patient's surgery reports from VistA sources if successful.
     */
    @WebMethod(operationName = "getSurgeryReports")
    public TaggedSurgeryReportArrays getSurgeryReports(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets gets all the patient's non-VA meds from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedMedicationArrays -list of all the patient's non-VA meds from VistA sources if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getOtherMeds")
	public TaggedMedicationArrays getOtherMeds(
			@WebParam(name = "queryBean") QueryBean queryBean
	); 

    /**
     * Gets all the patient's electron microscopy reports from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of all the patient's electron microscopy reports from VistA sources if successful.
     */
    @WebMethod(operationName = "getElectronMicroscopyReports")
    public TaggedTextArray getElectronMicroscopyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets the patient's surgical pathology lab results
     * 
     * VIA Increment 4
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean
     * 
     * @return TaggedSurgicalPathologyRptArrays - list of the patient's surgical pathology lab results if successful.
     */
    @WebMethod(operationName = "getSurgicalPathologyReports")
    public TaggedSurgicalPathologyRptArrays getSurgicalPathologyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Gets all the patient's IV meds from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedMedicationArrays - list of all the patient's IV meds from VistA sources if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getIvMeds")    
    public TaggedMedicationArrays getIvMeds(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );   
    
    /**
     * Gets all the patient's outpatient meds from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedMedicationArrays - list of all the patient's outpatient meds from VistA sources if successful.
     */
    @WebMethod(operationName = "getOutpatientMeds")   
    public TaggedMedicationArrays getOutpatientMeds(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets gets all the patient's outpatient meds profiles from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of all the patient's outpatient meds profiles from VistA sources if successful.
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getOutpatientRxProfile")
    public TaggedTextArray getOutpatientRxProfile(
    		@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's problem lists of a given type from VistA sources.
     * 
     * VIA Increment 4
     * 
     * @param type
     * @param queryBean the query bean
     * 
     * @return TaggedProblemArrays - list of all the patient's problem lists of a given type from VistA sources if successful.
     */
    @WebMethod(operationName = "getProblemList")
    public TaggedProblemArrays getProblemList(
    	@WebParam(name = "type") String type,
        @WebParam(name = "queryBean") QueryBean queryBean
    );        

    /**
     * Gets all the patient's outpatient, inpatient and non-VA meds from VistA sources.
     * 
     * VIA Increment 7
     * 
	 * @param queryBean the query bean	 
	 * 
	 * @return TaggedMedicationArrays - a list of all the patient's outpatient, inpatient and non-VA meds from VistA sources if successful.
     *
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.patient.mpiPid cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     **/
	@WebMethod(operationName = "getAllMeds")
	public TaggedMedicationArrays getAllMeds(
            @WebParam(name = "queryBean") QueryBean queryBean
    );
	
    /**
     * Gets all the patient's chem/hem lab results from VistA sources.
     * 
     * VIA Increment 7
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean
     * 
     * @return TaggedChemHemRptArrays - list of all the patient's chem/hem lab results from VistA sources if successful.
     */
    @WebMethod(operationName = "getChemHemReports")
    public TaggedChemHemRptArrays getChemHemReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all the patient's cytology reports from VistA sources.
     * 
     * VIA Increment 8
     * 
     * @param fromDate
     * @param toDate
     * @param nRpts
     * @param queryBean the query bean
     * 
     * @return TaggedCytologyRptArrays - list of all the patient's cytology reports from VistA sources if successful.
     */
    @WebMethod(operationName = "getCytologyReports")
    public TaggedCytologyRptArrays getCytologyReports(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,
            @WebParam(name = "nRpts") int nRpts,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets all of a patient's secure messages from the SM database
     * 
     * VIA Increment 8
     * 
     * @param patientId
     * @param message
     * @param queryBean the query bean
     * 
     * @return TextTO - contains all of a patient's secure messages from the SM database if successful.
     */
    @WebMethod(operationName = "getSecureMessages")
    public TextTO getSecureMessages(
        	@WebParam(name = "patientId") String patientId,
            @WebParam(name = "message") String message,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Gets a synopsis of the patient's data from VistA sources.
     * 
     * VIA Increment 8
     * 
     * @param fromDate
     * @param toDate
     * @param queryBean the query bean      
     * @return Synopsis - a synopsis of the patient's data from VistA sources if successful.
     */
    @WebMethod(operationName = "getSynopsis")
    public Synopsis getSynopsis(
        	@WebParam(name = "fromDate") String fromDate,
            @WebParam(name = "toDate") String toDate,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Writes a new Secure Message to the SM database
     * 
     * VIA Increment 8
     * 
     * @param patientId
     * @param message
     * @param queryBean the query bean
     * 
     * @return TextTO - contains the patient's secure messages from the SM database if successful.
     */
    @WebMethod(operationName = "writSecureMessage")
    public TextTO writeSecureMessage(
        	@WebParam(name = "patientId") String patientId,
            @WebParam(name = "message") String message,            
    	    @WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Authenticates a user with his/her MHV or DS Logon credentials
     * 
     * VIA Increment 8
     * 
     * @param cert
     * @param username
     * @param password
     * @param credentialType the credential type
     * 
     * @return UserTO - contains the user found if successful.
     */
    @WebMethod(operationName = "patientLogin")
    public UserTO patientLogin(
    		@WebParam(name = "cert") byte[] cert, 
    		@WebParam(name = "username") String username, 
    		@WebParam(name = "password") String password, 
    		@WebParam(name = "credentialType") String credentialType
    );
}