package gov.va.med.via.service;

import gov.va.med.via.dto.common.QueryBean;
import gov.va.med.via.dto.common.NoteResultTO;
import gov.va.med.via.dto.common.PersonsTO;
import gov.va.med.via.dto.common.TaggedConsultArrays;
import gov.va.med.via.dto.common.TaggedHospitalLocationArray;
import gov.va.med.via.dto.common.TaggedInpatientStayArray;
import gov.va.med.via.dto.common.TaggedTextArray;
import gov.va.med.via.dto.common.TaggedVisitArray;
import gov.va.med.via.dto.common.TextTO;
import gov.va.med.via.dto.common.UserArray;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

/**
 * The TbiServiceInterface is the Traumatic Brain Injury service.  
 * This interface provides access to Tramatic Brain injury patient data consult notes.  
 * The service also provides one write service method – writeNote.
 * 
 * The interface supports single site data access.
 * 
 * Users of this interface are intended to be healthcare providers. 
 * 
 */
@WebService
public interface TbiServiceInterface extends BaseServiceInterface
{
	
	 /**
     * Call when launching from CPRS Tools menu: connects, authenticates, selects patient.
     * 
     * @param pwd - password
     * @param siteCode - 
     * @param duz
     * @param dfn
     * @param queryBean the query bean
     * 
     * @return PersonsTO - list of consults for the patient if successful.
     * 
     * @mdws pwd queryBean.criteria
     * @mdws siteCode queryBean.patient.localSiteId
     * @mdws siteCode queryBean.provider.loginSiteCode
     * @mdws duz queryBean.provider.userId
     * @mdws dfn queryBean.patient.localPid
     *
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     */
    @WebMethod(operationName = "cprsLaunch")
    public @WebResult(name="PersonsTO") PersonsTO cprsLaunch(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Lookup a CPRS-enabled user
     * 
     * @param target
     * @param queryBean the query bean
     * 
     * @return UserArray - list of users found in CPRS.
     * 
     * @mdws target queryBean.target
     *
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target must be a user name in LASTNAME, FIRSTNAME format
     */
    @WebMethod(operationName = "cprsUserLookup")
    public @WebResult(name="UserArray") UserArray cprsUserLookup(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
	    
    /**
     * Get patient's current consults.
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedConsultArrays - list of consults for the patient if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getConsultsForPatient")
    public @WebResult(name="TaggedConsultArrays") TaggedConsultArrays getConsultsForPatient(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Is cosigner required for this user/author pair?
     *  
     * @param queryBean the query bean
     * 
     * @return TextTO
     * 
     * @mdws noteDefinitionIEN queryBean.itemId
     * @mdws authorDUZ queryBean.criteria
     *
     * @autorule The optional queryBean.criteria value must be the valid, numeric DUZ of the provider in question.  When queryBean.criteria value is not defined, the logged in provider's user id will be used
     * @autorule The queryBean.itemId needs to be a numeric note definition ien (note definition id)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "isCosignerRequired")
    public @WebResult(name="TextTO") TextTO isCosignerRequired(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
       
    /**
     * Get selected patient's admissions
     * 
     * VIA Increment 2
     * 
     * @param queryBean the query bean
     * 
     * @return TaggedInpatientStayArray - list of in-patient stay if successful.
     *
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getAdmissions")
    public @WebResult(name="TaggedInpatientStayArray") TaggedInpatientStayArray getAdmissions(
    	@WebParam(name = "queryBean") QueryBean queryBean);
    
    /**
     * Get a VistA's hospital locations (clinics, etc.).
     * 
     * VIA Increment 2
     * 
     * @param target
     * @param direction
	 * @param queryBean the query bean
     * 
     * @return TaggedHospitalLocationArray - list of hospital locations if successful.
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     *
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned hospital locations.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of matching hospital locations.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getLocations")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getLocations(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );   
  
    /**
     * Get selected patient's visits
     * 
     * VIA Increment 3
     * 
     * @param fromDate
     * @param toDate
	 * @param queryBean the query bean
     * 
     * @return TaggedVisitArray - list of patients if successful.
     * 
     * @mdws fromDate queryBean.startDate
     * @mdws toDate queryBean.endDate
     *
     * @autorule queryBean.endDate needs to have a valid UTC format: yyyymmdd
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.startDate needs to have a valid UTC format: yyyymmdd
     */
    @WebMethod(operationName = "getVisits")
    public @WebResult(name="TaggedVisitArray") TaggedVisitArray getVisits(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Get a VistA's TIU note titles.
     * 
     * @param target
     * @param direction
	 * @param queryBean the query bean
     * 
     * @return TaggedTextArray - list of note titles if successful.
     * 
     * @mdws target queryBean.target
     * @mdws direction queryBean.direction
     *
     * @autorule The optional queryBean.direction argument must have a value of '1' (ascending) or '-1' (descending). This defines the order of the returned note titles.
     * @autorule The queryBean.target can be empty but must be defined. The queryBean.target value enables the return of up to 44 matching note titles.
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "getNoteTitles")
    public @WebResult(name="TaggedTextArray") TaggedTextArray getNoteTitles(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );
    
    /**
     * Write a note.
     * 
     * VIA Increment 2
     * 
     * @param titleIEN
     * @param encounterString
     * @param text
     * @param authorDUZ
     * @param cosignerDUZ
     * @param consultIEN
     * @param prfIEN
	 * @param queryBean the query bean
     * 
     * @return NoteResultTO - 
     *
     *
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule When set, the prfIEN must be a valid IEN (a numeric or decimal value)
     * @autorule encounterString must be a valid encounter string
     * @autorule queryBean.patient.localPid must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule text cannot be a null value or an empty string
     * @autorule titleIEN must be a valid IEN (a numeric or decimal value)
     */
    @WebMethod(operationName = "writeNote")
    public @WebResult(name="NoteResultTO") NoteResultTO writeNote(
    	@WebParam(name = "titleIEN") String titleIEN,
    	@WebParam(name = "encounterString") String encounterString,
    	@WebParam(name = "text") String text,
    	@WebParam(name = "authorDUZ") String authorDUZ,
    	@WebParam(name = "cosignerDUZ") String cosignerDUZ,
    	@WebParam(name = "consultIEN") String consultIEN,
    	@WebParam(name = "prfIEN") String prfIEN,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /**
     * Close a note.
     * 
     * @param noteIEN
     * @param consultIEN
     * @param queryBean the query bean
     * 
     * @return TextTO
     *
     * @autorule When set, the consultIEN must be a valid IEN (a numeric or decimal value)
     * @autorule noteIEN must be a valid IEN (a numeric or decimal value)
     * @autorule queryBean.provider.userId must be a valid DUZ. It must be an integer or decimal value greater than 0
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     */
    @WebMethod(operationName = "closeNote")
    public @WebResult(name="TextTO") TextTO closeNote(
    	@WebParam(name = "noteIEN") String noteIEN,
    	@WebParam(name = "consultIEN") String consultIEN,
    	@WebParam(name = "queryBean") QueryBean queryBean
    );

    /*
	/**
     * Authenticates a provider against a local system.
     *
     * If successful, a qualified provider object will be returned.
     *
     * Otherwise, a provider object with an error message contained in the provider id will be returned.
     *
     * VIA Increment 1
     * 
     * @param siteCode  site associated with access and verify credentials.
     * @param accessCode provider's access code.
     * @param verifyCode provider's verify code.
     * @param requestingApp Application calling method.
     * @return Provider profile if successful.
	 * @throws ValidationException 
     *
     *
     * @autorule accessCode cannot be a null value or an empty string
     * @autorule requestingApp cannot be a null value or an empty string
     * @autorule siteCode cannot be a null value or an empty string
     * @autorule verifyCode cannot be a null value or an empty string
     **/
    /*@WebMethod(operationName = "login")
    public @WebResult(name="UserTO") UserTO login(
    	@WebParam(name = "siteCode") String siteCode, 
    	@WebParam(name = "accessCode") String accessCode, 
    	@WebParam(name = "verifyCode") String verifyCode, 
    	@WebParam(name = "requestingApp") String requestingApp
    ) throws ValidationException;*/
	
	/*
    /**
     * Is given note a consult note?
     * 
     * VIA Increment 1
     * 
     * @param queryBean the query bean - needs the noteDefinitionIEN in queryBean.itemId.
     * queryBean.patient and queryBean.provider must be populated.
     * 
     * @return TextTO containing a value of true in the text field if the note received is PRF, if successful.
     *
     */
   /* @WebMethod(operationName = "isConsultNote")
    public @WebResult(name="TextTO") TextTO isConsultNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
	/*
    /**
     * Is given note a prf note?
     * 
     * VIA Increment 1
     * 
     * @param queryBean the query bean - needs the noteDefinitionIEN in queryBean.itemId.
     * queryBean.patient and queryBean.provider must be populated.
     * 
     * @return TextTO containing a value of true in the text field if the note received is PRF, if successful.
     *
     */
    /*@WebMethod(operationName = "isPrfNote")
    public @WebResult(name="TextTO") TextTO isPrfNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
   /*
	/**
     * Is given note a surgery note?
     * 
     * VIA Increment 1
     * 
     * @param noteTitle - contains the title of the note used in the search
     * 
     * THE noteTitle ATTRIBUTE IS NOT USED! IT WAS JUST KEPT TO MATCH THE MDWS METHOD SIGNATURE
     * 
     * @param visitString - contains text for the patioent's visit associated to the note. 
     * @param queryBean the query bean - needs the noteDefinitionIEN in queryBean.itemId.
     * queryBean.patient and queryBean.provider must be populated.
     * 
     * @return TaggedText containing the response received from Vista that matches the criteria recieved, if successful.     
     *
     *
     * @autorule queryBean.criteria cannot be a null value or an empty string
     * @autorule queryBean.itemId needs to be numeric
     * @autorule queryBean.patient cannot be a null value
     * @autorule queryBean.patient.localPid cannot be a null value or an empty string
     * @autorule queryBean.patient.localSiteId cannot be a null value or an empty string
     * @autorule queryBean.provider cannot be a null value
     * @autorule queryBean.provider.loginSiteCode cannot be a null value or an empty string
     * @autorule queryBean.provider.name cannot be a null value or an empty string
     * @autorule queryBean.provider.userId cannot be a null value or an empty string
     * @autorule queryBean.recordSiteCode cannot be a null value or an empty string
     * @autorule queryBean.requestingApp cannot be a null value or an empty string
     * @autorule queryBean.target cannot be a null value or an empty string
     */
   /* @WebMethod(operationName = "isOneVisitNote")
    public @WebResult(name="TaggedText") TaggedText isOneVisitNote(
    	@WebParam(name = "noteTitle") String noteTitle,
    	@WebParam(name = "visitString") String visitString,
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    /*
    /**
     * Is given note a surgery note?
     * 
     * VIA Increment 1
     * 
     * @param queryBean the query bean - needs the noteDefinitionIEN in queryBean.itemId.
     * queryBean.patient and queryBean.provider must be populated.
     * 
     * @return TaggedText containing the response received from Vista, if successful.
     *
     */
    /*@WebMethod(operationName = "isSurgeryNote")
    public @WebResult(name="TaggedText") TaggedText isSurgeryNote(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
    /* 
    /**
     * Get the clinic's  locations
     * 
     * VIA Increment 2
     * 
     * @param target
     * @param queryBean the query bean
     * 
     * @return TaggedHospitalLocationArray - list of hospital locations if successful.
     *
     */
   /* @WebMethod(operationName = "getClinics")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getClinics(
    		@WebParam(name = "target")  String target,
        @WebParam(name = "queryBean") QueryBean queryBean);*/
    /*
    /**
     * Get a list of all in-patients currently at the ward provided
     * 
     * VIA Increment 2
     * 
     * Ward ID is stored at the queryBean.itemId
     * 
     *
     */
   /* @WebMethod(operationName = "getPatientsByWard")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByWard(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
    /*
    /**
     * Get a VistA's TIU note text using the note IEN received.
     * 
     * VIA Increment 2
     * 
	 * @param queryBean the query bean
     * 
     * @return TaggedTextArray - note text matching the note's IEN received if successful.
     *
     */
   /* @WebMethod(operationName = "getNoteText")
    public @WebResult(name="TextTO") TextTO getNoteText(
    	@WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
   /*
    /**
     * Get a list of all the in-patients currently at the clinic provided
     * 
     * VIA Increment 2
     * 
     * Ward ID is stored at the queryBean.itemId
     * 
     *
     */
   /*@WebMethod(operationName = "getPatientsByClinic")
    public @WebResult(name="TaggedPatientArray") TaggedPatientArray getPatientsByClinic(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/

    /* @WebMethod(operationName = "getWards")
    public @WebResult(name="TaggedHospitalLocationArray") TaggedHospitalLocationArray getWards(
        @WebParam(name = "queryBean") QueryBean queryBean
    );*/
    
   /* @WebMethod(operationName="signNote")
    public @WebResult(name="TextTO")TextTO signNote(
    	@WebParam(name="noteIEN") String noteIEN, 
    	@WebParam(name="esig") String esig, 
    	@WebParam(name = "queryBean") QueryBean queryBean) throws ValidationException;    */
    
    
    
}