/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import gov.va.med.via.rpc.site.DataSource;
import gov.va.med.via.rpc.site.DataSourceUtils;
import gov.va.med.via.rpc.site.SiteException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.LinkedHashMap;

/**
 *
 * @author vhahonbrunos
 */
public class RPCConnection
{
    private DataSource dataSource;
    private boolean isVisitCxn;

    private String errorMsg;
    private String uid;
    private String dfn;

    private static final int CONNECTION_TIMEOUT = 30000;
  //  private static final int DEFAULT_PORT = 0000;

    private Socket socket;
    private String currentContext;

    private LinkedHashMap lookupTables;

    public RPCConnection()
    {
        lookupTables = new LinkedHashMap();
    }

    public RPCConnection(DataSource dataSource)
    {
        this.dataSource = dataSource;
        lookupTables = new LinkedHashMap();
    }

    public RPCConnection(String siteId) throws SiteException
    {
        this.dataSource = DataSourceUtils.getDataSourceBySiteId(siteId);
        lookupTables = new LinkedHashMap();
    }
    
    /*
    public Object getDao(String daoName)
    {
        DaoFactory df = DaoFactory.getDaoFactory(DaoFactory.getConstant(dataSource.Protocol));
        return df.getDaoByName(daoName, this);
    }
    */

    public KeyValuePair getSiteId()
    {
        return dataSource.getSiteId();
    }

    public Socket getRPCSocket() 
    {
        return socket;
    }

    public void connect() throws RPCException
    {
        try
        {
            if(isConnected())
            {
                return;
            }
            if (dataSource == null)
            {
                throw new RPCException("No data source");
            }
            String hostname = dataSource.getProvider();
            if (hostname.isEmpty())
            {
                throw new RPCException("No provider (hostname)");
            }
            int vistaPort = dataSource.getPort();
            if (-1 == vistaPort)
            {
                throw new RPCException("DataSource doesn't have a valid VistA broker:" + dataSource.getProvider());
            }
            ServerSocket listener = new ServerSocket(0, 2, InetAddress.getLocalHost());
            listener.setSoTimeout(CONNECTION_TIMEOUT);
            //Build the request message
            String request = "TCPconnect^" + listener.getLocalSocketAddress().toString() + '^' + listener.getLocalPort() + '^' + hostname;
            request = "{XWB}" + RPCStringUtils.strPack(request, 5);
            try
            {
                //make socket connection
                socket = new Socket();
                socket.connect(new InetSocketAddress(hostname, vistaPort), CONNECTION_TIMEOUT);
                if (!socket.isConnected())
                {
                    listener.close();
                    throw new RPCException("Socket was unable to connect to " + hostname + ":" + vistaPort);
                }
            }
            catch (RPCException e)
            {
                throw new RPCException(e.getMessage() + ". Tried " + hostname + ":" + vistaPort, e.getCause());
            }
            String reply = (String) query(request);
            if (!reply.equals("accept"))
            {
                listener.close();
                socket.close();
                throw new RPCException("Unaccepted by " + hostname);
            }
            socket.close();
            socket = listener.accept();
            socket.setSoTimeout(CONNECTION_TIMEOUT);
            listener.close();
            //isConnected = true;
        }
        catch (Exception ex)
        {            
            throw new RPCException(ex);
        }
    }

    public void disconnect() throws RPCException
    {
        if(!isConnected())
            return;
        
        try
        {            
            String msg = RPCStringUtils.strPack(RPCStringUtils.strPack("#BYE#", 5), 5);
            msg = (String) query(msg);
            //todo msg should contain "BYE" - do anything if it doesn't?
            socket.close();
        }
        catch (Exception ex)
        {  
            throw new RPCException(ex);
        }
    }

    public String getCurrentContext()
    {
        return currentContext;
    }

    public void setCurrentContext(String currentContext)
    {
        this.currentContext = currentContext;
    }

    public VistaResponse query(VistaQuery vq) throws RPCException, IllegalStateException
    {
        String result = (String) query(vq.buildMessage());
        return new VistaResponse(result);
    }

    public Object query(String request) throws RPCException
    {
        Object result = null;

        /**
         * If the socket is not connected then connect; don't worry
         * about authorization as a program could perform a simple
         * rpc query such as get welcome message which doesn't require
         * login.
         */
        boolean disconnectAfterQuery = false;
        if(!isConnected())
        {
            disconnectAfterQuery = true;
            connect();
        }

        try
        {
            result = queryRequest(request);
        }
        catch (RPCException ex)
        {
            //retry query on re-authentication errors
            String exMsgRetry = "You are re-authenticating an already authenticated session";
            if (!ex.getMessage().contains(exMsgRetry))
            {
                throw ex;
            }
            else
            {
                boolean isRetry = true;
                int retryCnt = 0;
                while(isRetry && retryCnt < 2)
                {
                    isRetry = false;
                    this.disconnect();
                    this.connect();
                    try
                    {
                        result = queryRequest(request);
                    }
                    catch(RPCException ex2)
                    {
                        if (!ex2.getMessage().contains(exMsgRetry) || retryCnt >= 2)
                        {
                            throw ex2;
                        }
                        else
                        {
                            isRetry = true;
                            retryCnt++;
                        }
                    }
                }
            }
        }

        if(disconnectAfterQuery)
            disconnect();
        return result;
    }

    private Object queryRequest(String request) throws RPCException
    {
        OutputStream out = null;
        try
        {
            byte[] bytesSent = request.getBytes();
            byte[] bytesReceived = new byte[256];
            out = socket.getOutputStream();
            out.write(bytesSent);
            int bytes = 0;
            String reply = "";
            boolean isHdr = true;
            boolean isErrorMsg = false;
            int endIdx = -1;
            do
            {
                InputStream inpt = socket.getInputStream();
                bytes = inpt.read(bytesReceived, 0, bytesReceived.length);
                String thisBatch = new String(bytesReceived, 0, bytes);
                char eot = 0x04;
                endIdx = thisBatch.indexOf(eot);
                if (endIdx != -1)
                {
                    thisBatch = RPCStringUtils.CSubstring(thisBatch, 0, endIdx);
                }
                if (isHdr)
                {
                    if (bytesReceived[0] != 0)
                    {
                        String msg = RPCStringUtils.CSubstring(thisBatch, 1, bytesReceived[0]);
                        if (msg.startsWith("The remote procedure XUS SIGNON SETUP is not registered to the option XUS SIGNON."))
                        {
                            msg = "You are re-authenticating an already authenticated session";
                        }
                        thisBatch = "Security Error: " + msg;
                        isErrorMsg = true;
                    }
                    else if (bytesReceived[1] != 0)
                    {
                        thisBatch = "Application Error: " + thisBatch.substring(2);
                        isErrorMsg = true;
                    }
                    else
                    {
                        thisBatch = thisBatch.substring(2);
                    }
                    isHdr = false;
                }
                reply += thisBatch;
            }
            while (endIdx == -1);
            if (isErrorMsg || reply.contains("M  ERROR"))
            {
                reply += " (This message has come directly from " + this.dataSource.getProvider() + ").";
                throw new RPCException(reply);
            }
            return reply;
        }
        catch (IOException ex)
        {
            throw new RPCException(ex);
        }
    }

    public boolean setContext(String value)
    {
        /*VistaUserDao dao = new VistaUserDao(this);
        boolean result = dao.authorize(value);
        if (result)
        {
                currentContext = value;
        }
        return result;*/
        currentContext = value;
        //return false;
        return true;
    }

    public LinkedHashMap<String, String> getLookupTable(String fileNum) 
            throws RPCException
    {
        if (!lookupTables.containsKey(fileNum))
        {
            DdrLister query = DdrLister.buildIenNameQuery(this, fileNum);
            String[] response = query.execute();
            lookupTables.put(fileNum, RPCUtils.toStringDictionary(response));
        }
        return (LinkedHashMap<String,String>)lookupTables.get(fileNum);
    }


    public DataSource getDataSource()
    {
        return dataSource;
    }

    public void setDataSource(DataSource dataSource)
    {
        this.dataSource = dataSource;
    }

    public String getErrorMsg()
    {
        return errorMsg;
    }

    public void setErrorMsg(String errorMsg)
    {
        this.errorMsg = errorMsg;
    }

    public boolean isConnected()
    {
        if(socket != null)
        {
            if(socket.isConnected() && socket.isBound() && !socket.isClosed())
            {
                return true;
            }
        }
        
        return false;
    }

    public String getDFN()
    {
        return dfn;
    }

    public void setDFN(String dfn)
    {
        this.dfn = dfn;
    }

    public String getUid()
    {
        return uid;
    }

    public void setUid(String uid)
    {
        this.uid = uid;
    }

    public boolean isVisitCxn()
    {
        return isVisitCxn;
    }

    public void setIsVisitCxn(boolean visitCxn)
    {
        this.isVisitCxn = visitCxn;
    }

}
