/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import gov.va.med.via.rpc.site.DataSource;
import gov.va.med.via.rpc.site.Site;
import gov.va.med.via.rpc.site.SiteTable;
import java.util.LinkedHashMap;
import java.util.Iterator;

/**
 *
 * @author vhahonbrunos
 */
public final class RPCConnectionManager
{
    private LinkedHashMap cxnTbl;
    private String modality;
    private static long THREAD_TIMEOUT = 30000;

    public RPCConnectionManager(String modality)
    {
        this.modality = modality;
        cxnTbl = new LinkedHashMap();
    }
   
    public RPCConnectionManager(RPCConnection cxn) throws RPCException
    {
        this.modality = cxn.getDataSource().getModality();
        cxnTbl = new LinkedHashMap();
        addConnection(cxn);
    }

    public RPCConnectionManager(DataSource src) throws RPCException
    {
        this.modality = src.getModality();
        RPCConnection cxn = new RPCConnection(src);
        cxnTbl = new LinkedHashMap();
        addConnection(cxn);
    }

    public RPCConnectionManager(DataSource[] sources) throws RPCException
    {
        this.modality = sources[0].getModality();
        cxnTbl = new LinkedHashMap();
        for(DataSource src : sources)
        {
            addConnection(new RPCConnection(src));
        }
    }

    private void checkModality(RPCConnection cxn) throws RPCException
    {
        checkModality(cxn.getDataSource().getModality());
    }
/*
    private void checkModality(DataSource src) throws RPCException
    {
        checkModality(src.getModality());
    }
*/
    private void checkModality(String modality) throws RPCException
    {
        if (!modality.equals(this.modality))
        {
            throw new RPCException("Invalid modality");
        }
    }

    public void addConnection(RPCConnection cxn) throws RPCException
    {
        if (cxnTbl == null)
        {
            cxnTbl = new LinkedHashMap();
            this.modality = cxn.getDataSource().getModality();
        }
        checkModality(cxn);
        if (!cxnTbl.containsKey(cxn.getSiteId().getKey()))
        {
            cxnTbl.put(cxn.getSiteId().getKey(), cxn);
        }
    }

    public void removeConnection(String siteId) throws RPCException
    {
        if (cxnTbl.containsKey(siteId))
        {
            RPCConnection cxn = (RPCConnection)cxnTbl.get(siteId);
            if (cxn.isConnected())
            {
                cxn.disconnect();
            }
            cxnTbl.remove(siteId);
        }
    }

    public void addConnections(String sitelist, SiteTable siteTbl) throws RPCException
    {
        String[] siteIds = RPCStringUtils.split(sitelist, RPCStringUtils.COMMA);
        for (int i = 0; i < siteIds.length; i++)
        {
            Site site = siteTbl.getSite(siteIds[i]);
            if (site == null)
            {
                throw new RPCException("No such site: " + siteIds[i]);
            }
            DataSource src = site.getDataSourceByModality(this.modality);
            if (src == null)
            {
                throw new RPCException("No " + modality + " data source at site " + siteIds[i]);
            }
            
            addConnection(new RPCConnection(src));
        }
    }

    public void addConnections(DataSource[] sources) throws RPCException
    {
        for (int i = 0; i < sources.length; i++)
        {
            addConnection(new RPCConnection(sources[i]));
        }
    }

    public void addConnections(LinkedHashMap t) throws RPCException
    {
        for (int i = 0; i < t.size(); i++)
        {
            RPCConnection cxn = (RPCConnection)t.get(i);
            addConnection(cxn);
        }
    }

    public LinkedHashMap connect() throws RPCException
    {
        int lth = cxnTbl.size();
        QueryThread[] queries = new QueryThread[lth];
        Thread[] threads = new Thread[lth];
        int i = 0;
        Iterator cxnIter = cxnTbl.values().iterator();
        while(cxnIter.hasNext())
        {
            queries[i] = new QueryThread((RPCConnection) cxnIter.next(), "connect", new Object[0]);
            threads[i] = new Thread(queries[i]);
            threads[i].start();
            i++;
        }

        LinkedHashMap result = new LinkedHashMap(lth);
        Object[] keys = cxnTbl.keySet().toArray();
        for (i = 0; i < lth; i++)
        {
            String key = (String) keys[i];
            try
            {
                threads[i].join(THREAD_TIMEOUT);
                if (threads[i].isAlive())
                {
                    String timeoutStr = "RPC Connection Timeout. Connection time exceeded " + THREAD_TIMEOUT + "ms";
                    result.put(key, new RPCException(timeoutStr));
                    RPCConnection cxn = (RPCConnection) cxnTbl.get((String)keys[i]);
                    cxn.setErrorMsg(timeoutStr);
                }
                //Need to report result whether it's a connection or an exception.
                else if(queries[i].isExceptionResult())
                {
                    result.put(key, queries[i].getResult());
                    RPCConnection cxn = (RPCConnection) cxnTbl.get((String)keys[i]);
                    cxn.setErrorMsg(((Exception) queries[i].getResult()).getMessage());
                }
                else
                {
                    result.put(key, ((RPCConnection) cxnTbl.get(key)).getDataSource());
                }
            }
            catch (InterruptedException ex)
            {
                result.put(key, new RPCException(ex));
                RPCConnection cxn = (RPCConnection) cxnTbl.get((String)keys[i]);
                cxn.setErrorMsg(ex.getMessage());
            }
        }

        return result;
    }

    public LinkedHashMap getConnections()
    {
        LinkedHashMap result = new LinkedHashMap();
        Iterator cxnIter = cxnTbl.values().iterator();
        while(cxnIter.hasNext())
        {
            RPCConnection cxn = (RPCConnection)cxnIter.next();
            //if (cxn.isConnected())
            //{
                result.put(cxn.getSiteId().getKey(), cxn);
            //}
        }

        if (result.size() == 0)
        {
            return null;
        }
        return result;
    }

    public RPCConnection getConnection(String id)
    {
        if (cxnTbl == null || !cxnTbl.containsKey(id))
        {
            return null;
        }
        return (RPCConnection)cxnTbl.get(id);
    }

    public RPCConnection loginConnection()
    {
        return (RPCConnection)cxnTbl.get(cxnTbl.keySet().toArray()[0]);
    }

    public LinkedHashMap disconnect() throws RPCException
    {
        if (cxnTbl.size() == 0)
        {
            throw new RPCException("No connections");
        }

        // Only disconnect from the ones that are connected
        LinkedHashMap myCxns = getConnections();

        int lth = myCxns.size();
        QueryThread[] queries = new QueryThread[lth];
        Thread[] threads = new Thread[lth];
        Iterator cxnIter = myCxns.values().iterator();
        int i = 0;
        while(cxnIter.hasNext())
        {
            queries[i] = new QueryThread((RPCConnection)cxnIter.next(), "disconnect", new Object[0]);
            threads[i] = new Thread(queries[i]);
            threads[i].start();
            i++;
        }

        LinkedHashMap result = new LinkedHashMap(lth);
        Object[] keys = myCxns.keySet().toArray();
        for (i = 0; i < lth; i++)
        {
            String key = (String) keys[i];
            
            try
            {
                threads[i].join(THREAD_TIMEOUT);
                if (threads[i].isAlive())
                {
                    String timeoutStr = "RPC Connection Timeout. Connection time exceeded " + THREAD_TIMEOUT + "ms";
                    result.put(key, new RPCException(timeoutStr));
                }
                //Need to report result whether it's a connection or an exception.
                else if (queries[i].isExceptionResult())
                {
                    result.put(key, queries[i].getResult());
                }
                else
                {
                    result.put(key, "OK");
                }
            }
            catch (InterruptedException ex)
            {
                result.put(key, new RPCException(ex));
            }
        }

        // We only disconnected from connected connections, but we want to clear
        // the entire table
        cxnTbl = new LinkedHashMap();
        return result;
    }

    public LinkedHashMap disconnectRemoteSites() throws RPCException
    {
        if (cxnTbl.size() < 2)
        {
            throw new RPCException("No remote connections");
        }

        // Only disconnect from the ones that are connected
        LinkedHashMap myCxns = getConnections();

        int lth = myCxns.size() - 1;
        QueryThread[] queries = new QueryThread[lth];
        Thread[] threads = new Thread[lth];
        Object[] keys = myCxns.keySet().toArray();
        for (int threadIdx = 0, cxnIdx = 1; threadIdx < lth; threadIdx++, cxnIdx++)
        {
            queries[threadIdx] = new QueryThread((RPCConnection) myCxns.get(keys[cxnIdx]), "disconnect", new Object[0]);
            threads[threadIdx] = new Thread(queries[threadIdx]);
            threads[threadIdx].start();
        }
        LinkedHashMap result = new LinkedHashMap(lth);
        for (int threadIdx = 0, cxnIdx = 1; threadIdx < lth; threadIdx++, cxnIdx++)
        {
            String key = (String) myCxns.get(keys[cxnIdx]);
            
            try
            {
                threads[threadIdx].join(THREAD_TIMEOUT);
                if (threads[threadIdx].isAlive())
                {
                    String timeoutStr = "RPC Connection Timeout. Connection time exceeded " + THREAD_TIMEOUT + "ms";
                    result.put(key, new RPCException(timeoutStr));
                }
                //Need to report result whether it's a connection or an exception.
                else if (queries[threadIdx].isExceptionResult())
                {
                    result.put(key, queries[threadIdx].getResult());
                }
                else
                {
                    result.put(key, "OK");
                }
            }
            catch (InterruptedException ex)
            {
                result.put(key, new RPCException(ex));
            }
        }

        // Now remove all but the logon connection from the table
        RPCConnection loginCxn = loginConnection();
        cxnTbl = new LinkedHashMap();
        cxnTbl.put(loginCxn.getSiteId().getKey(), loginCxn);
        return result;
    }

    public String getModality()
    {
        return modality;
    }

    public boolean isAuthorized()
    {
        if (cxnTbl == null || cxnTbl.size() == 0)
        {
            return false;
        }
        RPCConnection cxn = (RPCConnection)cxnTbl.get(0);
        if (!cxn.isConnected() || cxn.getUid().equals(""))
        {
            return false;
        }
        return true;
    }

    public boolean isAuthorizedForSite(String siteCode)
    {
        if (!isAuthorized())
        {
            return false;
        }
        RPCConnection cxn = getConnection(siteCode);
        if (cxn == null || !cxn.isConnected() || cxn.getUid().equals(""))
        {
            return false;
        }
        return true;
    }
}
