/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.via.rpc;

import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 *
 * @author vhahonbrunos
 */
public class Timestamp
{
    private static final int yrIdx = 0;
    private static final int yrLth = 3;
    private static final int moIdx = 3;
    private static final int moLth = 2;
    private static final int dyIdx = 5;
    private static final int dyLth = 2;
    private static final int dotIdx = 7;
    private static final int dotLth = 1;
    private static final int hrIdx = 8;
    private static final int hrLth = 2;
    private static final int mnIdx = 10;
    private static final int mnLth = 2;
    private static final int scIdx = 12;
    //private static final int scLth = 2;

    public static String getDatePart(String ts)
    {
        return RPCStringUtils.CSubstring(padZeroes(ts), yrIdx,yrLth);
    }

    public static String getTimePart(String ts)
    {
        return padZeroes(ts).substring(hrIdx);
    }

    public static int getYear(String ts)
    {
        return Integer.parseInt(RPCStringUtils.CSubstring(padZeroes(ts), yrIdx,yrLth)) + 1700;
    }

    public static int getMonth(String ts)
    {
        return Integer.parseInt(RPCStringUtils.CSubstring(padZeroes(ts), moIdx,moLth));
    }

    public static int getDay(String ts)
    {
        return Integer.parseInt(RPCStringUtils.CSubstring(padZeroes(ts), dyIdx,dyLth));
    }

    public static int getHour(String ts)
    {
        return Integer.parseInt(RPCStringUtils.CSubstring(padZeroes(ts), hrIdx,hrLth));
    }

    public static int getMinute(String ts)
    {
        return Integer.parseInt(RPCStringUtils.CSubstring(padZeroes(ts), mnIdx,mnLth));
    }

    public static int getSecond(String ts)
    {
        return Integer.parseInt(padZeroes(ts).substring(scIdx));
    }

    public static String padZeroes(String ts)
    {
        if (ts.equals("*SENSITIVE*"))
        {
            return ts;
        }
        if (ts.indexOf("@") != -1)
        {
            ts = ts.replace('@','.');
        }
        if (ts.indexOf(".") != -1)
        {
            String[] parts = RPCStringUtils.split(ts, ".");
            ts = parts[0] + RPCStringUtils.CSubstring("0000000", 0, 7 - parts[0].length()) + '.' +
                 parts[1] + RPCStringUtils.CSubstring("000000", 0, 6 - parts[1].length());
        }
        else
        {
            ts = ts + RPCStringUtils.CSubstring("0000000", 0,7-ts.length()) + ".000000";
        }
        return ts;
    }

    public static boolean isValid(String ts)
    {
        ts = padZeroes(ts);
        int yr = getYear(ts);
        if (yr < 1900)
        {
            return false;
        }
        int mo = getMonth(ts);
        if (mo > 12)
        {
            return false;
        }
        int dy = getDay(ts);
        if (mo > 0 && dy > 0 && !DateUtils.isValidDay(yr,mo,dy))
        {
            return false;
        }
        if (getHour(ts) > 23)
        {
            return false;
        }
        if (getMinute(ts) > 59)
        {
            return false;
        }
        if (getSecond(ts) > 59)
        {
            return false;
        }
        return true;
    }

    public static String corrected(String ts)
    {
        if (ts.equals("*SENSITIVE*"))
        {
            return ts;
        }
        ts = padZeroes(ts);
        int yr = getYear(ts);
        if (yr < 1900)
        {
            return "";
        }
        int mo = getMonth(ts);
        if (mo == -1 || mo > 12)
        {
            ts = RPCStringUtils.CSubstring(ts, yrIdx,yrLth) + "00" + ts.substring(dyIdx);
            mo = 0;
        }
        int dy = getDay(ts);
        if (dy == -1 || !DateUtils.isValidDay(yr,mo,dy))
        {
            ts = RPCStringUtils.CSubstring(ts, yrIdx,yrLth+moLth) + "00" + ts.substring(dotIdx);
            dy = 0;
        }
        int hr = getHour(ts);
        if (hr == -1 || hr > 23)
        {
            ts = RPCStringUtils.CSubstring(ts, yrIdx,yrLth+moLth+dyLth+dotLth) + "00" + ts.substring(mnIdx);
        }
        int mn = getMinute(ts);
        if (mn == -1 || mn > 59)
        {
            ts = RPCStringUtils.CSubstring(ts, yrIdx,yrLth+moLth+dyLth+dotLth+hrLth) + "00" + ts.substring(scIdx);
        }
        int sc = getSecond(ts);
        if (sc == -1 || sc > 59)
        {
            ts = RPCStringUtils.CSubstring(ts, yrIdx,yrLth+moLth+dyLth+dotLth+hrLth+mnLth) + "00";
        }
        return ts;
    }

    public static String toUtcString(String vistaTS)
    {
        if (vistaTS.equals("*SENSITIVE*"))
        {
            return vistaTS;
        }
        String correctedTS = corrected(vistaTS);
        if (correctedTS.equals(""))
        {
            return "";
        }
        int yr = getYear(correctedTS);
        String result = "" + yr;
        result += correctedTS.substring(moIdx);
        return result;
    }

    public static Calendar toDateTimeFromRdv(String rdvTS) throws RPCException
    {
        String[] parts = RPCStringUtils.split(rdvTS, RPCStringUtils.SPACE);
        String[] dateFlds = RPCStringUtils.split(parts[0], RPCStringUtils.SLASH);

        // TBD - should we do this? Technically '00' is an invalid value for the day and should be fixed at that VistA
        // sometimes we get '00' as the day value from VistA - we change this to '01' to create our DateTime object
        if (dateFlds[1].equals("0") || dateFlds[1].equals("00"))
        {
            dateFlds[1] = "01";
        }

        if (parts.length == 1)
        {
            return new GregorianCalendar(
                //year
                Integer.parseInt(dateFlds[2]),
                //month
                Integer.parseInt(dateFlds[0])-1,
                //day
                Integer.parseInt(dateFlds[1]),
                //hour
                0,
                //minute
                0,
                //second
                0);
        }
        else
        {
            String[] timeFlds = RPCStringUtils.split(parts[1], RPCStringUtils.COLON);
            try
            {
                return new GregorianCalendar(
                //year
                Integer.parseInt(dateFlds[2]),
                //month
                Integer.parseInt(dateFlds[0])-1,
                //day
                Integer.parseInt(dateFlds[1]),
                //hour
                Integer.parseInt(timeFlds[0]),
                //minute
                Integer.parseInt(timeFlds[1]),
                //second
                0);
            }
            catch (Exception e)
            {
                if (timeFlds[0].equals("24") && timeFlds[1].equals("00"))
                {
                    timeFlds[0] = "00";

                    return new GregorianCalendar(
                            //year
                            Integer.parseInt(dateFlds[2]),
                            //month
                            Integer.parseInt(dateFlds[0])-1,
                            //day
                            Integer.parseInt(dateFlds[1]),
                            //hour
                            Integer.parseInt(timeFlds[0]),
                            //minute
                            Integer.parseInt(timeFlds[1]),
                            //second
                            0);
                }
                throw new RPCException(e);
            }
        }
    }

    /// <summary>
    /// Pad a month or day string with zeroes
    /// </summary>
    /// <param name="part">Month or day as string (e.g. "03" or "2" or "28" etc.)</param>
    /// <returns>Padded string using zero. Always returns a string of length 2</returns>
    private static String padDatePartWithZeroes(String part)
    {
        if (part == null || part.equals(""))
        {
            return "00";
        }
        part = part.trim();
        if (part.length() == 1)
        {
            return "0" + part;
        }
        return part;
    }
    public static String toUtcFromRdv(String rdvTS)
    {
        if (rdvTS == null || rdvTS.equals(""))
        {
            return "";
        }
        String[] parts = RPCStringUtils.split(rdvTS, RPCStringUtils.SPACE);
        String[] dateFlds = RPCStringUtils.split(parts[0], RPCStringUtils.SLASH);

        String datePartyyyyMMdd = dateFlds[2] + padDatePartWithZeroes(dateFlds[0]) + padDatePartWithZeroes(dateFlds[1]);
        if (parts.length == 1)
        {
            return datePartyyyyMMdd + ".000000";
        }
        else
        {
            String[] timeFlds = RPCStringUtils.split(parts[1], RPCStringUtils.COLON);
            return datePartyyyyMMdd + "." + timeFlds[0] + timeFlds[1] + "00";
            //return new DateTime(Convert.ToInt16(dateFlds[2]),
            //                    Convert.ToInt16(dateFlds[0]),
            //                    Convert.ToInt16(dateFlds[1]),
            //                    Convert.ToInt16(timeFlds[0]),
            //                    Convert.ToInt16(timeFlds[1]),
            //                    0);
        }
    }

    public static Calendar toDateTimeFromTextTS(String textTS)
    {
        return new GregorianCalendar(
                //year
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 0, 4)),
                //month
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 4, 2))-1,
                //day
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 6, 2)),
                //hour
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 9, 2)),
                //minute
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 11, 2)),
                //second
                Integer.parseInt(RPCStringUtils.CSubstring(textTS, 13, 2)));
    }

    public static Calendar toDateTime(String vistaTS)
    {
        if (vistaTS.indexOf("SENSITIVE") != -1)
        {
                return Calendar.getInstance();
        }
        if (vistaTS.equals("TODAY") || vistaTS.equals("NOW"))
        {
                return Calendar.getInstance();
        }
        if (vistaTS.equals(""))
        {
            return Calendar.getInstance();
        }
        String correctedTS = corrected(vistaTS);
              
        return new GregorianCalendar(
                getYear(correctedTS),
                getMonth(correctedTS)-1,
                getDay(correctedTS),
                getHour(correctedTS),
                getMinute(correctedTS),
                getSecond(correctedTS));
    }

    public static String fromUtcString(String ts)
    {
        //ts = corrected(ts);
        //ts = ts.Substring(0,ts.Length-3);
        int yr = Integer.parseInt(RPCStringUtils.CSubstring(ts, 0,4));
        String result = "" + (yr - 1700);
        result += ts.substring(4);
        return result;
    }

    /// <summary>Convert DateTime into VistA format
    /// </summary>
    /// <param name="t"></param>
    /// <returns>yyyMMdd.HHmmss with year - 1700</returns>
    public static String fromDateTime(Calendar t)
    {
        String sYear = "" + (t.get(Calendar.YEAR) - 1700);
        String sMonth = RPCStringUtils.prependChars("" + (t.get(Calendar.MONTH)+1), '0', 2);
        String sDay = RPCStringUtils.prependChars("" + t.get(Calendar.DAY_OF_MONTH), '0', 2);
        String result = sYear + sMonth + sDay;
        result += '.';
        String sHour = RPCStringUtils.prependChars("" + t.get(Calendar.HOUR), '0', 2);
        String sMins = RPCStringUtils.prependChars("" + t.get(Calendar.MINUTE), '0', 2);
        String sSecs = RPCStringUtils.prependChars("" + t.get(Calendar.SECOND), '0', 2);
        result += sHour + sMins + sSecs;
        return result;
    }

    public static String fromDateTimeToTextTS(Calendar t)
    {
        String sYear = "" + t.get(Calendar.YEAR);
        String sMonth = RPCStringUtils.prependChars("" + (t.get(Calendar.MONTH)+1), '0', 2);
        String sDay = RPCStringUtils.prependChars("" + t.get(Calendar.DAY_OF_MONTH), '0', 2);
        String result = sYear + sMonth + sDay;
        result += '.';
        String sHour = RPCStringUtils.prependChars("" + t.get(Calendar.HOUR), '0', 2);
        String sMins = RPCStringUtils.prependChars("" + t.get(Calendar.MINUTE), '0', 2);
        String sSecs = RPCStringUtils.prependChars("" + t.get(Calendar.SECOND), '0', 2);
        result += sHour + sMins + sSecs;
        return result;
    }

    public static String noSeconds(String vistaTS)
    {
        if (vistaTS.indexOf('.') == -1)
        {
            return vistaTS;
        }
        vistaTS = padTimeToSeconds(vistaTS);
        String[] flds = RPCStringUtils.split(vistaTS, RPCStringUtils.PERIOD);
        return flds[0] + '.' + RPCStringUtils.CSubstring(flds[1], 0, 4);
    }

    public static String padTimeToSeconds(String vistaTS)
    {
        String result = vistaTS;
        while (result.length() < 12)
        {
                result += '0';
        }
        return result;
    }

    public static String fromUtcFromDate(String utcFrom)
    {
        if (utcFrom.indexOf('.') == -1)
        {
            utcFrom += ".000000";
        }
        return fromUtcString(utcFrom);
    }

    public static String fromUtcToDate(String utcTo)
    {
        if (utcTo.indexOf('.') == -1)
        {
            utcTo += ".235959";
        }
        return fromUtcString(utcTo);
    }

    public static String invertTimestamp(String ts)
    {
        Double dTs = Double.parseDouble(ts);
        Double newTs = 9999999 - dTs;
        return "" + newTs;
    }

    public static String trimZeroes(String ts)
    {
        int idx = ts.length() - 1;
        while (ts.charAt(idx) == '0')
        {
            idx--;
        }
        return RPCStringUtils.CSubstring(ts, 0, idx + 1);
    }

    public static String fromText(String textTS)
    {
        if (textTS == null || textTS.equals(""))
        {
            throw new IllegalArgumentException();
        }
        boolean fHasTime = true;
        int validLength = 18;
        if (textTS.indexOf('@') == -1)
        {
            fHasTime = false;
            validLength = 12;
        }
        if (textTS.length() != validLength)
        {
            throw new IllegalArgumentException("Invalid text date");
        }
        String mo = txtMonthToNum(RPCStringUtils.CSubstring(textTS, 0, 3));
        String dy = RPCStringUtils.CSubstring(textTS, 4, 2);
        String yr = RPCStringUtils.CSubstring(textTS, 8, 4);
        if (!DateUtils.isValidDay(Integer.parseInt(yr), Integer.parseInt(mo), Integer.parseInt(dy)))
        {
            throw new IllegalArgumentException("Invalid text date");
        }
        String hr = "00";
        String mn = "00";
        if (fHasTime)
        {
            hr = RPCStringUtils.CSubstring(textTS, 13, 2);
            mn = RPCStringUtils.CSubstring(textTS, 16, 2);
        }
        return yr + mo + dy + '.' + hr + mn + "00";
    }

    public static String txtMonthToNum(String sMonth)
    {
        sMonth = sMonth.toUpperCase();
        if (sMonth.equals("JAN"))
        {
            return "01";
        }
        else if (sMonth.equals("FEB"))
        {
            return "02";
        }
        else if (sMonth.equals("MAR"))
        {
            return "03";
        }
        else if (sMonth.equals("APR"))
        {
            return "04";
        }
        else if (sMonth.equals("MAY"))
        {
            return "05";
        }
        else if (sMonth.equals("JUN"))
        {
            return "06";
        }
        else if (sMonth.equals("JUL"))
        {
            return "07";
        }
        else if (sMonth.equals("AUG"))
        {
            return "08";
        }
        else if (sMonth.equals("SEP"))
        {
            return "09";
        }
        else if (sMonth.equals("OCT"))
        {
            return "10";
        }
        else if (sMonth.equals("NOV"))
        {
            return "11";
        }
        else if (sMonth.equals("DEC"))
        {
            return "12";
        }
        else
        {
            throw new IllegalArgumentException("Invalid month");
        }
    }
    

    public static boolean isWellFormedTimePart(String dt)
    {
        int p = dt.indexOf('.');
        if (p == -1)
        {
            return true;
        }
        if (p != 8)
        {
            return false;
        }
        String t = dt.substring(9);
        // Valid Vista dates do NOT need to have 6 time digits: 3110101.10 is a valid timestamp, for example
        //if (t.Length != 6)
        //{
        //    return false;
        //}
        //if (!StringUtils.isNumeric(t))
        if (!t.matches("\\d+"))
        {
            return false;
        }
        if (t.length() >= 2)
        {
          //  int hr = Convert.ToInt16(t.substring(0, 0+2));
            int hr = Integer.parseInt(t.substring(0, 0+2));
            if (hr > 23)
            {
                return false;
            }
        }
        if (t.length() >= 4)
        {
           // int mn = Convert.ToInt16(t.substring(2, 2+2));
            int mn = Integer.parseInt(t.substring(2, 2+2));
            if (mn > 59)
            {
                return false;
            }
        }
        if (t.length() >= 6)
        {
           // int sc = Convert.ToInt16(t.substring(4));
            int sc = Integer.parseInt(t.substring(4));
            if (sc > 59)
            {
                return false;
            }
        }
        return true;
    }
    
    public static boolean isWellFormedDatePart(String dt)
    {
        if (dt == null || dt.isEmpty())
        {
            return false;
        }
        String d = "";
        int p = dt.indexOf('.');
        if (p == -1)
        {
            d = dt;
        }
        else if (p != 8)
        {
            return false;
        }
        else
        {
            d = dt.substring(0, 0+8);
        }
        if (d.length() != 8)
        {
            return false;
        }
        //if (!StringUtils.isNumeric(d))
        if (!d.matches("\\d+"))
        {
            return false;
        }
     //   int yr = Convert.ToInt16(d.Substring(0, 0+4));
        int yr = Integer.parseInt(d.substring(0, 4));
        if (yr < 1890)
        {
            return false;
        }
      //  int mo = Convert.ToInt16(d.Substring(4, 4+2));
        int mo = Integer.parseInt(d.substring(4, 4+2));
        if (mo > 12)
        {
            return false;
        }
        //int dy = Convert.ToInt16(d.Substring(6));
        int dy = Integer.parseInt(d.substring(6));
        if (dy > 31)
        {
            return false;
        }
        if ((mo == 4 || mo == 6 || mo == 9 || mo == 11) && dy > 30)
        {
            return false;
        }
        if (mo == 2 && dy > 29)
        {
            return false;
        }
        if (mo == 2 && !isLeapYear(yr) && dy > 28)
        {
            return false;
        }
        return true;
    }

    public static boolean isLeapYear(int year)
    {
        return ((year > 0) && (year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0)));
    }

    public static boolean isWellFormedUtcDateTime(String dt)
    {
        if (isWellFormedDatePart(dt) && isWellFormedTimePart(dt))
        {
            return true;
        }
        return false;
    }

}
