package gov.va.med.via.rpc.client;

import gov.va.med.via.rpc.RPCConnection;
import gov.va.med.via.rpc.RPCException;
import gov.va.med.via.rpc.RPCStringUtils;
import gov.va.med.via.rpc.VistaQuery;
import gov.va.med.via.rpc.VistaResponse;
import gov.va.med.via.rpc.site.DataSource;
import gov.va.med.via.rpc.site.SiteException;

/**
 * RPCClient is a helper class which is designed to ease the use of talking
 * RPC's with a VistA system.
 *
 * An RPC client does not need to authenticate in order to perform certain
 * queries; ie RPC "XUS INTRO MSG". Other RPCs which write data to the VistA
 * system require the RPC Client to login.
 *
 * By default, when the RPC client executes a single query using the
 * executeQuery method, the RPC client will perform the query, get the result
 * and immediately disconnect.
 *
 * To perform a query in sequence while keeping the state of the underlying
 * M stack alive, use the executeMultiQuery method. In contrast to executeQuery,
 * executeMultiQuery will keep the RPC connection open UNTIL TOLD to logout.
 *
 * Example usage can be found in the JUnit tests.
 *
 * @author vhahonuyehac
 */
public class RPCClient
{

    private RPCConnection rpcConnection = null;
  

	private String encryptedAvCode = "";
    private String brokerContext = null;
    private boolean isAuthenticated = false;
    private boolean isLoggedIn = false;
    
    public final static String CAPRI_CONTEXT = "DVBA CAPRI GUI";
    public final static String CPRS_CONTEXT = "OR CPRS GUI CHART";
    public final static String BCMA_CONTEXT = "PSB GUI CONTEXT - USER";
    public final static String VISTA_IMAGING_CONTEXT = "MAG WINDOWS";
	private String duz;


	private String greeting;


	/**
     * Instantiate an RPC client without credentials. Used for querying a
     * VistA system without providing credentials. This assumes the user knows
     * whether or not the RPC requires login in order to execute successfully.
     * This overloaded method does not set the DUZ attribute.
     * @param siteCode typically a three digit site # as found in vhasites.xml
     * @throws RPCException occurs when the given site code cannot be found
     */
    public RPCClient(String siteCode)
            throws RPCException
    {
        try
        {
            rpcConnection = new RPCConnection(siteCode);
        }
        catch(SiteException e)
        {
            throw new RPCException("Invalid siteCode passed to the RPCClient.");
        }
    }

    /**
     * Instantiates an RPC client using the given credentials. Used for reading
     * and writing data via a VistA system's RPCs. This overloaded method sets the DUZ attribute.
     * @param encryptedAvCode encrypted form of the access and verify code
     * @param brokerContext the context to login as; such as
     * RPCClient.CPRS_CONTEXT
     * @param siteCode typically a three digit site # as found in vhasites.xml
     * @throws RPCException occurs when the given site code cannot be found
     * or when the rpc client fails to validate the given access / verify
     * code pair.
     */
    public RPCClient(String encryptedAvCode, String brokerContext,
            String siteCode) throws RPCException
    {
        if(encryptedAvCode == null || siteCode == null || brokerContext == null)
        {
            throw new IllegalArgumentException("RPCClient - constructor "
                    + "arguments cannot be null.");
        }

        try
        {
            this.rpcConnection = new RPCConnection(siteCode);
        }
        catch(SiteException e)
        {
            throw new RPCException("Invalid siteCode passed to the RPCClient.");
        }

        connect();
        login(encryptedAvCode, brokerContext);
        disconnect();
    }

    /**
     * Instantiates a RPC client using the given credentials. Used for reading
     * and writing data via a VistA system's RPCs.  This overloaded method sets the DUZ attribute.
     * @param accessCode unencrypted form of the access code
     * @param verifyCode unencrypted form of the verify code
     * @param brokerContext the context to login as; such as
     * RPCClient.CPRS_CONTEXT
     * @param siteCode typically a three digit site # as found in vhasites.xml
     * @throws RPCException occurs when the rpc client fails to validate the
     * given access and verify code pair or when the site code doesn't exist
     * or the broker context is invalid.
     */
    public RPCClient(String accessCode, String verifyCode, String brokerContext, String siteCode, boolean disconnectAfterConstruction) throws RPCException
    {
        if(accessCode == null || verifyCode == null ||
                siteCode == null || brokerContext == null)
        {
            throw new IllegalArgumentException("RPCClient - constructor arguments cannot be null.");
        }

        try
        {
            this.rpcConnection = new RPCConnection(siteCode);
        }
        catch(SiteException e)
        {
            throw new RPCException("Invalid siteCode passed to the RPCClient.");
        }

        connect();
        login(accessCode, verifyCode, brokerContext);
        if ( disconnectAfterConstruction) disconnect();
    }

    
    
    /**
     * Instantiates a RPC client using the given credentials. Used for reading
     * and writing data via a VistA system's RPCs.  This overloaded method sets the DUZ attribute.
     * @param accessCode unencrypted form of the access code
     * @param verifyCode unencrypted form of the verify code
     * @param brokerContext the context to login as; such as
     * RPCClient.CPRS_CONTEXT
     * @param siteCode typically a three digit site # as found in vhasites.xml
     * @throws RPCException occurs when the rpc client fails to validate the
     * given access and verify code pair or when the site code doesn't exist
     * or the broker context is invalid.
     */
    public RPCClient(String accessCode, String verifyCode, String brokerContext,
            String siteCode) throws RPCException
    {
    	this(accessCode,verifyCode,brokerContext,siteCode,true);
    }
    
	/** This overloaded method does not set the DUZ attribute required by the VIA application.
	 * 
	 * @param dataSource
	 * @throws RPCException
	 */
    public RPCClient(DataSource dataSource) throws RPCException
    {
        this.rpcConnection = new RPCConnection(dataSource);
    }

    private void connect() throws RPCException
    {
        rpcConnection.connect();
    }

    public void disconnect() throws RPCException
    {
        isLoggedIn = false;
        rpcConnection.disconnect();
    }

    /**
     * Login using the supplied access and verify code and broker context.
     * @param accessCode unencrypted form of the access code
     * @param verifyCode unencrypted form of the verify code
     * @param brokerContext xwb broker context, ie - RPCClient.CPRS_CONTEXT
     * @throws RPCException occurs when the user is already logged in or
     * the supplied access and verify code is wrong.
     */
    public void login(String accessCode, String verifyCode, String brokerContext)
            throws RPCException
    {
        String encryptedAvCode = VistaQuery.encrypt(accessCode + ";" + verifyCode);
        login(encryptedAvCode, brokerContext);
    }

    /**
     * Login using the supplied ENCRYPTED access and verify code. By DEFAULT
     * this method will login using the CPRS context.
     * @param encryptedAvCode encrypted form of the access code + verify code
     * @throws RPCException when the given access and verify code are incorrect.
     */
    public void login(String encryptedAvCode) throws RPCException
    {
        login(encryptedAvCode, RPCClient.CPRS_CONTEXT);
    }

    /**
     * Login using the supplied ENCRYPTED access and verify code. The client
     * will login using the supplied broker context.
     * @param encryptedAvCode encrypted form of the access code + verify code
     * @param brokerContext the xwb context to login as after authenticating
     * @throws RPCException when the given access and verify code are incorrect
     * or the broker context is invalid
     */
    public void login(String encryptedAvCode, String brokerContext)
            throws RPCException
    {
        if(isLoggedIn())
            throw new RPCException("RPCClient is already logged in.");
        
        try
        {
            /* Record that the user is logging into the system. */
            isLoggedIn = true;
            authenticate(encryptedAvCode, brokerContext);
        }
        catch(RPCException e)
        {
            /* Authentication failed: record that the user is NOT logged in. */
            isLoggedIn = false;
            throw new RPCException(e);
        }
    }

    /**
     * Called when an RPC client has completed its sequence of queries.
     * @throws RPCException
     */
    public void logout() throws RPCException
    {
        disconnect();
    }

    public String getRPCServer()
    {
        String rpcServer = rpcConnection.getDataSource().getProvider() + ":" +
                rpcConnection.getDataSource().getPort();
        return rpcServer;
    }
    /**
     * authenticates a user at the specific site using the encrypted av code.
     * 
     * @param encryptedAvCode
     * @param brokerContext
     * @return the user's DUZ or VPID
     * @throws RPCException
     */
    private String authenticate(String encryptedAvCode, String brokerContext)
            throws RPCException
    {        
        VistaResponse vr = null;
        
        VistaQuery vq = new VistaQuery("XUS SIGNON SETUP");
        executeMultiQuery(vq);

        //XUS AV CODE
//        DESCRIPTION:
//        	  1>This API checks if a ACCESS/VERIFY code pair is valid.
//        	  2>It returns an array of values
//        	  3>
//        	  4>R(0)=DUZ if sign-on was OK, zero if not OK.
//        	  5>R(1)=(0=OK, 1,2...=Can't sign-on for some reason).
//        	  6>R(2)=verify needs changing.
//        	  7>R(3)=Message.
//        	  8>R(4)=0
//        	  9>R(5)=count of the number of lines of text, zero if none.
//        	 10>R(5+n)=message text.
        
        
        vq = new VistaQuery("XUS AV CODE");
        vq.addParameter(vq.LITERAL, encryptedAvCode);
        vr = executeMultiQuery(vq);
        
        /* If the 7th piece is null then authentication has failed. */
        if(vr.getPiece(RPCStringUtils.CRLF, 7) == null)
        {
            encryptedAvCode = "";
            throw new RPCException(vr.getPiece(RPCStringUtils.CRLF, 4));
        }

        duz = vr.getPiece(RPCStringUtils.CRLF, 1);
        /// perform checks to ensure that the DUZ is set (later PID)
        if ( duz==null || duz.trim().equals(""))
        {
        	throw new RPCException("Provider DUZ cannot be an empty string");        	
        }
        else if ( duz.trim().equals("0")  )
        {
        	throw new RPCException("sign-in returned '0' value for DUZ; signin was not OK");
        }
        try
        {
        	setGreeting(vr.getPiece(RPCStringUtils.CRLF,8));
        }
        catch ( Exception e)
        {
        	setGreeting("");
        }
        
        vq = new VistaQuery("XWB CREATE CONTEXT");
        vq.addEncryptedParameter(vq.LITERAL, brokerContext);
        vr = executeMultiQuery(vq);

        /* If the RPC doesn't return 1 then creating context has failed. */
        if(!"1".equals(vr.getRawResponse()))
        {
            throw new RPCException("Failed to create XWB context: " + brokerContext);
        }

        /* If the context change is successful then inform the rpc connection. */
        rpcConnection.setContext(brokerContext);
        this.brokerContext = brokerContext;

        /* The client has successfully authenticated. */
        this.isAuthenticated = true;

        /* Store the encrypted access/verify code. */
        this.encryptedAvCode = encryptedAvCode;
        
        
        return duz;
        
    }

    private synchronized VistaResponse query(VistaQuery vq, boolean singleQuery)
            throws RPCException
    {
        if(!isConnected())
        {
            connect();
        }

        if(isAuthenticated() && !isLoggedIn())
        {
            login(encryptedAvCode, brokerContext);
        }

        VistaResponse vr = rpcConnection.query(vq);

        if(isConnected() && singleQuery)
        {
            disconnect();
        }

        return vr;
    }

    /**
     * Executes a RPC query, immediately disconnects and returns the result
     * @param vistaQuery the RPC to execute
     * @return a VistaResponse object which encapsulates the RPC response
     * @throws RPCException when the query fails to execute
     */
    public VistaResponse executeQuery(VistaQuery vistaQuery) throws RPCException
    {
        return query(vistaQuery, true);
    }

    /**
     * Executes a RPC query, KEEPS the RPC client connection open and returns
     * the results. This method enables a programmer to retain the M stack
     * while issuing RPCs in a procedural fashion.
     * @param vistaQuery the RPC to execute
     * @return a VistaResponse object which encapsulates the RPC response
     * @throws RPCException when the query fails to execute
     */
    public VistaResponse executeMultiQuery(VistaQuery vistaQuery)
            throws RPCException
    {
        return query(vistaQuery, false);
    }

    /**
     * Returns the status of the RPC client at the socket level.
     * @return true if the rpc client is still connected otherwise false.
     */
    public boolean isConnected()
    {
        return rpcConnection.isConnected();
    }

    /**
     * Returns whether or not the rpc client has provided access and verify
     * codes which have been validated.
     * @return true if the rpc client has validated the given access and verify
     * code; otherwise false.
     */
    public boolean isAuthenticated()
    {
        return isAuthenticated;
    }

    /**
     * Returns whether or not the current rpc connection has logged in using
     * the supplied access and verify codes. Note: an rpc connection does not
     * have to authenticated in order for queries to execute.
     * @return true if the rpc client is currently connected and the connection
     * has logged in using the provided access and verify codes.
     */
    public boolean isLoggedIn()
    {
        return isLoggedIn;
    }

    /**
     * Performs an RPC select of the patient. RPC client must be authenticated
     * before calling this method.
     * @param patientIen the vista patient ien
     * @throws RPCException when the patient is not found in vista.
     */
    public void selectPatient(String patientIen) throws RPCException
    {
        if(!isAuthenticated())
        {
            throw new IllegalStateException("RPC client requires "
                    + "authentication before calling this method.");
        }

        VistaQuery vq = new VistaQuery("ORWPT SELECT");
        vq.addParameter(vq.LITERAL, patientIen);

        VistaResponse vr = executeQuery(vq);
        if(vr.getPiece(1).equalsIgnoreCase("-1"))
        {
            throw new RPCException("Patient not found in VistA: " + patientIen);
        }
        if(!vr.getRawResponse().startsWith("ZZTEST"))
        {
            throw new RPCException("Write back is not permitted to patient: " 
                    + vr.getPiece(1));
        }
    }

    /** returns the authenticated user's DUZ or null if not set.  The DUZ value is parsed in the private authenticate method.  
     *  This class was originally part of JMeadows and then inherited by the VIA application.  It was determined that the 
     *  XUS AV CODE RPC could not be executed by VistALink since it interferes with the VistALink Security mechanism.  Thus,
     *  this "raw socket" RPC approach is used to login the user, provide a DUZ, and enable follow up VistALink calls 
     *  with a DUZ.
     * 
     * @return the authenticated user's DUZ, null if not set.
     */
    public String getDuz() {
		return duz;
	}

    
    public RPCConnection getRpcConnection() {
  		return rpcConnection;
  	}

	public String getGreeting() {
		return greeting;
	}

	public void setGreeting(String greeting) {
		this.greeting = greeting;
	}
}
