 
package gov.va.med.via.rpc.site;

import gov.va.med.via.rpc.KeyValuePair;
import gov.va.med.via.rpc.RPCStringUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.InputStream;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SiteTable
{
    private SortedMap<Integer,Region> regions;
    private SortedMap<String,Site> sites;
    private List<DataSource> sources;
    
    private static SiteTable instance;
    
    public static SiteTable getInstance()
    {
    	if ( instance==null )
    	{
    		try 
    		{
				instance = new SiteTable("VhaSites.xml");
			} 
    		catch (SiteException e) 
			{
				throw new RuntimeException("Unable to load site table", e);
			}
    	}
    	return instance;    	
    }
    
    private SiteTable(String filepath) throws SiteException
    {
        regions = new TreeMap<Integer,Region>();
        sites = new TreeMap<String,Site>();
        sources = new ArrayList<DataSource>();
        parse(filepath);
    }
    /**
     * gets a site's name 
     * @param siteId the site Id. Example: 631
     * @return the site's name. Example: Northampton, MA
     * @throw SiteException if the site is not found in the SiteTable
     */
    public String getSiteName( String siteId) throws SiteException
    {
    	Site site = getSite(siteId);
    	if ( site==null ) throw new SiteException("site for siteId " + siteId + " could not be found");
    	return site.getName();    	
    }
    /**
     * gets a site's display name
     * @param siteId the site's id. Example: 442
     * @return the site's display name. Example: Connecticut HCS (Newington, West Haven)
     * @throw SiteException if the site is not found in the SiteTable
     */
    public String getSiteDisplayName( String siteId) throws SiteException
    {
    	Site site = getSite(siteId);
    	if ( site==null ) throw new SiteException("site for siteId " + siteId + " could not be found");
    	return site.getDisplayName();
    }
    public String getSiteMoniker( String siteId) throws SiteException
    {
    	Site site = getSite(siteId);
    	if ( site==null ) throw new SiteException("site for siteId " + siteId + " could not be found");
    	return site.getMoniker();
    }
    

    public SortedMap<Integer,Region> getRegions() {
        return regions;
    }

    public SortedMap<String,Site> getSites() {
        return sites;
    }

    public List<DataSource> getSources() {
        return sources;
    }

    public Site getSite(String siteId)
    {
        return (Site)sites.get(siteId);
    }
    

    public Site[] getSites(String sitelist)
    {
        String[] siteCodes = RPCStringUtils.split(sitelist, RPCStringUtils.COMMA);
        siteCodes = RPCStringUtils.trimArray(siteCodes);
        if (siteCodes.length == 0)
        {
            return null;
        }
        Site[] _sites = new Site[siteCodes.length];
        
        for (int i = 0; i < siteCodes.length; i++)
        {
            _sites[i] = getSite(siteCodes[i]);
        }

        return _sites;
    }

    public Site[] getSites(SiteId[] siteIds)
    {
        if (siteIds == null || siteIds.length == 0)
        {
            return null;
        }
        Site[] _sites = new Site[siteIds.length];
        for (int i = 0; i < siteIds.length; i++)
        {
            _sites[i] = getSite(siteIds[i].getId());
        }
        return _sites;
    }

    public Region getRegion(String regionId)
    {
        int regId = Integer.parseInt(regionId);
        return (Region)regions.get(regId);
    }

    private void parse(String filepath) throws SiteException
    {
        try
        {
            if (RPCStringUtils.isNullOrEmpty(filepath))
            {
                throw new IllegalArgumentException("No source for site table");
            }
            Region currentRegion = null;
            ArrayList<Site> currentSites = null;
            Site currentSite = null;
            DataSource dataSource = null;
            ArrayList<DataSource> currentSources = null;
            InputStream inputstream = this.getClass().getClassLoader().getResourceAsStream(filepath);
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(inputstream);
            doc.getDocumentElement().normalize();
            NodeList nodes = doc.getDocumentElement().getChildNodes();
            for (int i = 0; i < nodes.getLength(); i++)
            {
                Node regionNode = nodes.item(i);
                //String name = regionNode.getNodeName();
                if (regionNode.getNodeName().equals("VhaVisn"))
                {
                    currentRegion = new Region();
                    currentRegion.setId(Integer.parseInt(tryNodeAttribute(regionNode, "ID")));
                    currentRegion.setName(tryNodeAttribute(regionNode, "name"));
                    currentSites = new ArrayList<Site>();
                    NodeList siteNodes = regionNode.getChildNodes();
                    for (int j = 0; j < siteNodes.getLength(); j++)
                    {
                        Node siteNode = siteNodes.item(j);
                        if (siteNode.getNodeName().equals("VhaSite"))
                        {
                            currentSite = parseSite(siteNode, currentRegion);
                            currentSources = new ArrayList<DataSource>();
                            currentSites.add(currentSite);
                            NodeList dataNodes = siteNode.getChildNodes();
                            for (int k = 0; k < dataNodes.getLength(); k++)
                            {
                                Node dataSourceNode = dataNodes.item(k);
                                if (dataSourceNode.getNodeName().equals("DataSource"))
                                {
                                    dataSource = parseDataSource(dataSourceNode, currentSite);
                                    currentSources.add(dataSource);
                                    sources.add(dataSource);
                                    break;
                                }
                            }
                            currentSite.setDataSources((DataSource[]) currentSources.toArray(new DataSource[0]));
                            sites.put(currentSite.getId(), currentSite);
                        }
                    }
                    currentRegion.setSites(currentSites);
                    regions.put(currentRegion.getId(), currentRegion);
                }
            }
        }
        catch (Exception ex)
        {
            
            throw new SiteException(ex);
        }
    }

    private Site parseSite(Node siteNode, Region siteRegion)
    {
        Site site = new Site();
        site.setId(tryNodeAttribute(siteNode, "ID"));
        site.setName(tryNodeAttribute(siteNode, "name"));
        String displayName = null;
        displayName = tryNodeAttribute(siteNode, "displayname");
        
        if (displayName == null)
        {
            displayName = site.getName();
        }
        site.setDisplayName(displayName);
        site.setMoniker(tryNodeAttribute(siteNode, "moniker"));
        site.setRegionId("" + siteRegion.getId());

        return site;
    }

    private DataSource parseDataSource(Node dataSourceNode, Site dataSourceSite)
    {
        DataSource dataSource = new DataSource();
        KeyValuePair<String, String> kvp =
            new KeyValuePair<String, String>(dataSourceSite.getId(), dataSourceSite.getName());
        dataSource.setSiteId(kvp);
        dataSource.setModality(tryNodeAttribute(dataSourceNode, "modality"));
        dataSource.setProtocol(tryNodeAttribute(dataSourceNode, "protocol"));
        dataSource.setProvider(tryNodeAttribute(dataSourceNode, "source"));
        dataSource.setStatus(tryNodeAttribute(dataSourceNode, "status"));
        String sTmp = tryNodeAttribute(dataSourceNode, "port");
        if (RPCStringUtils.isEmpty(sTmp))
        {
            /* Negative 1 denoting a port was not given */
            dataSource.setPort(-1);
        }
        else
        {
            dataSource.setPort(Integer.parseInt(sTmp));
        }
        sTmp = tryNodeAttribute(dataSourceNode, "provider");
        if (sTmp != null)
        {
            dataSource.setProvider(sTmp);
        }
        sTmp = tryNodeAttribute(dataSourceNode, "description");
        if (sTmp != null)
        {
            dataSource.setDescription(sTmp);
        }
        sTmp = tryNodeAttribute(dataSourceNode, "vendor");
        if (sTmp != null)
        {
            dataSource.setVendor(sTmp);
        }
        sTmp = tryNodeAttribute(dataSourceNode, "version");
        if (sTmp != null)
        {
            dataSource.setVersion(sTmp);
        }
        sTmp = tryNodeAttribute(dataSourceNode, "context");
        if (sTmp != null)
        {
            dataSource.setContext(sTmp);
        }

        return dataSource;
    }

    private String tryNodeAttribute(Node node, String attributeName)
    {
        String attrValue = null;
        try 
        {
            attrValue = node.getAttributes().getNamedItem(attributeName).getNodeValue();
        }
        catch(Exception e){}

        return attrValue;
    }
}
